diff -aurN u-boot-2016.05.orig/arch/arm/dts/armada-370-buffalo-ls421de.dts u-boot-2016.05-88fxxxx/arch/arm/dts/armada-370-buffalo-ls421de.dts
--- u-boot-2016.05.orig/arch/arm/dts/armada-370-buffalo-ls421de.dts	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/arch/arm/dts/armada-370-buffalo-ls421de.dts	2016-05-21 11:33:32.828226194 -0700
@@ -0,0 +1,399 @@
+/*
+ * Device Tree file for Buffalo LS421DE
+ *
+ * Copyright (C) 2014, 2016
+ * Steve Shih <shihsung@gmail.com>
+ *
+ * This file is licensed under the terms of the GNU General Public
+ * License version 2.  This program is licensed "as is" without any
+ * warranty of any kind, whether express or implied.
+ */
+
+/dts-v1/;
+
+#include <dt-bindings/input/input.h>
+#include <dt-bindings/gpio/gpio.h>
+#include "armada-370.dtsi"
+
+/ {
+	model = "Buffalo LS421DE";
+	compatible = "buffalo,ls400",
+		     "marvell,armada370", "marvell,armada-370-xp";
+
+	aliases {
+		spi0 = &spi0;
+	};
+
+	chosen {
+		stdout-path = "serial0:115200n8";
+	};
+
+	memory {
+		device_type = "memory";
+		reg = <0x00000000 0x20000000>; /* 512 MB */
+	};
+
+	soc {
+		ranges = <MBUS_ID(0xf0, 0x01) 0 0xf1000000 0x100000
+			  MBUS_ID(0x01, 0xe0) 0 0xfff00000 0x100000>;
+		u-boot,dm-pre-reloc;
+
+		internal-regs {
+			spi@10600 {
+				status = "okay";
+				pinctrl-0 = <&spi0_pins2>;
+				pinctrl-names = "default";
+				u-boot,dm-pre-reloc;
+
+				spi-flash@0 {
+					#address-cells = <1>;
+					#size-cells = <1>;
+					compatible = "jedec,spi-nor";
+					reg = <0>;
+					spi-max-frequency = <80000000>;
+					mode = <3>;
+
+					partition@0 {
+						reg = <0x0 0xf0000>;
+						label = "U-Boot";
+					};
+
+					partition@f0000 {
+						reg = <0xf0000 0x10000>;
+						label = "ENV";
+					};
+				};
+			};
+
+			i2c@11000 {
+				clock-frequency = <100000>;
+				status = "okay";
+
+				rs5c372a: rs5c372a@32 {
+					compatible = "ricoh,rs5c372a";
+					reg = <0x32>;
+				};
+			};
+
+			serial@12000 {
+				status = "okay";
+				u-boot,dm-pre-reloc;
+			};
+
+			usb@50000 {
+				status = "okay";
+			};
+
+			mdio {
+				/* Marvell 88E1518 */
+				phy0: ethernet-phy@0 {
+					reg = <0>;
+					marvell,reg-init = <0x3 0x10
+							    0xf000 0x091e>,
+							   <0x3 0x11
+							    0xfff0 0x0001>;
+				};
+			};
+
+			ethernet@74000 {
+				status = "okay";
+				phy = <&phy0>;
+				phy-mode = "rgmii-id";
+			};
+
+			sata@a0000 {
+				nr-ports = <2>;
+				status = "okay";
+			};
+
+			nand@d0000 {
+				status = "okay";
+				num-cs = <1>;
+				marvell,nand-keep-config;
+				marvell,nand-enable-arbiter;
+				nand-on-flash-bbt;
+				nand-ecc-strength = <4>;
+				nand-ecc-step-size = <512>;
+
+				partition@0 {
+					label = "UBIFS";
+					reg = <0x0 0x20000000>;
+				};
+			};
+		};
+
+		pcie-controller {
+			status = "okay";
+
+			/* Connected to NEC D720202 USB 3.0 xHCI controller */
+			pcie@1,0 {
+				/* Port 0, Lane 0 */
+				status = "okay";
+			};
+		};
+	};
+
+	gpio_fan {
+		compatible = "gpio-fan";
+		pinctrl-0 = <&fan_lock
+			     &fan_low
+			     &fan_high>;
+		pinctrl-names = "default";
+		gpios = <&gpio0 13 GPIO_ACTIVE_LOW
+			 &gpio0 14 GPIO_ACTIVE_LOW>;
+		gpio-fan,speed-map = <   0 3
+				      1500 2
+				      2000 1
+				      2400 0>;
+		alarm-gpios = <&gpio0 10 GPIO_ACTIVE_HIGH>;
+	};
+
+	gpio-keys {
+		compatible = "gpio-keys";
+		pinctrl-0 = <&usb_oc_n
+			     &hdd_hot0_n
+			     &hdd_hot1_n
+			     &power_sw_n
+			     &func_sw_n>;
+		pinctrl-names = "default";
+
+		usb-oc {
+			label = "USB Over Current Fault";
+			linux,code = <KEY_EXIT>;
+			gpios = <&gpio0 6 GPIO_ACTIVE_LOW>;
+		};
+
+		hdd0-hot {
+			label = "Hard Disk 1 Present";
+			linux,code = <KEY_PROG1>;
+			gpios = <&gpio0 11 GPIO_ACTIVE_LOW>;
+		};
+
+		hdd1-hot {
+			label = "Hard Disk 2 Present";
+			linux,code = <KEY_PROG2>;
+			gpios = <&gpio0 12 GPIO_ACTIVE_LOW>;
+		};
+
+		power-switch {
+			label = "Power Switch";
+			linux,code = <KEY_POWER>;
+			gpios = <&gpio0 15 GPIO_ACTIVE_HIGH>;
+			debounce-interval = <3000>;
+		};
+
+		func-button {
+			label = "Function Button";
+			linux,code = <KEY_EJECTCD>;
+			gpios = <&gpio0 16 GPIO_ACTIVE_LOW>;
+			debounce-interval = <3000>;
+		};
+	};
+
+	gpio-leds {
+		compatible = "gpio-leds";
+		pinctrl-0 = <&usb_on
+			     &func_red
+			     &hdd_pwr0
+			     &hdd_pwr1
+			     &power_led
+			     &err_led
+			     &info_led
+			     &func_led
+			     &hdd0_err_led
+			     &hdd1_err_led>;
+		pinctrl-names = "default";
+
+		usb-power {
+			label = "PowerUSB";
+			gpios = <&gpio0 5 GPIO_ACTIVE_HIGH>;
+			default-state = "on";
+		};
+
+		func-red {
+			label = "ls400:red:function";
+			gpios = <&gpio0 7 GPIO_ACTIVE_HIGH>;
+			default-state = "off";
+		};
+
+		hd0-power {
+			label = "PowerHD1";
+			gpios = <&gpio0 8 GPIO_ACTIVE_HIGH>;
+			default-state = "keep";
+		};
+
+		hd1-power {
+			label = "PowerHD2";
+			gpios = <&gpio0 9 GPIO_ACTIVE_HIGH>;
+			default-state = "keep";
+		};
+
+		power-led {
+			label = "ls400:white:power";
+			gpios = <&gpio1 22 GPIO_ACTIVE_HIGH>;
+			default-state = "on";
+		};
+
+		err-led {
+			label = "ls400:red:power";
+			gpios = <&gpio1 23 GPIO_ACTIVE_HIGH>;
+			default-state = "off";
+		};
+
+		info-led {
+			label = "ls400:orange:power";
+			gpios = <&gpio1 25 GPIO_ACTIVE_HIGH>;
+			default-state = "off";
+		};
+
+		func-led {
+			label = "ls400:white:function";
+			gpios = <&gpio1 27 GPIO_ACTIVE_HIGH>;
+			default-state = "off";
+		};
+
+		hdd0-err-led {
+			label = "ls400:red:hdd0";
+			gpios = <&gpio1 29 GPIO_ACTIVE_HIGH>;
+			default-state = "off";
+		};
+
+		hdd1-err-led {
+			label = "ls400:red:hdd1";
+			gpios = <&gpio1 30 GPIO_ACTIVE_HIGH>;
+			default-state = "off";
+		};
+	};
+
+	restart_poweroff {
+		compatible = "restart-poweroff";
+	};
+};
+
+&pinctrl {
+	/* out: default 1 */
+	usb_on: usb-on {
+		marvell,pins = "mpp5";
+		marvell,function = "gpo";
+	};
+
+	/* in */
+	usb_oc_n: usb-oc-n {
+		marvell,pins = "mpp6";
+		marvell,function = "gpio";
+	};
+
+	/* out: default 0 */
+	func_red: func-red {
+		marvell,pins = "mpp7";
+		marvell,function = "gpo";
+	};
+
+	/* out: default 1 */
+	hdd_pwr0: hdd-pwr0 {
+		marvell,pins = "mpp8";
+		marvell,function = "gpio";
+	};
+
+	/* out: default 1 */
+	hdd_pwr1: hdd-pwr1 {
+		marvell,pins = "mpp9";
+		marvell,function = "gpo";
+	};
+
+	/* in */
+	fan_lock: fan-lock {
+		marvell,pins = "mpp10";
+		marvell,function = "gpio";
+	};
+
+	/* in */
+	hdd_hot0_n: hdd-hot0-n {
+		marvell,pins = "mpp11";
+		marvell,function = "gpio";
+	};
+
+	/* in */
+	hdd_hot1_n: hdd-hot1-n {
+		marvell,pins = "mpp12";
+		marvell,function = "gpio";
+	};
+
+	/* out: default 0 */
+	fan_high: fan-high {
+		marvell,pins = "mpp13";
+		marvell,function = "gpio";
+	};
+
+	/* out: default 0 */
+	fan_low: fan-low {
+		marvell,pins = "mpp14";
+		marvell,function = "gpio";
+	};
+
+	/* in */
+	power_sw_n: power-sw-n {
+		marvell,pins = "mpp15";
+		marvell,function = "gpio";
+	};
+
+	/* in */
+	func_sw_n: func-sw-n {
+		marvell,pins = "mpp16";
+		marvell,function = "gpio";
+	};
+
+	/* in */
+	uart_en_n: uart-en-n {
+		marvell,pins = "mpp31";
+		marvell,function = "gpio";
+	};
+
+	/* in */
+	usb3_irq: usb3-irq {
+		marvell,pins = "mpp53";
+		marvell,function = "gpio";
+	};
+
+	/* out: default 1 */
+	power_led: power-led {
+		marvell,pins = "mpp54";
+		marvell,function = "gpo";
+	};
+
+	/* out: default 0 */
+	err_led: err-led {
+		marvell,pins = "mpp55";
+		marvell,function = "gpio";
+	};
+
+	/* out: default 0 */
+	info_led: info-led {
+		marvell,pins = "mpp57";
+		marvell,function = "gpio";
+	};
+
+	/* out: default 1 */
+	cpu_off_n: cpu-off-n {
+		marvell,pins = "mpp58";
+		marvell,function = "gpio";
+	};
+
+	/* out: default 0 */
+	func_led: func-led {
+		marvell,pins = "mpp59";
+		marvell,function = "gpo";
+	};
+
+	/* out: default 0 */
+	hdd0_err_led: hdd0-err-led {
+		marvell,pins = "mpp61";
+		marvell,function = "gpo";
+	};
+
+	/* out: default 0 */
+	hdd1_err_led: hdd1-err-led {
+		marvell,pins = "mpp62";
+		marvell,function = "gpio";
+	};
+};
diff -aurN u-boot-2016.05.orig/arch/arm/dts/armada-370-buffalo-ls441de.dts u-boot-2016.05-88fxxxx/arch/arm/dts/armada-370-buffalo-ls441de.dts
--- u-boot-2016.05.orig/arch/arm/dts/armada-370-buffalo-ls441de.dts	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/arch/arm/dts/armada-370-buffalo-ls441de.dts	2016-05-21 11:33:32.828226194 -0700
@@ -0,0 +1,453 @@
+/*
+ * Device Tree file for Buffalo LS441DE
+ *
+ * Copyright (C) 2014, 2016
+ * Steve Shih <shihsung@gmail.com>
+ *
+ * This file is licensed under the terms of the GNU General Public
+ * License version 2.  This program is licensed "as is" without any
+ * warranty of any kind, whether express or implied.
+ */
+
+/dts-v1/;
+
+#include <dt-bindings/input/input.h>
+#include <dt-bindings/gpio/gpio.h>
+#include "armada-370.dtsi"
+
+/ {
+	model = "Buffalo LS441DE";
+	compatible = "buffalo,ls400",
+		     "marvell,armada370", "marvell,armada-370-xp";
+
+	aliases {
+		spi0 = &spi0;
+	};
+
+	chosen {
+		stdout-path = "serial0:115200n8";
+	};
+
+	memory {
+		device_type = "memory";
+		reg = <0x00000000 0x20000000>; /* 512 MB */
+	};
+
+	soc {
+		ranges = <MBUS_ID(0xf0, 0x01) 0 0xf1000000 0x100000
+			  MBUS_ID(0x01, 0xe0) 0 0xfff00000 0x100000>;
+		u-boot,dm-pre-reloc;
+
+		internal-regs {
+			spi@10600 {
+				status = "okay";
+				pinctrl-0 = <&spi0_pins2>;
+				pinctrl-names = "default";
+				u-boot,dm-pre-reloc;
+
+				spi-flash@0 {
+					#address-cells = <1>;
+					#size-cells = <1>;
+					compatible = "jedec,spi-nor";
+					reg = <0>;
+					spi-max-frequency = <80000000>;
+					mode = <3>;
+
+					partition@0 {
+						reg = <0x0 0xf0000>;
+						label = "U-Boot";
+					};
+
+					partition@f0000 {
+						reg = <0xf0000 0x10000>;
+						label = "ENV";
+					};
+				};
+			};
+
+			i2c@11000 {
+				clock-frequency = <100000>;
+				status = "okay";
+
+				rs5c372a: rs5c372a@32 {
+					compatible = "ricoh,rs5c372a";
+					reg = <0x32>;
+				};
+			};
+
+			serial@12000 {
+				status = "okay";
+				u-boot,dm-pre-reloc;
+			};
+
+			usb@50000 {
+				status = "okay";
+			};
+
+			mdio {
+				/* Marvell 88E1518 */
+				phy0: ethernet-phy@0 {
+					reg = <0>;
+					marvell,reg-init = <0x3 0x10
+							    0xf000 0x081e>,
+							   <0x3 0x11
+							    0xfff0 0x0001>;
+				};
+			};
+
+			ethernet@74000 {
+				status = "okay";
+				phy = <&phy0>;
+				phy-mode = "rgmii-id";
+			};
+
+			nand@d0000 {
+				status = "okay";
+				num-cs = <1>;
+				marvell,nand-keep-config;
+				marvell,nand-enable-arbiter;
+				nand-on-flash-bbt;
+				nand-ecc-strength = <4>;
+				nand-ecc-step-size = <512>;
+
+				partition@0 {
+					label = "UBIFS";
+					reg = <0x0 0x20000000>;
+				};
+			};
+		};
+
+		pcie-controller {
+			status = "okay";
+
+			/* Connected to NEC D720202 USB 3.0 xHCI controller */
+			pcie@1,0 {
+				/* Port 0, Lane 0 */
+				status = "okay";
+			};
+
+			/* Connected to Marvell 88SE9215 AHCI controller */
+			pcie@2,0 {
+				/* Port 1, Lane 0 */
+				status = "okay";
+			};
+		};
+	};
+
+	gpio_fan {
+		compatible = "gpio-fan";
+		pinctrl-0 = <&fan_lock
+			     &fan_low
+			     &fan_high>;
+		pinctrl-names = "default";
+		gpios = <&gpio0 13 GPIO_ACTIVE_LOW
+			 &gpio0 14 GPIO_ACTIVE_LOW>;
+		gpio-fan,speed-map = <   0 3
+				      1200 2
+				      1800 1
+				      2050 0>;
+		alarm-gpios = <&gpio0 12 GPIO_ACTIVE_HIGH>;
+	};
+
+	gpio-keys {
+		compatible = "gpio-keys";
+		pinctrl-0 = <&usb_oc_n
+			     &hdd_hot0_n
+			     &hdd_hot1_n
+			     &power_sw_n
+			     &func_sw_n
+			     &hdd_hot2_n
+			     &hdd_hot3_n>;
+		pinctrl-names = "default";
+
+		usb-oc {
+			label = "USB Over Current Fault";
+			linux,code = <KEY_EXIT>;
+			gpios = <&gpio0 6 GPIO_ACTIVE_LOW>;
+		};
+
+		hdd0-hot {
+			label = "Hard Disk 1 Present";
+			linux,code = <KEY_PROG1>;
+			gpios = <&gpio0 10 GPIO_ACTIVE_LOW>;
+		};
+
+		hdd1-hot {
+			label = "Hard Disk 2 Present";
+			linux,code = <KEY_PROG2>;
+			gpios = <&gpio0 11 GPIO_ACTIVE_LOW>;
+		};
+
+		power-switch {
+			label = "Power Switch";
+			linux,code = <KEY_POWER>;
+			gpios = <&gpio0 15 GPIO_ACTIVE_LOW>;
+			debounce-interval = <3000>;
+		};
+
+		func-button {
+			label = "Function Button";
+			linux,code = <KEY_EJECTCD>;
+			gpios = <&gpio0 16 GPIO_ACTIVE_LOW>;
+			debounce-interval = <3000>;
+		};
+
+		hdd2-hot {
+			label = "Hard Disk 3 Present";
+			linux,code = <KEY_PROG3>;
+			gpios = <&gpio1 18 GPIO_ACTIVE_LOW>;
+		};
+
+		hdd3-hot {
+			label = "Hard Disk 4 Present";
+			linux,code = <KEY_PROG4>;
+			gpios = <&gpio1 19 GPIO_ACTIVE_LOW>;
+		};
+	};
+
+	gpio-leds {
+		compatible = "gpio-leds";
+		pinctrl-0 = <&power_led
+			     &hdd_pwr1
+			     &hdd_pwr0
+			     &func_red
+			     &hdd3_err_led
+			     &hdd_pwr3
+			     &hdd_pwr2
+			     &info_led
+			     &hdd2_err_led
+			     &func_led
+			     &err_led
+			     &hdd0_err_led
+			     &hdd1_err_led>;
+		pinctrl-names = "default";
+
+		power-led {
+			label = "ls440:white:power";
+			gpios = <&gpio0 5 GPIO_ACTIVE_HIGH>;
+			default-state = "on";
+		};
+
+		hd1-power {
+			label = "PowerHD2";
+			gpios = <&gpio0 7 GPIO_ACTIVE_HIGH>;
+			default-state = "keep";
+		};
+
+		hd0-power {
+			label = "PowerHD1";
+			gpios = <&gpio0 8 GPIO_ACTIVE_HIGH>;
+			default-state = "keep";
+		};
+
+		func-red {
+			label = "ls440:red:function";
+			gpios = <&gpio0 9 GPIO_ACTIVE_HIGH>;
+			default-state = "off";
+		};
+
+		hdd3-err-led {
+			label = "ls440:red:hdd3";
+			gpios = <&gpio1 15 GPIO_ACTIVE_HIGH>;
+			default-state = "off";
+		};
+
+		hd3-power {
+			label = "PowerHD4";
+			gpios = <&gpio1 16 GPIO_ACTIVE_HIGH>;
+			default-state = "keep";
+		};
+
+		hd2-power {
+			label = "PowerHD3";
+			gpios = <&gpio1 20 GPIO_ACTIVE_HIGH>;
+			default-state = "keep";
+		};
+
+		info-led {
+			label = "ls440:orange:power";
+			gpios = <&gpio1 22 GPIO_ACTIVE_HIGH>;
+			default-state = "off";
+		};
+
+		hdd2-err-led {
+			label = "ls440:red:hdd2";
+			gpios = <&gpio1 26 GPIO_ACTIVE_HIGH>;
+			default-state = "off";
+		};
+
+		func-led {
+			label = "ls440:white:function";
+			gpios = <&gpio1 27 GPIO_ACTIVE_HIGH>;
+			default-state = "off";
+		};
+
+		err-led {
+			label = "ls440:red:power";
+			gpios = <&gpio1 28 GPIO_ACTIVE_HIGH>;
+			default-state = "off";
+		};
+
+		hdd0-err-led {
+			label = "ls440:red:hdd0";
+			gpios = <&gpio1 29 GPIO_ACTIVE_HIGH>;
+			default-state = "off";
+		};
+
+		hdd1-err-led {
+			label = "ls440:red:hdd1";
+			gpios = <&gpio1 30 GPIO_ACTIVE_HIGH>;
+			default-state = "off";
+		};
+	};
+
+	restart_poweroff {
+		compatible = "restart-poweroff";
+	};
+};
+
+&pinctrl {
+	/* out: default 1 */
+	power_led: power-led {
+		marvell,pins = "mpp5";
+		marvell,function = "gpo";
+	};
+
+	/* ??? in */
+	usb_oc_n: usb-oc-n {
+		marvell,pins = "mpp6";
+		marvell,function = "gpio";
+	};
+
+	/* out: default 0 */
+	hdd_pwr1: hdd-pwr1 {
+		marvell,pins = "mpp7";
+		marvell,function = "gpo";
+	};
+
+	/* out: default 0 */
+	hdd_pwr0: hdd-pwr0 {
+		marvell,pins = "mpp8";
+		marvell,function = "gpio";
+	};
+
+	/* out: default 0 */
+	func_red: func-red {
+		marvell,pins = "mpp9";
+		marvell,function = "gpo";
+	};
+
+	/* in */
+	hdd_hot0_n: hdd-hot0-n {
+		marvell,pins = "mpp10";
+		marvell,function = "gpio";
+	};
+
+	/* in */
+	hdd_hot1_n: hdd-hot1-n {
+		marvell,pins = "mpp11";
+		marvell,function = "gpio";
+	};
+
+	/* ??? in */
+	fan_lock: fan-lock {
+		marvell,pins = "mpp12";
+		marvell,function = "gpio";
+	};
+
+	/* out: default 0 */
+	fan_high: fan-high {
+		marvell,pins = "mpp13";
+		marvell,function = "gpio";
+	};
+
+	/* out: default 0 */
+	fan_low: fan-low {
+		marvell,pins = "mpp14";
+		marvell,function = "gpio";
+	};
+
+	/* in */
+	power_sw_n: power-sw-n {
+		marvell,pins = "mpp15";
+		marvell,function = "gpio";
+	};
+
+	/* in */
+	func_sw_n: func-sw-n {
+		marvell,pins = "mpp16";
+		marvell,function = "gpio";
+	};
+
+	/* ??? in */
+	uart_en_n: uart-en-n {
+		marvell,pins = "mpp31";
+		marvell,function = "gpio";
+	};
+
+	/* out: default 0 */
+	hdd3_err_led: hdd3-err-led {
+		marvell,pins = "mpp47";
+		marvell,function = "gpo";
+	};
+
+	/* out: default 0 */
+	hdd_pwr3: hdd-pwr3 {
+		marvell,pins = "mpp49";
+		marvell,function = "gpio";
+	};
+
+	/* in */
+	hdd_hot2_n: hdd-hot2-n {
+		marvell,pins = "mpp50";
+		marvell,function = "gpio";
+	};
+
+	/* in */
+	hdd_hot3_n: hdd-hot3-n {
+		marvell,pins = "mpp51";
+		marvell,function = "gpio";
+	};
+
+	/* out: default 0 */
+	hdd_pwr2: hdd-pwr2 {
+		marvell,pins = "mpp52";
+		marvell,function = "gpio";
+	};
+
+	/* out: default 0 */
+	info_led: info-led {
+		marvell,pins = "mpp54";
+		marvell,function = "gpo";
+	};
+
+	/* out: default 0 */
+	hdd2_err_led: hdd2-err-led {
+		marvell,pins = "mpp58";
+		marvell,function = "gpio";
+	};
+
+	/* out: default 0 */
+	func_led: func-led {
+		marvell,pins = "mpp59";
+		marvell,function = "gpo";
+	};
+
+	/* out: default 0 */
+	err_led: err-led {
+		marvell,pins = "mpp60";
+		marvell,function = "gpio";
+	};
+
+	/* out: default 0 */
+	hdd0_err_led: hdd0-err-led {
+		marvell,pins = "mpp61";
+		marvell,function = "gpo";
+	};
+
+	/* out: default 0 */
+	hdd1_err_led: hdd1-err-led {
+		marvell,pins = "mpp62";
+		marvell,function = "gpio";
+	};
+};
diff -aurN u-boot-2016.05.orig/arch/arm/dts/armada-370.dtsi u-boot-2016.05-88fxxxx/arch/arm/dts/armada-370.dtsi
--- u-boot-2016.05.orig/arch/arm/dts/armada-370.dtsi	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/arch/arm/dts/armada-370.dtsi	2016-05-21 11:33:32.828226194 -0700
@@ -0,0 +1,449 @@
+/*
+ * Device Tree Include file for Marvell Armada 370 family SoC
+ *
+ * Copyright (C) 2012 Marvell
+ *
+ * Lior Amsalem <alior@marvell.com>
+ * Gregory CLEMENT <gregory.clement@free-electrons.com>
+ * Thomas Petazzoni <thomas.petazzoni@free-electrons.com>
+ *
+ * This file is dual-licensed: you can use it either under the terms
+ * of the GPL or the X11 license, at your option. Note that this dual
+ * licensing only applies to this file, and not this project as a
+ * whole.
+ *
+ *  a) This file is free software; you can redistribute it and/or
+ *     modify it under the terms of the GNU General Public License as
+ *     published by the Free Software Foundation; either version 2 of the
+ *     License, or (at your option) any later version.
+ *
+ *     This file is distributed in the hope that it will be useful
+ *     but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *     GNU General Public License for more details.
+ *
+ * Or, alternatively
+ *
+ *  b) Permission is hereby granted, free of charge, to any person
+ *     obtaining a copy of this software and associated documentation
+ *     files (the "Software"), to deal in the Software without
+ *     restriction, including without limitation the rights to use
+ *     copy, modify, merge, publish, distribute, sublicense, and/or
+ *     sell copies of the Software, and to permit persons to whom the
+ *     Software is furnished to do so, subject to the following
+ *     conditions:
+ *
+ *     The above copyright notice and this permission notice shall be
+ *     included in all copies or substantial portions of the Software.
+ *
+ *     THE SOFTWARE IS PROVIDED , WITHOUT WARRANTY OF ANY KIND
+ *     EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
+ *     OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ *     NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
+ *     HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY
+ *     WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ *     FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *     OTHER DEALINGS IN THE SOFTWARE.
+ *
+ * Contains definitions specific to the Armada 370 SoC that are not
+ * common to all Armada SoCs.
+ */
+
+#include "armada-370-xp.dtsi"
+/include/ "skeleton.dtsi"
+
+/ {
+	model = "Marvell Armada 370 family SoC";
+	compatible = "marvell,armada370", "marvell,armada-370-xp";
+
+	aliases {
+		gpio0 = &gpio0;
+		gpio1 = &gpio1;
+		gpio2 = &gpio2;
+	};
+
+	soc {
+		compatible = "marvell,armada370-mbus", "simple-bus";
+
+		bootrom {
+			compatible = "marvell,bootrom";
+			reg = <MBUS_ID(0x01, 0xe0) 0 0x100000>;
+		};
+
+		pcie-controller {
+			compatible = "marvell,armada-370-pcie";
+			status = "disabled";
+			device_type = "pci";
+
+			#address-cells = <3>;
+			#size-cells = <2>;
+
+			msi-parent = <&mpic>;
+			bus-range = <0x00 0xff>;
+
+			ranges =
+			       <0x82000000 0 0x40000 MBUS_ID(0xf0, 0x01) 0x40000 0 0x00002000
+				0x82000000 0 0x80000 MBUS_ID(0xf0, 0x01) 0x80000 0 0x00002000
+				0x82000000 0x1 0     MBUS_ID(0x04, 0xe8) 0       1 0 /* Port 0.0 MEM */
+				0x81000000 0x1 0     MBUS_ID(0x04, 0xe0) 0       1 0 /* Port 0.0 IO  */
+				0x82000000 0x2 0     MBUS_ID(0x08, 0xe8) 0       1 0 /* Port 1.0 MEM */
+				0x81000000 0x2 0     MBUS_ID(0x08, 0xe0) 0       1 0 /* Port 1.0 IO  */>;
+
+			pcie@1,0 {
+				device_type = "pci";
+				assigned-addresses = <0x82000800 0 0x40000 0 0x2000>;
+				reg = <0x0800 0 0 0 0>;
+				#address-cells = <3>;
+				#size-cells = <2>;
+				#interrupt-cells = <1>;
+                                ranges = <0x82000000 0 0 0x82000000 0x1 0 1 0
+                                          0x81000000 0 0 0x81000000 0x1 0 1 0>;
+				interrupt-map-mask = <0 0 0 0>;
+				interrupt-map = <0 0 0 0 &mpic 58>;
+				marvell,pcie-port = <0>;
+				marvell,pcie-lane = <0>;
+				clocks = <&gateclk 5>;
+				status = "disabled";
+			};
+
+			pcie@2,0 {
+				device_type = "pci";
+				assigned-addresses = <0x82002800 0 0x80000 0 0x2000>;
+				reg = <0x1000 0 0 0 0>;
+				#address-cells = <3>;
+				#size-cells = <2>;
+				#interrupt-cells = <1>;
+                                ranges = <0x82000000 0 0 0x82000000 0x2 0 1 0
+                                          0x81000000 0 0 0x81000000 0x2 0 1 0>;
+				interrupt-map-mask = <0 0 0 0>;
+				interrupt-map = <0 0 0 0 &mpic 62>;
+				marvell,pcie-port = <1>;
+				marvell,pcie-lane = <0>;
+				clocks = <&gateclk 9>;
+				status = "disabled";
+			};
+		};
+
+		internal-regs {
+			L2: l2-cache {
+				compatible = "marvell,aurora-outer-cache";
+				reg = <0x08000 0x1000>;
+				cache-id-part = <0x100>;
+				cache-level = <2>;
+				cache-unified;
+				wt-override;
+			};
+
+			/*
+			 * Default SPI pinctrl setting, can be overwritten on
+			 * board level if a different configuration is used.
+			 */
+			spi0: spi@10600 {
+				compatible = "marvell,armada-370-spi",
+						"marvell,orion-spi";
+				pinctrl-0 = <&spi0_pins1>;
+				pinctrl-names = "default";
+			};
+
+			spi1: spi@10680 {
+				compatible = "marvell,armada-370-spi",
+						"marvell,orion-spi";
+				pinctrl-0 = <&spi1_pins>;
+				pinctrl-names = "default";
+			};
+
+			i2c0: i2c@11000 {
+				reg = <0x11000 0x20>;
+			};
+
+			i2c1: i2c@11100 {
+				reg = <0x11100 0x20>;
+			};
+
+			gpio0: gpio@18100 {
+				compatible = "marvell,orion-gpio";
+				reg = <0x18100 0x40>;
+				ngpios = <32>;
+				gpio-controller;
+				#gpio-cells = <2>;
+				interrupt-controller;
+				#interrupt-cells = <2>;
+				interrupts = <82>, <83>, <84>, <85>;
+			};
+
+			gpio1: gpio@18140 {
+				compatible = "marvell,orion-gpio";
+				reg = <0x18140 0x40>;
+				ngpios = <32>;
+				gpio-controller;
+				#gpio-cells = <2>;
+				interrupt-controller;
+				#interrupt-cells = <2>;
+				interrupts = <87>, <88>, <89>, <90>;
+			};
+
+			gpio2: gpio@18180 {
+				compatible = "marvell,orion-gpio";
+				reg = <0x18180 0x40>;
+				ngpios = <2>;
+				gpio-controller;
+				#gpio-cells = <2>;
+				interrupt-controller;
+				#interrupt-cells = <2>;
+				interrupts = <91>;
+			};
+
+			/*
+			 * Default UART pinctrl setting without RTS/CTS, can
+			 * be overwritten on board level if a different
+			 * configuration is used.
+			 */
+			uart0: serial@12000 {
+				pinctrl-0 = <&uart0_pins>;
+				pinctrl-names = "default";
+			};
+
+			uart1: serial@12100 {
+				pinctrl-0 = <&uart1_pins>;
+				pinctrl-names = "default";
+			};
+
+			system-controller@18200 {
+				compatible = "marvell,armada-370-xp-system-controller";
+				reg = <0x18200 0x100>;
+			};
+
+			gateclk: clock-gating-control@18220 {
+				compatible = "marvell,armada-370-gating-clock";
+				reg = <0x18220 0x4>;
+				clocks = <&coreclk 0>;
+				#clock-cells = <1>;
+			};
+
+			coreclk: mvebu-sar@18230 {
+				compatible = "marvell,armada-370-core-clock";
+				reg = <0x18230 0x08>;
+				#clock-cells = <1>;
+			};
+
+			thermal@18300 {
+				compatible = "marvell,armada370-thermal";
+				reg = <0x18300 0x4
+					0x18304 0x4>;
+				status = "okay";
+			};
+
+			sscg@18330 {
+				reg = <0x18330 0x4>;
+			};
+
+			interrupt-controller@20a00 {
+				reg = <0x20a00 0x1d0>, <0x21870 0x58>;
+			};
+
+			timer@20300 {
+				compatible = "marvell,armada-370-timer";
+				clocks = <&coreclk 2>;
+			};
+
+			watchdog@20300 {
+				compatible = "marvell,armada-370-wdt";
+				clocks = <&coreclk 2>;
+			};
+
+			cpurst@20800 {
+				compatible = "marvell,armada-370-cpu-reset";
+				reg = <0x20800 0x8>;
+			};
+
+			cpu-config@21000 {
+				compatible = "marvell,armada-370-cpu-config";
+				reg = <0x21000 0x8>;
+			};
+
+			audio_controller: audio-controller@30000 {
+				#sound-dai-cells = <1>;
+				compatible = "marvell,armada370-audio";
+				reg = <0x30000 0x4000>;
+				interrupts = <93>;
+				clocks = <&gateclk 0>;
+				clock-names = "internal";
+				status = "disabled";
+			};
+
+			usb@50000 {
+				clocks = <&coreclk 0>;
+			};
+
+			usb@51000 {
+				clocks = <&coreclk 0>;
+			};
+
+			xor@60800 {
+				compatible = "marvell,orion-xor";
+				reg = <0x60800 0x100
+				       0x60A00 0x100>;
+				status = "okay";
+
+				xor00 {
+					interrupts = <51>;
+					dmacap,memcpy;
+					dmacap,xor;
+				};
+				xor01 {
+					interrupts = <52>;
+					dmacap,memcpy;
+					dmacap,xor;
+					dmacap,memset;
+				};
+			};
+
+			xor@60900 {
+				compatible = "marvell,orion-xor";
+				reg = <0x60900 0x100
+				       0x60b00 0x100>;
+				status = "okay";
+
+				xor10 {
+					interrupts = <94>;
+					dmacap,memcpy;
+					dmacap,xor;
+				};
+				xor11 {
+					interrupts = <95>;
+					dmacap,memcpy;
+					dmacap,xor;
+					dmacap,memset;
+				};
+			};
+
+			ethernet@70000 {
+				compatible = "marvell,armada-370-neta";
+			};
+
+			ethernet@74000 {
+				compatible = "marvell,armada-370-neta";
+			};
+
+			crypto@90000 {
+				compatible = "marvell,armada-370-crypto";
+				reg = <0x90000 0x10000>;
+				reg-names = "regs";
+				interrupts = <48>;
+				clocks = <&gateclk 23>;
+				clock-names = "cesa0";
+				marvell,crypto-srams = <&crypto_sram>;
+				marvell,crypto-sram-size = <0x7e0>;
+			};
+		};
+
+		crypto_sram: sa-sram {
+			compatible = "mmio-sram";
+			reg = <MBUS_ID(0x09, 0x01) 0 0x800>;
+			reg-names = "sram";
+			clocks = <&gateclk 23>;
+			#address-cells = <1>;
+			#size-cells = <1>;
+			ranges = <0 MBUS_ID(0x09, 0x01) 0 0x800>;
+
+			/*
+			 * The Armada 370 has an erratum preventing the use of
+			 * the standard workflow for CPU idle support (relying
+			 * on the BootROM code to enter/exit idle state).
+			 * Reserve some amount of the crypto SRAM to put the
+			 * cpuidle workaround.
+			 */
+			idle-sram@0 {
+				reg = <0x0 0x20>;
+			};
+		};
+	};
+};
+
+&pinctrl {
+	compatible = "marvell,mv88f6710-pinctrl";
+
+	spi0_pins1: spi0-pins1 {
+		marvell,pins = "mpp33", "mpp34",
+			       "mpp35", "mpp36";
+		marvell,function = "spi0";
+	};
+
+	spi0_pins2: spi0_pins2 {
+		marvell,pins = "mpp32", "mpp63",
+			       "mpp64", "mpp65";
+		marvell,function = "spi0";
+	};
+
+	spi1_pins: spi1-pins {
+		marvell,pins = "mpp49", "mpp50",
+			       "mpp51", "mpp52";
+		marvell,function = "spi1";
+	};
+
+	uart0_pins: uart0-pins {
+		marvell,pins = "mpp0", "mpp1";
+		marvell,function = "uart0";
+	};
+
+	uart1_pins: uart1-pins {
+		marvell,pins = "mpp41", "mpp42";
+		marvell,function = "uart1";
+	};
+
+	sdio_pins1: sdio-pins1 {
+		marvell,pins = "mpp9",  "mpp11", "mpp12",
+				"mpp13", "mpp14", "mpp15";
+		marvell,function = "sd0";
+	};
+
+	sdio_pins2: sdio-pins2 {
+		marvell,pins = "mpp47", "mpp48", "mpp49",
+				"mpp50", "mpp51", "mpp52";
+		marvell,function = "sd0";
+	};
+
+	sdio_pins3: sdio-pins3 {
+		marvell,pins = "mpp48", "mpp49", "mpp50",
+				"mpp51", "mpp52", "mpp53";
+		marvell,function = "sd0";
+	};
+
+	i2c0_pins: i2c0-pins {
+		marvell,pins = "mpp2", "mpp3";
+		marvell,function = "i2c0";
+	};
+
+	i2s_pins1: i2s-pins1 {
+		marvell,pins = "mpp5", "mpp6", "mpp7",
+			       "mpp8", "mpp9", "mpp10",
+			       "mpp12", "mpp13";
+		marvell,function = "audio";
+	};
+
+	i2s_pins2: i2s-pins2 {
+		marvell,pins = "mpp49", "mpp47", "mpp50",
+			       "mpp59", "mpp57", "mpp61",
+			       "mpp62", "mpp60", "mpp58";
+		marvell,function = "audio";
+	};
+
+	mdio_pins: mdio-pins {
+		marvell,pins = "mpp17", "mpp18";
+		marvell,function = "ge";
+	};
+
+	ge0_rgmii_pins: ge0-rgmii-pins {
+		marvell,pins = "mpp5", "mpp6", "mpp7", "mpp8",
+			       "mpp9", "mpp10", "mpp11", "mpp12",
+			       "mpp13", "mpp14", "mpp15", "mpp16";
+		marvell,function = "ge0";
+	};
+
+	ge1_rgmii_pins: ge1-rgmii-pins {
+		marvell,pins = "mpp19", "mpp20", "mpp21", "mpp22",
+			       "mpp23", "mpp24", "mpp25", "mpp26",
+			       "mpp27", "mpp28", "mpp29", "mpp30";
+		marvell,function = "ge1";
+	};
+};
diff -aurN u-boot-2016.05.orig/arch/arm/dts/Makefile u-boot-2016.05-88fxxxx/arch/arm/dts/Makefile
--- u-boot-2016.05.orig/arch/arm/dts/Makefile	2016-05-16 07:40:32.000000000 -0700
+++ u-boot-2016.05-88fxxxx/arch/arm/dts/Makefile	2016-05-21 11:33:32.832226195 -0700
@@ -51,6 +51,8 @@
 	tegra210-p2571.dtb
 
 dtb-$(CONFIG_ARCH_MVEBU) +=			\
+	armada-370-buffalo-ls421de.dtb		\
+	armada-370-buffalo-ls441de.dtb		\
 	armada-375-db.dtb			\
 	armada-388-clearfog.dtb			\
 	armada-388-gp.dtb			\
diff -aurN u-boot-2016.05.orig/arch/arm/include/asm/mach-types.h u-boot-2016.05-88fxxxx/arch/arm/include/asm/mach-types.h
--- u-boot-2016.05.orig/arch/arm/include/asm/mach-types.h	2016-05-16 07:40:32.000000000 -0700
+++ u-boot-2016.05-88fxxxx/arch/arm/include/asm/mach-types.h	2016-05-21 11:33:32.832226195 -0700
@@ -1100,9 +1100,11 @@
 #define MACH_TYPE_THALES_ADC           3492
 #define MACH_TYPE_UBISYS_P9D_EVP       3493
 #define MACH_TYPE_ATDGP318             3494
+#define MACH_TYPE_POGO_E02             3542
 #define MACH_TYPE_OMAP5_SEVM           3777
 #define MACH_TYPE_PICOSAM9G45          3838
 #define MACH_TYPE_ARMADILLO_800EVA     3863
+#define MACH_TYPE_POGOPLUGV4           3960
 #define MACH_TYPE_KZM9G                4140
 #define MACH_TYPE_COLIBRI_T30          4493
 #define MACH_TYPE_APALIS_T30           4513
diff -aurN u-boot-2016.05.orig/arch/arm/Kconfig u-boot-2016.05-88fxxxx/arch/arm/Kconfig
--- u-boot-2016.05.orig/arch/arm/Kconfig	2016-05-16 07:40:32.000000000 -0700
+++ u-boot-2016.05-88fxxxx/arch/arm/Kconfig	2016-05-21 11:33:32.840226195 -0700
@@ -115,7 +115,7 @@
 	select CPU_ARM926EJS
 
 config ARCH_MVEBU
-	bool "Marvell MVEBU family (Armada XP/375/38x)"
+	bool "Marvell MVEBU family (Armada XP/370/375/38x)"
 	select CPU_V7
 	select SUPPORT_SPL
 	select OF_CONTROL
diff -aurN u-boot-2016.05.orig/arch/arm/mach-kirkwood/cpu.c u-boot-2016.05-88fxxxx/arch/arm/mach-kirkwood/cpu.c
--- u-boot-2016.05.orig/arch/arm/mach-kirkwood/cpu.c	2016-05-16 07:40:32.000000000 -0700
+++ u-boot-2016.05-88fxxxx/arch/arm/mach-kirkwood/cpu.c	2016-05-21 11:33:32.840226195 -0700
@@ -251,10 +251,13 @@
 	reg = readl(KWGBE_PORT_SERIAL_CONTROL1_REG(0));
 	reg &= ~(1 << 4);	/* Clear PortReset Bit */
 	writel(reg, (KWGBE_PORT_SERIAL_CONTROL1_REG(0)));
+/* workaround RAMBOOT reading EGIGA1 Port Serial Control1 Register hang */
+#ifndef CONFIG_KIRKWOOD_EGIGA1_DISABLE
 	reg = readl(KWGBE_PORT_SERIAL_CONTROL1_REG(1));
 	reg &= ~(1 << 4);	/* Clear PortReset Bit */
 	writel(reg, (KWGBE_PORT_SERIAL_CONTROL1_REG(1)));
 #endif
+#endif
 #ifdef CONFIG_KIRKWOOD_PCIE_INIT
 	/*
 	 * Enable PCI Express Port0
diff -aurN u-boot-2016.05.orig/arch/arm/mach-kirkwood/Kconfig u-boot-2016.05-88fxxxx/arch/arm/mach-kirkwood/Kconfig
--- u-boot-2016.05.orig/arch/arm/mach-kirkwood/Kconfig	2016-05-16 07:40:32.000000000 -0700
+++ u-boot-2016.05-88fxxxx/arch/arm/mach-kirkwood/Kconfig	2016-05-21 11:33:32.840226195 -0700
@@ -19,6 +19,21 @@
 config TARGET_LSXL
 	bool "lsxl Board"
 
+config TARGET_POGO1
+	bool "Pogo Plug 1 Board"
+
+config TARGET_POGO2
+	bool "Pogo Plug 2 Board"
+
+config TARGET_POGO4
+	bool "Pogo Plug 4 Board"
+
+config TARGET_POGOM
+	bool "Pogo Plug M Board"
+
+config TARGET_SHEEVA
+	bool "Sheeva Plug Board"
+
 config TARGET_POGO_E02
 	bool "pogo_e02 Board"
 
@@ -62,6 +77,7 @@
 source "board/Marvell/guruplug/Kconfig"
 source "board/Marvell/sheevaplug/Kconfig"
 source "board/buffalo/lsxl/Kconfig"
+source "board/cloudengines/pogo/Kconfig"
 source "board/cloudengines/pogo_e02/Kconfig"
 source "board/d-link/dns325/Kconfig"
 source "board/iomega/iconnect/Kconfig"
diff -aurN u-boot-2016.05.orig/arch/arm/mach-mvebu/cpu.c u-boot-2016.05-88fxxxx/arch/arm/mach-mvebu/cpu.c
--- u-boot-2016.05.orig/arch/arm/mach-mvebu/cpu.c	2016-05-16 07:40:32.000000000 -0700
+++ u-boot-2016.05-88fxxxx/arch/arm/mach-mvebu/cpu.c	2016-05-21 11:33:32.840226195 -0700
@@ -55,6 +55,11 @@
 	case SOC_MV78460_ID:
 		return MVEBU_SOC_AXP;
 
+	case SOC_88F6707_ID:
+	case SOC_88F6710_ID:
+	case SOC_88F6W11_ID:
+		return MVEBU_SOC_A370;
+
 	case SOC_88F6720_ID:
 		return MVEBU_SOC_A375;
 
@@ -69,7 +74,17 @@
 
 #if defined(CONFIG_DISPLAY_CPUINFO)
 
-#if defined(CONFIG_ARMADA_375)
+#if defined(CONFIG_ARMADA_370)
+/* SAR frequency values for Armada 370 */
+static const struct sar_freq_modes sar_freq_tab[] = {
+	{  0x1, 0x13,  533, 533, 533 },
+	{  0x2,  0x1,  667, 333, 333 },
+	{  0x3, 0x14,  800, 533, 533 },
+	{  0x4, 0x14, 1000, 667, 667 },
+	{  0x6,  0x5, 1200, 600, 600 },
+	{ 0xff, 0xff,    0,   0,   0 }	/* 0xff marks end of array */
+};
+#elif defined(CONFIG_ARMADA_375)
 /* SAR frequency values for Armada 375 */
 static const struct sar_freq_modes sar_freq_tab[] = {
 	{  0,  0x0,  266,  133,  266 },
@@ -175,7 +190,7 @@
 	}
 
 	/* SAR value not found, return 0 for frequencies */
-	*sar_freq = sar_freq_tab[i - 1];
+	*sar_freq = sar_freq_tab[i];
 }
 
 int print_cpuinfo(void)
@@ -196,6 +211,15 @@
 	case SOC_MV78460_ID:
 		puts("MV78460-");
 		break;
+	case SOC_88F6707_ID:
+		puts("MV88F6707-");
+		break;
+	case SOC_88F6710_ID:
+		puts("MV88F6710-");
+		break;
+	case SOC_88F6W11_ID:
+		puts("MV88F6W11-");
+		break;
 	case SOC_88F6720_ID:
 		puts("MV88F6720-");
 		break;
@@ -227,6 +251,20 @@
 		}
 	}
 
+	if (mvebu_soc_family() == MVEBU_SOC_A370) {
+		switch (revid) {
+		case MV_88F6710_A0_ID:
+			puts("A0");
+			break;
+		case MV_88F6710_A1_ID:
+			puts("A1");
+			break;
+		default:
+			printf("?? (%x)", revid);
+			break;
+		}
+	}
+
 	if (mvebu_soc_family() == MVEBU_SOC_A375) {
 		switch (revid) {
 		case MV_88F67XX_A0_ID:
@@ -435,7 +473,10 @@
 		clrsetbits_le32(ARMADA_XP_PUP_ENABLE, 0,
 				GE0_PUP_EN | GE1_PUP_EN | LCD_PUP_EN |
 				NAND_PUP_EN | SPI_PUP_EN);
+	}
 
+	if (mvebu_soc_family() == MVEBU_SOC_AXP ||
+	    mvebu_soc_family() == MVEBU_SOC_A370) {
 		/* Configure USB PLL and PHYs on AXP */
 		setup_usb_phys();
 	}
@@ -443,6 +484,13 @@
 	/* Enable NAND and NAND arbiter */
 	clrsetbits_le32(MVEBU_SOC_DEV_MUX_REG, 0, NAND_EN | NAND_ARBITER_EN);
 
+#ifdef CONFIG_SYS_MVEBU_NAND_CLOCK
+	/* set NAND clock divisor ( divior = 2000 MHz / CLK_RATE / 2) */
+	clrsetbits_le32(MVEBU_CORE_DIV_CLK_CTRL(1), NAND_ECC_DIVCKL_RATIO_MASK,
+			DIV_ROUND_UP(1000000000, CONFIG_SYS_MVEBU_NAND_CLOCK) <<
+				NAND_ECC_DIVCKL_RATIO_OFFS);
+#endif
+
 	/* Disable MBUS error propagation */
 	clrsetbits_le32(SOC_COHERENCY_FABRIC_CTRL_REG, MBUS_ERR_PROP_EN, 0);
 
@@ -549,7 +597,8 @@
 
 void v7_outer_cache_enable(void)
 {
-	if (mvebu_soc_family() == MVEBU_SOC_AXP) {
+	if (mvebu_soc_family() == MVEBU_SOC_AXP ||
+	    mvebu_soc_family() == MVEBU_SOC_A370) {
 		struct pl310_regs *const pl310 =
 			(struct pl310_regs *)CONFIG_SYS_PL310_BASE;
 		u32 u;
diff -aurN u-boot-2016.05.orig/arch/arm/mach-mvebu/dram.c u-boot-2016.05-88fxxxx/arch/arm/mach-mvebu/dram.c
--- u-boot-2016.05.orig/arch/arm/mach-mvebu/dram.c	2016-05-16 07:40:32.000000000 -0700
+++ u-boot-2016.05-88fxxxx/arch/arm/mach-mvebu/dram.c	2016-05-21 11:33:32.840226195 -0700
@@ -12,7 +12,7 @@
 #include <asm/arch/cpu.h>
 #include <asm/arch/soc.h>
 
-#if defined(CONFIG_ARCH_MVEBU)
+#if defined(CONFIG_ARCH_MVEBU) && !defined(CONFIG_ARMADA_370)
 /* Use common XOR definitions for A3x and AXP */
 #include "../../../drivers/ddr/marvell/axp/xor.h"
 #include "../../../drivers/ddr/marvell/axp/xor_regs.h"
@@ -109,7 +109,7 @@
 	mvebu_sdram_bs_set(bank, size);
 }
 
-#if defined(CONFIG_ARCH_MVEBU)
+#if defined(CONFIG_ARCH_MVEBU) && !defined(CONFIG_ARMADA_370)
 static u32 xor_ctrl_save;
 static u32 xor_base_save;
 static u32 xor_mask_save;
diff -aurN u-boot-2016.05.orig/arch/arm/mach-mvebu/include/mach/config.h u-boot-2016.05-88fxxxx/arch/arm/mach-mvebu/include/mach/config.h
--- u-boot-2016.05.orig/arch/arm/mach-mvebu/include/mach/config.h	2016-05-16 07:40:32.000000000 -0700
+++ u-boot-2016.05-88fxxxx/arch/arm/mach-mvebu/include/mach/config.h	2016-05-21 11:33:32.840226195 -0700
@@ -24,6 +24,8 @@
  * for A38x as well here.
  */
 #define MV88F78X60 /* for the DDR training bin_hdr code */
+#elif defined(CONFIG_ARMADA_370)
+#define MV88F67XX /* for the DDR training bin_hdr code */
 #endif
 
 #define CONFIG_SYS_CACHELINE_SIZE	32
@@ -102,6 +104,10 @@
 /* Use common timer */
 #define CONFIG_SYS_TIMER_COUNTS_DOWN
 #define CONFIG_SYS_TIMER_COUNTER	(MVEBU_TIMER_BASE + 0x14)
+#if defined(CONFIG_ARMADA_370)
+#define CONFIG_SYS_TIMER_RATE		CONFIG_SYS_L2CLK
+#else
 #define CONFIG_SYS_TIMER_RATE		25000000
+#endif
 
 #endif /* __MVEBU_CONFIG_H */
diff -aurN u-boot-2016.05.orig/arch/arm/mach-mvebu/include/mach/cpu.h u-boot-2016.05-88fxxxx/arch/arm/mach-mvebu/include/mach/cpu.h
--- u-boot-2016.05.orig/arch/arm/mach-mvebu/include/mach/cpu.h	2016-05-16 07:40:32.000000000 -0700
+++ u-boot-2016.05-88fxxxx/arch/arm/mach-mvebu/include/mach/cpu.h	2016-05-21 11:33:32.840226195 -0700
@@ -61,6 +61,7 @@
 
 enum {
 	MVEBU_SOC_AXP,
+	MVEBU_SOC_A370,
 	MVEBU_SOC_A375,
 	MVEBU_SOC_A38X,
 	MVEBU_SOC_UNKNOWN,
diff -aurN u-boot-2016.05.orig/arch/arm/mach-mvebu/include/mach/soc.h u-boot-2016.05-88fxxxx/arch/arm/mach-mvebu/include/mach/soc.h
--- u-boot-2016.05.orig/arch/arm/mach-mvebu/include/mach/soc.h	2016-05-16 07:40:32.000000000 -0700
+++ u-boot-2016.05-88fxxxx/arch/arm/mach-mvebu/include/mach/soc.h	2016-05-21 11:33:32.840226195 -0700
@@ -14,11 +14,18 @@
 #define SOC_MV78230_ID		0x7823
 #define SOC_MV78260_ID		0x7826
 #define SOC_MV78460_ID		0x7846
+#define SOC_88F6707_ID		0x6707
+#define SOC_88F6710_ID		0x6710
+#define SOC_88F6W11_ID		0x6711
 #define SOC_88F6720_ID		0x6720
 #define SOC_88F6810_ID		0x6810
 #define SOC_88F6820_ID		0x6820
 #define SOC_88F6828_ID		0x6828
 
+/* A370 revisions */
+#define MV_88F6710_A0_ID	0x0
+#define MV_88F6710_A1_ID	0x1
+
 /* A375 revisions */
 #define MV_88F67XX_A0_ID	0x3
 
@@ -91,7 +98,7 @@
 #define NAND_PUP_EN		BIT(4)
 #define SPI_PUP_EN		BIT(5)
 
-#define MVEBU_CORE_DIV_CLK_CTRL(i)	(MVEBU_CLOCK_BASE + ((i) * 0x8))
+#define MVEBU_CORE_DIV_CLK_CTRL(i)	(MVEBU_CLOCK_BASE + 0x40 + ((i) * 0x8))
 #define NAND_ECC_DIVCKL_RATIO_OFFS	8
 #define NAND_ECC_DIVCKL_RATIO_MASK	(0x3F << NAND_ECC_DIVCKL_RATIO_OFFS)
 
@@ -111,7 +118,24 @@
 #define BOOTROM_ERR_MODE_MASK	(0xf << BOOTROM_ERR_MODE_OFFS)
 #define BOOTROM_ERR_MODE_UART	0x6
 
-#if defined(CONFIG_ARMADA_375)
+#if defined(CONFIG_ARMADA_370)
+/* SAR values for Armada 370 */
+#define CONFIG_SAR_REG		(MVEBU_REGISTER(0x18230))
+#define CONFIG_SAR2_REG		(MVEBU_REGISTER(0x18234))
+
+#define SAR_CPU_FREQ_OFFS	11
+#define SAR_CPU_FREQ_MASK	(0xf << SAR_CPU_FREQ_OFFS)
+#define SAR_FFC_FREQ_OFFS	15
+#define SAR_FFC_FREQ_MASK	(0x1f << SAR_FFC_FREQ_OFFS)
+#define SAR_BOOT_DEVICE_OFFS	1
+#define SAR_BOOT_DEVICE_MASK	(0x3f << SAR_BOOT_DEVICE_OFFS)
+
+#define BOOT_DEV_SEL_OFFS	1
+#define BOOT_DEV_SEL_MASK	(0x3f << BOOT_DEV_SEL_OFFS)
+
+#define BOOT_FROM_UART		0x4
+#define BOOT_FROM_SPI		0x10
+#elif defined(CONFIG_ARMADA_375)
 /* SAR values for Armada 375 */
 #define CONFIG_SAR_REG		(MVEBU_REGISTER(0xe8200))
 #define CONFIG_SAR2_REG		(MVEBU_REGISTER(0xe8204))
diff -aurN u-boot-2016.05.orig/arch/arm/mach-mvebu/Kconfig u-boot-2016.05-88fxxxx/arch/arm/mach-mvebu/Kconfig
--- u-boot-2016.05.orig/arch/arm/mach-mvebu/Kconfig	2016-05-16 07:40:32.000000000 -0700
+++ u-boot-2016.05-88fxxxx/arch/arm/mach-mvebu/Kconfig	2016-05-21 11:33:32.840226195 -0700
@@ -9,6 +9,15 @@
 config ARMADA_XP
 	bool
 
+config ARMADA_370
+	bool
+
+config LS400
+	bool
+	select ARMADA_370
+	select CMD_POWEROFF
+	select DM_GPIO
+
 config MV78230
 	bool
 	select ARMADA_XP
@@ -26,7 +35,7 @@
 	select ARMADA_38X
 
 choice
-	prompt "Marvell MVEBU (Armada XP/375/38x) board select"
+	prompt "Marvell MVEBU (Armada XP/370/375/38x) board select"
 	optional
 
 config TARGET_CLEARFOG
@@ -49,6 +58,14 @@
 	bool "Support Synology DS414"
 	select MV78230
 
+config TARGET_LS420
+	bool "Support Buffalo LinkStation 420 Series NAS"
+	select LS400
+
+config TARGET_LS440
+	bool "Support Buffalo LinkStation 440 Series NAS"
+	select LS400
+
 config TARGET_MAXBCM
 	bool "Support maxbcm"
 	select MV78460
@@ -65,6 +82,7 @@
 	default "db-88f6820-gp" if TARGET_DB_88F6820_GP
 	default "db-mv784mp-gp" if TARGET_DB_MV784MP_GP
 	default "ds414" if TARGET_DS414
+	default "ls400" if LS400
 	default "maxbcm" if TARGET_MAXBCM
 	default "theadorable" if TARGET_THEADORABLE
 
@@ -74,10 +92,12 @@
 	default "db-88f6820-gp" if TARGET_DB_88F6820_GP
 	default "db-mv784mp-gp" if TARGET_DB_MV784MP_GP
 	default "ds414" if TARGET_DS414
+	default "ls400" if LS400
 	default "maxbcm" if TARGET_MAXBCM
 	default "theadorable" if TARGET_THEADORABLE
 
 config SYS_VENDOR
+	default "buffalo" if LS400
 	default "Marvell" if TARGET_DB_MV784MP_GP
 	default "Marvell" if TARGET_DB_88F6720
 	default "Marvell" if TARGET_DB_88F6820_GP
diff -aurN u-boot-2016.05.orig/arch/arm/mach-mvebu/Makefile u-boot-2016.05-88fxxxx/arch/arm/mach-mvebu/Makefile
--- u-boot-2016.05.orig/arch/arm/mach-mvebu/Makefile	2016-05-16 07:40:32.000000000 -0700
+++ u-boot-2016.05-88fxxxx/arch/arm/mach-mvebu/Makefile	2016-05-21 11:33:32.840226195 -0700
@@ -25,6 +25,7 @@
 obj-$(CONFIG_SPL_BUILD) += spl.o
 obj-$(CONFIG_SPL_BUILD) += lowlevel_spl.o
 
+obj-$(CONFIG_ARMADA_370) += serdes/a370/
 obj-$(CONFIG_ARMADA_38X) += serdes/a38x/
 obj-$(CONFIG_ARMADA_XP) += serdes/axp/
 
diff -aurN u-boot-2016.05.orig/arch/arm/mach-mvebu/serdes/a370/Makefile u-boot-2016.05-88fxxxx/arch/arm/mach-mvebu/serdes/a370/Makefile
--- u-boot-2016.05.orig/arch/arm/mach-mvebu/serdes/a370/Makefile	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/arch/arm/mach-mvebu/serdes/a370/Makefile	2016-05-21 11:33:32.840226195 -0700
@@ -0,0 +1,8 @@
+#
+# (C) Copyright 2016
+# Steve Shih <shihsung@gmail.com>
+#
+# SPDX-License-Identifier:	GPL-2.0+
+#
+
+obj-$(CONFIG_SPL_BUILD)	= serdes.o
diff -aurN u-boot-2016.05.orig/arch/arm/mach-mvebu/serdes/a370/serdes.c u-boot-2016.05-88fxxxx/arch/arm/mach-mvebu/serdes/a370/serdes.c
--- u-boot-2016.05.orig/arch/arm/mach-mvebu/serdes/a370/serdes.c	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/arch/arm/mach-mvebu/serdes/a370/serdes.c	2016-05-21 11:33:32.840226195 -0700
@@ -0,0 +1,296 @@
+/*
+ * (C) Copyright 2014, 2016
+ * Steve Shih <shihsung@gmail.com>
+ *
+ * SPDX-License-Identifier:	GPL-2.0+
+ */
+
+#include <common.h>
+#include <asm/io.h>
+#include <asm/arch/cpu.h>
+#include "serdes.h"
+
+#define SERDES_TIMEOUT			(3 * CONFIG_SYS_HZ)
+
+#define INDEX_NONE			~0U
+
+#define A370_PCIE_MAX			2
+#define A370_SATA_MAX			2
+#define A370_ETH_MAX			2
+
+#define A370_REGISTER(r)		MVEBU_REGISTER(r)
+#define A370_PCIE_BASE(p)		A370_REGISTER(0x40000 * ((p) + 1))
+#define A370_SATA_BASE(p)		A370_REGISTER(0xa2000 + 0x2000 * (p))
+#define A370_ETH_BASE(p)		A370_REGISTER(0x70000 + 0x4000 * (p))
+
+/* set in Boot ROM Routine and Error Code Register 0x182d0 by DDR3 training code */
+#define A370_DRAM_INIT			(1 << 12)
+
+struct serdes_map {
+	u32	index;
+	u32	select;
+};
+
+static struct serdes_map mv88f67xx_map_tbl[A370_SERDES_MAX][SERDES_SEL_MAX] =
+{
+	/*	index		select	*/
+	{
+		{ INDEX_NONE,	0 },	/* SERDES_SEL_NONE */
+		{ 0,		1 },	/* SERDES_SEL_PCIE */
+		{ 0,		2 },	/* SERDES_SEL_SATA */
+		{ 1,		3 }	/* SERDES_SEL_SGMII */
+	},
+	{
+		{ INDEX_NONE,	0 },
+		{ 1,		1 },
+		{ INDEX_NONE,	0 },
+		{ 0,		2 }
+	},
+	{
+		{ INDEX_NONE,	0 },
+		{ INDEX_NONE,	0 },
+		{ 0,		1 },
+		{ 0,		2 }
+	},
+	{
+		{ INDEX_NONE,	0 },
+		{ INDEX_NONE,	0 },
+		{ 1,		1 },
+		{ 1,		2 }
+	}
+};
+
+static struct serdes_map mv88f6w11_map_tbl[A370_SERDES_MAX][SERDES_SEL_MAX] =
+{
+	/*	index		select	*/
+	{
+		{ INDEX_NONE,	0 },	/* SERDES_SEL_NONE */
+		{ 0,		1 },	/* SERDES_SEL_PCIE */
+		{ INDEX_NONE,	0 },	/* SERDES_SEL_SATA */
+		{ 1,		3 }	/* SERDES_SEL_SGMII */
+	},
+	{
+		{ INDEX_NONE,	0 },
+		{ 1,		1 },
+		{ INDEX_NONE,	0 },
+		{ 0,		2 }
+	},
+	{
+		{ INDEX_NONE,	0 },
+		{ INDEX_NONE,	0 },
+		{ INDEX_NONE,	0 },
+		{ 0,		2 }
+	},
+	{
+		{ INDEX_NONE,	0 },
+		{ INDEX_NONE,	0 },
+		{ INDEX_NONE,	0 },
+		{ 1,		2 }
+	}
+};
+
+static void serdes_init_none (struct serdes_map *map, enum serdes_opt opt)
+{
+}
+
+static void serdes_init_pcie (struct serdes_map *map, enum serdes_opt opt)
+{
+	if (map->index >= A370_PCIE_MAX)
+		return;
+
+	/* PCIe Pipe Setting Phase I */
+	writel(0x00c10025, A370_PCIE_BASE(map->index) + 0x1b00);
+	writel(0x00c3000f, A370_PCIE_BASE(map->index) + 0x1b00);
+	writel(0x00c5011f, A370_PCIE_BASE(map->index) + 0x1b00);
+	writel(0x00c80005, A370_PCIE_BASE(map->index) + 0x1b00);
+	writel(0x00d00100, A370_PCIE_BASE(map->index) + 0x1b00);
+	writel(0x00d13014, A370_PCIE_BASE(map->index) + 0x1b00);
+	writel(0x00801000, A370_PCIE_BASE(map->index) + 0x1b00);
+	writel(0x00810011, A370_PCIE_BASE(map->index) + 0x1b00);
+
+	/* Operatonal Mode and Reference Clock Setting */
+	writel(0x0001fc60, A370_PCIE_BASE(map->index) + 0x1b00);
+
+	/* Reference Clock Select Setting */
+	writel(0x00460000, A370_PCIE_BASE(map->index) + 0x1b00);
+
+	/* Impedance Tuning */
+	writel(0x00489080, A370_PCIE_BASE(map->index) + 0x1b00);
+
+	/* PCIe Pipe Setting Phase II */
+	writel(0x00c10024, A370_PCIE_BASE(map->index) + 0x1b00);
+
+	/* enable PEX */
+	setbits_le32(A370_REGISTER(0x18204), 1 << map->index);
+
+	/* fixup */
+	writel(0x000f2a21, A370_PCIE_BASE(map->index) + 0x1b00);
+	writel(0x004f6219, A370_PCIE_BASE(map->index) + 0x1b00);
+}
+
+static void serdes_init_sata (struct serdes_map *map, enum serdes_opt opt)
+{
+	unsigned long start;
+
+	if (map->index >= A370_SATA_MAX)
+		return;
+
+	/* Operatonal Mode and Reference Clock Setting */
+	writel(0x0000f801, A370_SATA_BASE(map->index) + 0x0804);
+
+	/* Bits Decode Setting */
+	writel(0x00000400, A370_SATA_BASE(map->index) + 0x088c);
+
+	/* Reference Clock Select Setting */
+	writel(0x00000400, A370_SATA_BASE(map->index) + 0x0918);
+
+	/* Impedance Tuning */
+	writel(0x00008080, A370_SATA_BASE(map->index) + 0x0920);
+
+	/* Protocol Generation Setting */
+	if (opt == SERDES_OPT_SPD_LO)
+		writel(0x00000007, A370_SATA_BASE(map->index) + 0x0058);
+	else
+		writel(0x00000227, A370_SATA_BASE(map->index) + 0x0058);
+
+	/* Poll for Initialization Completed */
+	start = get_timer(0);
+	do {
+		if ((readl(A370_SATA_BASE(map->index) + 0x005c) & 7) == 7)
+			break;
+	} while (get_timer(start) < SERDES_TIMEOUT);
+
+	if (get_timer(start) >= SERDES_TIMEOUT)
+		printf("SATA%d SerDes initialization failed\n", map->index);
+
+	/* fixup */
+	writel(0x0000c928, A370_SATA_BASE(map->index) + 0x0834);
+	writel(0x000030f0, A370_SATA_BASE(map->index) + 0x0838);
+	writel(0x0000aa31, A370_SATA_BASE(map->index) + 0x083c);
+	writel(0x000030f5, A370_SATA_BASE(map->index) + 0x0840);
+}
+
+static void serdes_init_sgmii (struct serdes_map *map, enum serdes_opt opt)
+{
+	unsigned long start;
+
+	if (map->index >= A370_ETH_MAX)
+		return;
+
+	/* Operatonal Mode and Reference Clock Setting */
+	writel(0x0000f881, A370_ETH_BASE(map->index) + 0x2e04);
+
+	/* Bits Decode Setting */
+	writel(0x00000000, A370_ETH_BASE(map->index) + 0x2e8c);
+
+	/* Reference Clock Select Setting */
+	writel(0x00000400, A370_ETH_BASE(map->index) + 0x2f18);
+
+	/* Impedance Tuning */
+	writel(0x00008080, A370_ETH_BASE(map->index) + 0x2f20);
+
+	/* Protocol Generation Setting */
+	if (opt == SERDES_OPT_SPD_HI)
+		writel(0x00001107, A370_ETH_BASE(map->index) + 0x24a0);
+	else
+		writel(0x00000cc7, A370_ETH_BASE(map->index) + 0x24a0);
+
+	/* Poll for Initialization Completed */
+	start = get_timer(0);
+	do {
+		if ((readl(A370_ETH_BASE(map->index) + 0x24a4) & 7) == 7);
+			break;
+	} while (get_timer(start) < SERDES_TIMEOUT);
+
+	if (get_timer(start) >= SERDES_TIMEOUT)
+		printf("SGMII%d SerDes initialization failed\n", map->index);
+
+	/* fixup */
+	writel(0x0000c964, A370_ETH_BASE(map->index) + 0x2e34);
+}
+
+typedef void (*serdes_init)(struct serdes_map *map, enum serdes_opt opt);
+
+static serdes_init init_tbl[SERDES_SEL_MAX] = {
+	serdes_init_none,
+	serdes_init_pcie,
+	serdes_init_sata,
+	serdes_init_sgmii
+};
+
+static void a370_serdes_config(struct serdes_cfg setup[A370_SERDES_MAX])
+{
+	u16 devid = (readl(MVEBU_REG_PCIE_DEVID) >> 16) & 0xffff;
+	int i;
+	struct serdes_cfg *cfg;
+	struct serdes_map *map;
+	u32 reg = 0;
+
+	/* avoid re-init SerDes which often causes hang */
+	if (readl(CONFIG_BOOTROM_ERR_REG) & A370_DRAM_INIT)
+		return;
+
+	/* power down PCIe */
+	writel(0x00017c60, A370_PCIE_BASE(0) + 0x1b00);
+	writel(0x00017c60, A370_PCIE_BASE(1) + 0x1b00);
+	/* disable PCIe */
+	writel(0x00000000, A370_REGISTER(0x18204));
+
+	/* power down SATA */
+	writel(0x00000220, A370_SATA_BASE(0) + 0x0058);
+	writel(0x00000220, A370_SATA_BASE(1) + 0x0058);
+
+	/* power down SGMII */
+	writel(0x00000cc0, A370_ETH_BASE(0) + 0x24a0);
+	writel(0x00000cc0, A370_ETH_BASE(1) + 0x24a0);
+
+	/* configure serdes */
+	for (i = 0, cfg = setup; i < A370_SERDES_MAX; i++, cfg++) {
+		if (cfg->sel >= SERDES_SEL_MAX) {
+			printf("Invalid SerDes%d config %d\n", i, cfg->sel);
+			continue;
+		}
+
+		if (devid == SOC_88F6W11_ID)
+			map = &mv88f6w11_map_tbl[i][cfg->sel];
+		else
+			map = &mv88f67xx_map_tbl[i][cfg->sel];
+
+		if (map->index == INDEX_NONE) {
+			if (cfg->sel != SERDES_SEL_NONE)
+				printf("Invalid SerDes%d config %d\n", i, cfg->sel);
+			continue;
+		}
+		reg |= map->select << ( i * 4);
+	}
+	writel(reg, A370_REGISTER(0x18270));
+
+	/* initialize serdes */
+	for (i = 0, cfg = setup; i < A370_SERDES_MAX; i++, cfg++) {
+		if (cfg->sel >= SERDES_SEL_MAX)
+			continue;
+
+		if (devid == SOC_88F6W11_ID)
+			map = &mv88f6w11_map_tbl[i][cfg->sel];
+		else
+			map = &mv88f67xx_map_tbl[i][cfg->sel];
+
+		if (map->index != INDEX_NONE)
+			init_tbl[cfg->sel](map, cfg->opt);
+	}
+}
+
+__weak struct serdes_cfg* board_serdes_cfg (void)
+{
+	return NULL;
+}
+
+int serdes_phy_config (void)
+{
+	struct serdes_cfg *cfg = board_serdes_cfg();
+
+	if (cfg)
+		a370_serdes_config(cfg);
+
+	return 0;
+}
diff -aurN u-boot-2016.05.orig/arch/arm/mach-mvebu/serdes/a370/serdes.h u-boot-2016.05-88fxxxx/arch/arm/mach-mvebu/serdes/a370/serdes.h
--- u-boot-2016.05.orig/arch/arm/mach-mvebu/serdes/a370/serdes.h	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/arch/arm/mach-mvebu/serdes/a370/serdes.h	2016-05-21 11:33:32.840226195 -0700
@@ -0,0 +1,36 @@
+/*
+ * (C) Copyright 2014, 2016
+ * Steve Shih <shihsung@gmail.com>
+ *
+ * SPDX-License-Identifier:	GPL-2.0+
+ */
+
+#ifndef __A370_SERDES_H
+#define __A370_SERDES_H
+
+#define A370_SERDES_MAX			4
+
+enum serdes_sel {
+	SERDES_SEL_NONE,
+	SERDES_SEL_PCIE,
+	SERDES_SEL_SATA,
+	SERDES_SEL_SGMII,
+
+	SERDES_SEL_MAX
+};
+
+enum serdes_opt {
+	SERDES_OPT_SPD_HI,
+	SERDES_OPT_SPD_LO,
+
+	SERES_OPT__MAX
+};
+
+struct serdes_cfg {
+	enum serdes_sel	sel;
+	enum serdes_opt	opt;
+};
+
+struct serdes_cfg* board_serdes_cfg(void);
+
+#endif /* __A370_SERDES_H */
diff -aurN u-boot-2016.05.orig/arch/arm/mach-mvebu/timer.c u-boot-2016.05-88fxxxx/arch/arm/mach-mvebu/timer.c
--- u-boot-2016.05.orig/arch/arm/mach-mvebu/timer.c	2016-05-16 07:40:32.000000000 -0700
+++ u-boot-2016.05-88fxxxx/arch/arm/mach-mvebu/timer.c	2016-05-21 11:33:32.840226195 -0700
@@ -31,7 +31,7 @@
 	writel(TIMER_LOAD_VAL, MVEBU_TIMER_BASE + 0x10);
 	writel(TIMER_LOAD_VAL, MVEBU_TIMER_BASE + 0x14);
 
-#if defined(CONFIG_ARCH_MVEBU)
+#if defined(CONFIG_ARCH_MVEBU) && !defined(CONFIG_ARMADA_370)
 	/* On Armada XP / 38x ..., the 25MHz clock source needs to be enabled */
 	setbits_le32(MVEBU_TIMER_BASE + 0x00, BIT(11));
 #endif
diff -aurN u-boot-2016.05.orig/arch/arm/mach-orion5x/cpu.c u-boot-2016.05-88fxxxx/arch/arm/mach-orion5x/cpu.c
--- u-boot-2016.05.orig/arch/arm/mach-orion5x/cpu.c	2016-05-16 07:40:32.000000000 -0700
+++ u-boot-2016.05-88fxxxx/arch/arm/mach-orion5x/cpu.c	2016-05-21 11:33:32.840226195 -0700
@@ -234,10 +234,19 @@
 	/* Enable and invalidate L2 cache in write through mode */
 	invalidate_l2_cache();
 
-#ifdef CONFIG_SPL_BUILD
+#if defined(CONFIG_SPL_BUILD) || defined(CONFIG_SYS_ROMBOOT)
 	orion5x_config_adr_windows();
 #endif
 
+	/* Set GPIOs and MPPs - values provided by board
+	   include file */
+	writel(ORION5X_MPP0_7, ORION5X_MPP_BASE+0x00);
+	writel(ORION5X_MPP8_15, ORION5X_MPP_BASE+0x04);
+	writel(ORION5X_MPP16_23, ORION5X_MPP_BASE+0x50);
+	writel(ORION5X_GPIO_OUT_VALUE, ORION5X_GPIO_BASE+0x00);
+	writel(ORION5X_GPIO_OUT_ENABLE, ORION5X_GPIO_BASE+0x04);
+	writel(ORION5X_GPIO_IN_POLARITY, ORION5X_GPIO_BASE+0x0c);
+
 	return 0;
 }
 #endif /* CONFIG_ARCH_CPU_INIT */
@@ -272,17 +281,17 @@
 	temp = get_cr();
 	set_cr(temp & ~CR_V);
 
-	/* Set CPIOs and MPPs - values provided by board
-	   include file */
-	writel(ORION5X_MPP0_7, ORION5X_MPP_BASE+0x00);
-	writel(ORION5X_MPP8_15, ORION5X_MPP_BASE+0x04);
-	writel(ORION5X_MPP16_23, ORION5X_MPP_BASE+0x50);
-	writel(ORION5X_GPIO_OUT_VALUE, ORION5X_GPIO_BASE+0x00);
-	writel(ORION5X_GPIO_OUT_ENABLE, ORION5X_GPIO_BASE+0x04);
-	writel(ORION5X_GPIO_IN_POLARITY, ORION5X_GPIO_BASE+0x0c);
-
 	/* initialize timer */
 	timer_init_r();
+
+	/* USB workaround */
+#ifdef CONFIG_USB_EHCI_MARVELL
+	orion5x_usb_workaround(MVUSB0_BASE);
+#ifdef MVUSB1_BASE
+	orion5x_usb_workaround(MVUSB1_BASE);
+#endif
+#endif
+
 	return 0;
 }
 #endif /* CONFIG_ARCH_MISC_INIT */
diff -aurN u-boot-2016.05.orig/arch/arm/mach-orion5x/include/mach/cpu.h u-boot-2016.05-88fxxxx/arch/arm/mach-orion5x/include/mach/cpu.h
--- u-boot-2016.05.orig/arch/arm/mach-orion5x/include/mach/cpu.h	2016-05-16 07:40:32.000000000 -0700
+++ u-boot-2016.05-88fxxxx/arch/arm/mach-orion5x/include/mach/cpu.h	2016-05-21 11:33:32.840226195 -0700
@@ -239,5 +239,6 @@
 u32 orion5x_device_rev(void);
 unsigned int orion5x_winctrl_calcsize(unsigned int sizeval);
 void timer_init_r(void);
+void orion5x_usb_workaround(unsigned long port_base);
 #endif /* __ASSEMBLY__ */
 #endif /* _ORION5X_CPU_H */
diff -aurN u-boot-2016.05.orig/arch/arm/mach-orion5x/include/mach/orion5x.h u-boot-2016.05-88fxxxx/arch/arm/mach-orion5x/include/mach/orion5x.h
--- u-boot-2016.05.orig/arch/arm/mach-orion5x/include/mach/orion5x.h	2016-05-16 07:40:32.000000000 -0700
+++ u-boot-2016.05-88fxxxx/arch/arm/mach-orion5x/include/mach/orion5x.h	2016-05-21 11:33:32.840226195 -0700
@@ -43,7 +43,12 @@
 #define MVGBE0_BASE	ORION5X_EGIGA_BASE
 
 /* Orion5x USB Host controller is port 1 */
-#define MVUSB0_BASE			ORION5X_USB20_HOST_PORT_BASE
+#ifndef MVUSB0_BASE
+#define MVUSB0_BASE			ORION5X_USB20_PORT0_BASE
+#endif
+#ifndef MVUSB1_BASE
+#define MVUSB1_BASE			ORION5X_USB20_PORT1_BASE
+#endif
 #define MVUSB0_CPU_ATTR_DRAM_CS0	ORION5X_ATTR_DRAM_CS0
 #define MVUSB0_CPU_ATTR_DRAM_CS1	ORION5X_ATTR_DRAM_CS1
 #define MVUSB0_CPU_ATTR_DRAM_CS2	ORION5X_ATTR_DRAM_CS2
@@ -54,7 +59,9 @@
 #define MVCPU_WIN_ENABLE	ORION5X_WIN_ENABLE
 #define MVCPU_WIN_DISABLE	ORION5X_WIN_DISABLE
 
+#ifndef CONFIG_MAX_RAM_BANK_SIZE
 #define CONFIG_MAX_RAM_BANK_SIZE		(64*1024*1024)
+#endif
 
 /* include here SoC variants. 5181, 5281, 6183 should go here when
    adding support for them, and this comment should then be updated. */
diff -aurN u-boot-2016.05.orig/arch/arm/mach-orion5x/Kconfig u-boot-2016.05-88fxxxx/arch/arm/mach-orion5x/Kconfig
--- u-boot-2016.05.orig/arch/arm/mach-orion5x/Kconfig	2016-05-16 07:40:32.000000000 -0700
+++ u-boot-2016.05-88fxxxx/arch/arm/mach-orion5x/Kconfig	2016-05-21 11:33:32.840226195 -0700
@@ -8,11 +8,20 @@
 	bool "LaCie Ethernet Disk mini V2"
 	select SUPPORT_SPL
 
+config TARGET_KUROPRO
+	bool "Buffalo Kurobox Pro"
+	select CMD_POWEROFF
+
+config TARGET_LSGL
+	bool "Buffalo Linkstation Pro"
+	select CMD_POWEROFF
+
 endchoice
 
 config SYS_SOC
 	default "orion5x"
 
 source "board/LaCie/edminiv2/Kconfig"
+source "board/buffalo/lsgl/Kconfig"
 
 endif
diff -aurN u-boot-2016.05.orig/arch/arm/mach-orion5x/lowlevel_init.S u-boot-2016.05-88fxxxx/arch/arm/mach-orion5x/lowlevel_init.S
--- u-boot-2016.05.orig/arch/arm/mach-orion5x/lowlevel_init.S	2016-05-16 07:40:32.000000000 -0700
+++ u-boot-2016.05-88fxxxx/arch/arm/mach-orion5x/lowlevel_init.S	2016-05-21 11:33:32.840226195 -0700
@@ -15,6 +15,27 @@
  * Configuration values for SDRAM access setup
  */
 
+#ifdef CONFIG_SYS_DDR2
+
+/* DDR2 200MHz: 2 x 512Mbit x 16 ==> 128MB */
+#define SDRAM_EXT_MODE			0x00000440
+#define SDRAM_ODT_CTRL_LOW		0x00010000
+#define SDRAM_ODT_CTRL_HI		0x00000000
+#define SDRAM_ODT_CTRL			0x00000801
+#define DDR2_PAD_STRENGTH_DEFAULT	0x00003000
+#define FTDLL_DDR2_200MHZ		0x00f95001
+
+#define SDRAM_CONFIG			0x03158618
+#define SDRAM_MODE			0x00000432
+#define SDRAM_CONTROL			0x04041040
+#define SDRAM_TIME_CTRL_LOW		0x11812220
+#define SDRAM_TIME_CTRL_HI		0x0000030f
+#define SDRAM_OPEN_PAGE_EN		0x00000000
+#define SDRAM_BANK0_SIZE		0x07ff0001
+#define SDRAM_ADDR_CTRL			0x00000020
+
+#else
+
 #define SDRAM_CONFIG			0x3148400
 #define SDRAM_MODE			0x62
 #define SDRAM_CONTROL			0x4041000
@@ -25,8 +46,11 @@
 #define SDRAM_BANK0_SIZE		0x3ff0001
 #define SDRAM_ADDR_CTRL			0x10
 
+#endif
+
 #define SDRAM_OP_NOP			0x05
 #define SDRAM_OP_SETMODE		0x03
+#define SDRAM_OP_SETEXTMODE		0x04
 
 #define SDRAM_PAD_CTRL_WR_EN		0x80000000
 #define SDRAM_PAD_CTRL_TUNE_EN		0x00010000
@@ -70,7 +94,7 @@
 
 lowlevel_init:
 
-#ifdef CONFIG_SPL_BUILD
+#if defined(CONFIG_SPL_BUILD) || defined(CONFIG_SYS_ROMBOOT)
 
 	/* Use 'r4 as the base for internal register accesses */
 	ldr	r4, =ORION5X_REGS_PHY_BASE
@@ -166,6 +190,68 @@
 	cmp	r6, #0
 	bne	2b
 
+#ifdef CONFIG_SYS_DDR2
+	/* Config DDR2 registers (Extended mode, ODTs and pad calibration) */
+	ldr	r6, =SDRAM_CONFIG
+	tst	r6, #0x10000
+	beq	3f
+
+	/* 9) Write SDRAM Extended mode register */
+	/* This operation should be done for each memory bank */
+
+	/* write 'nop' to SDRAM operation */
+	ldr	r6, =SDRAM_OP_NOP
+	str	r6, [r3, #0x418]
+
+1:
+	/* Poll SDRAM operation. Make sure it's back to normal operation */
+	ldr	r6, [r3, #0x418]
+	cmp	r6, #0
+	bne	1b
+
+	/* Now it's safe to write new value to SDRAM Extended Mode register */
+	ldr	r6, =SDRAM_EXT_MODE
+	str	r6, [r3, #0x420]
+
+	/* Go over each of the banks */
+	ldr	r1, =0	/* r6 = DRAM bank num */
+
+2:
+	/* Set the SDRAM Operation Control to each of the DRAM banks */
+	mov	r6, r1	/* Do not swap the bank counter value */
+	str	r6, [r3, #0x42c]
+
+	/* Make the Dunit write the DRAM its new mode */
+	ldr	r6, =SDRAM_OP_SETEXTMODE
+	str	r6, [r3, #0x418]
+
+1:
+	/* Poll SDRAM operation. Make sure it's back to normal operation */
+	ldr	r6, [r3, #0x418]
+	cmp	r6, #0
+	bne	1b
+
+	add	r1, r1, #1
+	cmp	r1, #4		/* 4 = Number of banks */
+	bne	2b
+
+	/* Config DDR2 On Die Termination (ODT) registers */
+
+	/* Write SDRAM DDR2 ODT Control Low register */
+	ldr	r6, =SDRAM_ODT_CTRL_LOW
+	str	r6, [r3, #0x494]
+
+	/* Write SDRAM DDR2 ODT Control High register */
+	ldr	r6, =SDRAM_ODT_CTRL_HI
+	str	r6, [r3, #0x498]
+
+	/* Write SDRAM DDR2 ODT Control register */
+	ldr	r6, =SDRAM_ODT_CTRL
+	str	r6, [r3, #0x49c]
+
+3:
+#endif
+
 	/* DDR SDRAM Address/Control Pads Calibration */
 	ldr	r6, [r3, #0x4C0]
 
@@ -214,8 +300,23 @@
 	/* Implement Guideline (GL# MEM-3) Drive Strength Value		*/
 	/* Relevant for: 88F5181-A1/B0/B1 and 88F5281-A0/B0		*/
 
+#ifdef CONFIG_SYS_DDR2
+	ldr	r6, =SDRAM_CONFIG
+	tst	r6, #0x10000
+	beq	1f
+
+	ldr	r1, =DDR2_PAD_STRENGTH_DEFAULT
+	b	2f
+
+1:
+#endif
+
 	ldr	r1, =DDR1_PAD_STRENGTH_DEFAULT
 
+#ifdef CONFIG_SYS_DDR2
+2:
+#endif
+
 	/* Enable writes to DDR SDRAM Addr/Ctrl Pads Calibration register */
 	ldr	r6, [r3, #0x4C0]
 	orr	r6, r6, #SDRAM_PAD_CTRL_WR_EN
@@ -241,6 +342,17 @@
 	/* Implement Guideline (GL# MEM-4) DQS Reference Delay Tuning	*/
 	/* Relevant for: 88F5181-A1/B0/B1 and 88F5281-A0/B0		*/
 
+#ifdef CONFIG_SYS_DDR2
+	ldr	r6, =SDRAM_CONFIG
+	tst	r6, #0x10000
+	beq	1f
+
+	ldr	r6, =FTDLL_DDR2_200MHZ
+	b	3f
+
+1:
+#endif
+
 	/* Get the "sample on reset" register for the DDR frequancy	*/
 	ldr	r3, =0x10000
 	ldr	r6, [r3, #0x010]
@@ -275,13 +387,15 @@
 	orr	r2, r2, r6
 	str	r2, [r3, #0x484]
 
+#ifndef CONFIG_SYS_ROMBOOT
 	/* enable for 2 GB DDR; detection should find out real amount */
 	sub	r6, r6, r6
 	str	r6, [r3, #0x500]
 	ldr	r6, =0x7fff0001
 	str	r6, [r3, #0x504]
+#endif
 
-#endif /* CONFIG_SPL_BUILD */
+#endif /* CONFIG_SPL_BUILD || CONFIG_SYS_ROMBOOT */
 
 	/* Return to U-Boot via saved link register */
 	mov	pc, lr
diff -aurN u-boot-2016.05.orig/arch/arm/mach-orion5x/Makefile u-boot-2016.05-88fxxxx/arch/arm/mach-orion5x/Makefile
--- u-boot-2016.05.orig/arch/arm/mach-orion5x/Makefile	2016-05-16 07:40:32.000000000 -0700
+++ u-boot-2016.05-88fxxxx/arch/arm/mach-orion5x/Makefile	2016-05-21 11:33:32.840226195 -0700
@@ -26,3 +26,5 @@
 
 endif
 endif
+
+obj-$(CONFIG_USB_EHCI_MARVELL)	+= usb.o
diff -aurN u-boot-2016.05.orig/arch/arm/mach-orion5x/usb.c u-boot-2016.05-88fxxxx/arch/arm/mach-orion5x/usb.c
--- u-boot-2016.05.orig/arch/arm/mach-orion5x/usb.c	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/arch/arm/mach-orion5x/usb.c	2016-05-21 11:33:32.840226195 -0700
@@ -0,0 +1,138 @@
+/*
+ * (C) Copyright 2012
+ * Steve Shih <shihsung@gmail.com>
+ * Derived from u-boot-1.1.1_buf109
+ *
+ * Copyright (C) Marvell International Ltd. and its affiliates
+ *
+ * SPDX-License-Identifier:	GPL-2.0+
+ */
+
+#include <common.h>
+#include <asm/io.h>
+#include <asm/arch/cpu.h>
+
+void orion5x_usb_workaround(unsigned long port_base)
+{
+	u32 reg, timeout, did, rev;
+
+	/* clear interrupt cause and mask registers */
+	writel(0, port_base + 0x310);
+	writel(0, port_base + 0x314);
+
+	/* reset controller */
+	writel(readl(port_base + 0x140) | 0x00000002, port_base + 0x140);
+	for (timeout = 10000; timeout; timeout--) {
+		if (!(readl(port_base + 0x140) & 0x00000002))
+			break;
+		mdelay(1);
+	}
+
+	/******** Update USB PHY configuration ********/
+
+	/* The new register 0x360 USB 2.0 IPG Metal Fix Register
+	 * doesn't exist in the following chip revisions:
+	 * Orion1 B1 (id=0x5181, rev=3) and before
+	 * Orion1-VoIP A0 (id=0x5181, rev=8)
+	 * Orion1-NAS A1 (id=0x5182, rev=1) and before
+	 * Orion2 B0 (id=0x5281, rev=1) and before
+	 */
+	did = orion5x_device_id();
+	rev = orion5x_device_rev();
+	if (!(((did == 0x5181) && ((rev <= 3) || (rev == 8))) ||
+	      ((did == 0x5182) && (rev <= 1)) ||
+	      ((did == 0x5281) && (rev <= 1)))) {
+		/* Change value of new register 0x360 */
+		reg = readl(port_base + 0x360);
+
+		/* Bit[14:8] (IPG for non Start of Frame packets) = 0xc */
+		reg &= ~(0x7f << 8);
+		reg |= (0xc << 8);
+
+		writel(reg, port_base + 0x360);
+	}
+
+	/******** USB 2.0 Power Control Register 0x400 ********/
+	reg = readl(port_base + 0x400);
+
+	/* Bit[7:6] (BG_VSEL) = 0x1 */
+	reg &= ~(0x3 << 6);
+	reg |= (0x1 << 6);
+
+	writel(reg, port_base + 0x400);
+
+	/******** USB PHY Tx Control Register 0x420 ********/
+	reg = readl(port_base + 0x420);
+
+	if ((did == 0x5181) && (rev <= 1)) {
+		/* For Orion1 A1/A0 rev: Bit[21] (TXDATA_BLOCK_EN) = 0 */
+		reg &= ~(1 << 21);
+	} else {
+		reg |= (1 << 21);
+	}
+
+	/* Force Auto calibration to 0x8 */
+
+	/* Bit[13] (REG_EXT_RCAL_EN) = 0x1 */
+	reg |= (1 << 13);
+
+	/* Bit[6:3] (IMP_CAL) = 0x8 */
+	reg &= ~(0xf << 3);
+	reg |= (0x8 << 3);
+
+	writel(reg, port_base + 0x420);
+
+	/******** USB PHY Rx Control Register 0x430 ********/
+	reg = readl(port_base + 0x430);
+
+	/* Bit[9:8] (DISCON_THRESHOLD) */
+	/*
+	 * Orion1-A0/A1/B0 = 0x3
+	 * Orion2-A0 = 0x2
+	 * Orion1-B1 and Orion2-B0 later = 0x0
+	 */
+	reg &= ~(0x3 << 8);
+	if ((did == 0x5181) && (rev <= 2)) {
+		reg |= (0x3 << 8);
+	} else if ((did == 0x5281) && (rev == 0)) {
+		reg |= (0x2 << 8);
+	}
+
+	/* Bit[21] (CDR_FASTLOCK_EN) = 0x0 */
+	reg &= ~(1 << 21);
+
+	/* Bit[27:26] (EDGE_DET_SEL) = 0x0 */
+	reg &= ~(0x3 << 26);
+
+	/* Bit[31:30] (RXDATA_BLOCK_LENGTH) = 0x3 */
+	reg |= (0x3 << 30);
+
+	/* Bit[7:4] (SQ_THRESH) = 0x1 */
+	reg &= ~(0xf << 4);
+	reg |= (0x1 << 4);
+
+	writel(reg, port_base + 0x430);
+
+	/******** USB PHY IVREF Control Register 0x440 ********/
+	reg = readl(port_base + 0x440);
+
+	/* Bit[1:0] (PLLVDD12) = 0x2 */
+	reg &= ~(0x3 << 0);
+	reg |= (0x2 << 0);
+
+	/* Bit[5:4] (RXVDD) = 0x3 */
+	reg |= (0x3 << 4);
+
+	/* Bit[19] (Reserved) = 0x0 */
+	reg &= ~(0x1 << 19);
+
+	writel(reg, port_base + 0x440);
+
+	/******** USB PHY TEST GROUP CONTROL Register: 0x450 ********/
+	reg = readl(port_base + 0x450);
+
+	/* Bit[15] (REG_FIFO_SQ_RST) = 0x0 */
+	reg &= ~(0x1 << 15);
+
+	writel(reg, port_base + 0x450);
+}
diff -aurN u-boot-2016.05.orig/board/buffalo/ls400/kwbimage.cfg u-boot-2016.05-88fxxxx/board/buffalo/ls400/kwbimage.cfg
--- u-boot-2016.05.orig/board/buffalo/ls400/kwbimage.cfg	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/board/buffalo/ls400/kwbimage.cfg	2016-05-21 11:33:32.840226195 -0700
@@ -0,0 +1,15 @@
+#
+# (C) Copyright 2016
+# Steve Shih <shihsung@gmail.com>
+#
+# SPDX-License-Identifier:	GPL-2.0+
+#
+
+# Armada 370 uses version 1 image format
+VERSION		1
+
+# Boot device
+BOOT_FROM	spi
+
+# DDR3 and SerDes setup binary header
+BINARY	spl/u-boot-spl.bin	0000005b	00000068
diff -aurN u-boot-2016.05.orig/board/buffalo/ls400/ls400.c u-boot-2016.05-88fxxxx/board/buffalo/ls400/ls400.c
--- u-boot-2016.05.orig/board/buffalo/ls400/ls400.c	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/board/buffalo/ls400/ls400.c	2016-05-21 11:33:32.840226195 -0700
@@ -0,0 +1,577 @@
+/*
+ * (C) Copyright 2014, 2016
+ * Steve Shih <shihsung@gmail.com>
+ *
+ * SPDX-License-Identifier:	GPL-2.0+
+ */
+
+#include <common.h>
+#include <console.h>
+#include <dm.h>
+#include <netdev.h>
+#include <miiphy.h>
+#include <pci.h>
+#include <usb.h>
+#include <asm/io.h>
+#include <asm/gpio.h>
+#include <asm/arch/cpu.h>
+#include "../arch/arm/mach-mvebu/serdes/a370/serdes.h"
+
+#define LS400				"ls400"
+
+#define HD_POWERUP_DELAY		1000
+#define HD_SPINUP_DELAY			10000
+
+#define GPIO_POWER_ON			1
+#define GPIO_POWER_OFF			0
+
+#define GPIO_FAN_PIN0			13
+#define GPIO_FAN_PIN1			14
+
+#define GPIO_FAN_OFF			3
+#define GPIO_FAN_LO			2
+#define GPIO_FAN_MED			1
+#define GPIO_FAN_HI			0
+
+#define POWER_BUTTON_PIN		15
+#define FUNCTION_BUTTON_PIN		16
+
+#if defined(CONFIG_TARGET_LS420)
+#define GPIO_USB_PWR_PIN		5
+#define GPIO_HD0_PWR_PIN		8
+#define GPIO_HD1_PWR_PIN		9
+
+#define GPIO_HD0_GONE_PIN		11
+#define GPIO_HD1_GONE_PIN		12
+#elif defined(CONFIG_TARGET_LS440)
+#define GPIO_HD0_PWR_PIN		8
+#define GPIO_HD1_PWR_PIN		7
+#define GPIO_HD2_PWR_PIN		52
+#define GPIO_HD3_PWR_PIN		49
+
+#define GPIO_HD0_GONE_PIN		10
+#define GPIO_HD1_GONE_PIN		11
+#define GPIO_HD2_GONE_PIN		50
+#define GPIO_HD3_GONE_PIN		51
+
+#define USB_HC_LENGTH(v)		((v) & 0xff)
+#define USB_CMD_RESET			(1 << 1)
+#endif
+
+DECLARE_GLOBAL_DATA_PTR;
+
+#ifdef CONFIG_SPL_BUILD
+static struct serdes_cfg ls400_serdes_cfg[A370_SERDES_MAX] = {
+	{ SERDES_SEL_PCIE,	SERDES_OPT_SPD_HI },
+#if defined(CONFIG_TARGET_LS420)
+	{ SERDES_SEL_NONE,	SERDES_OPT_SPD_LO },
+	{ SERDES_SEL_SATA,	SERDES_OPT_SPD_HI },
+	{ SERDES_SEL_SATA,	SERDES_OPT_SPD_HI }
+#elif defined(CONFIG_TARGET_LS440)
+	{ SERDES_SEL_PCIE,	SERDES_OPT_SPD_HI },
+	{ SERDES_SEL_NONE,	SERDES_OPT_SPD_LO },
+	{ SERDES_SEL_NONE,	SERDES_OPT_SPD_LO }
+#endif
+};
+
+struct serdes_cfg* board_serdes_cfg (void)
+{
+	return ls400_serdes_cfg;
+}
+
+int board_early_init_f (void)
+{
+	u32 out;
+
+	/* disable clocks for unused devices */
+	writel(LS400_PM_CLK_GATING_CTRL, MVEBU_REGISTER(0x18220));
+
+	/* Disable TDM clock if TDM clock gate is disabled */
+	if (!(LS400_PM_CLK_GATING_CTRL & 0x02000000))
+		writel(0, MVEBU_REGISTER(0x18770));
+
+	/* init GPIO */
+	writel(LS400_GPIO_LO_IN_POL, MVEBU_GPIO0_BASE + 0x0c);
+	out = LS400_GPIO_LO_OUT_VAL & ~LS400_GPIO_LO_OUT_KEEP;
+	out |= readl(MVEBU_GPIO0_BASE + 0x10) & LS400_GPIO_LO_OUT_KEEP;
+	writel(out, MVEBU_GPIO0_BASE + 0x00);
+	writel(LS400_GPIO_LO_OUT_ENA, MVEBU_GPIO0_BASE + 0x04);
+	writel(LS400_GPIO_LO_BLINK_SEL, MVEBU_GPIO0_BASE + 0x20);
+	writel(LS400_GPIO_LO_BLINK_ENA, MVEBU_GPIO0_BASE + 0x08);
+
+	writel(LS400_GPIO_MID_IN_POL, MVEBU_GPIO1_BASE + 0x0c);
+#if defined(CONFIG_TARGET_LS420)
+	out = LS400_GPIO_MID_OUT_VAL;
+#elif defined(CONFIG_TARGET_LS440)
+	out = LS400_GPIO_MID_OUT_VAL & ~LS400_GPIO_MID_OUT_KEEP;
+	out |= readl(MVEBU_GPIO1_BASE + 0x10) & LS400_GPIO_MID_OUT_KEEP;
+#endif
+	writel(out, MVEBU_GPIO1_BASE + 0x00);
+	writel(LS400_GPIO_MID_OUT_ENA, MVEBU_GPIO1_BASE + 0x04);
+	writel(LS400_GPIO_MID_BLINK_SEL, MVEBU_GPIO1_BASE + 0x20);
+	writel(LS400_GPIO_MID_BLINK_ENA, MVEBU_GPIO1_BASE + 0x08);
+
+	writel(LS400_GPIO_HI_IN_POL, MVEBU_GPIO2_BASE + 0x0c);
+	writel(LS400_GPIO_HI_OUT_VAL, MVEBU_GPIO2_BASE + 0x00);
+	writel(LS400_GPIO_HI_OUT_ENA, MVEBU_GPIO2_BASE + 0x04);
+	writel(LS400_GPIO_HI_BLINK_SEL, MVEBU_GPIO2_BASE + 0x20);
+	writel(LS400_GPIO_HI_BLINK_ENA, MVEBU_GPIO2_BASE + 0x08);
+
+	/* init MPP */
+	writel(LS400_MPP_0_7, MVEBU_MPP_BASE + 0x00);
+	writel(LS400_MPP_8_15, MVEBU_MPP_BASE + 0x04);
+	writel(LS400_MPP_16_23, MVEBU_MPP_BASE + 0x08);
+	writel(LS400_MPP_24_31, MVEBU_MPP_BASE + 0x0c);
+	writel(LS400_MPP_32_39, MVEBU_MPP_BASE + 0x10);
+	writel(LS400_MPP_40_47, MVEBU_MPP_BASE + 0x14);
+	writel(LS400_MPP_48_55, MVEBU_MPP_BASE + 0x18);
+	writel(LS400_MPP_56_63, MVEBU_MPP_BASE + 0x1c);
+	writel(LS400_MPP_64_65, MVEBU_MPP_BASE + 0x20);
+
+	return 0;
+}
+#else
+int board_early_init_f (void)
+{
+	return 0;
+}
+
+int board_init (void)
+{
+	int rc;
+	int val;
+	int hd0, hd1;
+#if defined(CONFIG_TARGET_LS440)
+	int hd2, hd3;
+#endif
+
+#if defined(CONFIG_TARGET_LS420)
+	/* power up USB */
+	rc = gpio_request(GPIO_USB_PWR_PIN, LS400);
+	if (rc) {
+		printf("%s: can't get USB power pin\n", __FUNCTION__);
+		return rc;
+	}
+	gpio_direction_output(GPIO_USB_PWR_PIN, GPIO_POWER_ON);
+#endif
+
+	/* keep the current power state for hard disk 1 */
+	rc = gpio_request(GPIO_HD0_PWR_PIN, LS400);
+	if (rc) {
+		printf("%s: can't get HD0 power pin\n", __FUNCTION__);
+		return rc;
+	}
+	gpio_direction_input(GPIO_HD0_PWR_PIN);
+	val = gpio_get_value(GPIO_HD0_PWR_PIN);
+	gpio_direction_output(GPIO_HD0_PWR_PIN, val);
+
+	/* keep the current power state for hard disk 2 */
+	rc = gpio_request(GPIO_HD1_PWR_PIN, LS400);
+	if (rc) {
+		printf("%s: can't get HD1 power pin\n", __FUNCTION__);
+		return rc;
+	}
+	gpio_direction_input(GPIO_HD1_PWR_PIN);
+	val = gpio_get_value(GPIO_HD1_PWR_PIN);
+	gpio_direction_output(GPIO_HD1_PWR_PIN, val);
+
+#if defined(CONFIG_TARGET_LS440)
+	/* keep the current power state for hard disk 3 */
+	rc = gpio_request(GPIO_HD2_PWR_PIN, LS400);
+	if (rc) {
+		printf("%s: can't get HD2 power pin\n", __FUNCTION__);
+		return rc;
+	}
+	gpio_direction_input(GPIO_HD2_PWR_PIN);
+	val = gpio_get_value(GPIO_HD2_PWR_PIN);
+	gpio_direction_output(GPIO_HD2_PWR_PIN, val);
+
+	/* keep the current power state for hard disk 4 */
+	rc = gpio_request(GPIO_HD3_PWR_PIN, LS400);
+	if (rc) {
+		printf("%s: can't get HD3 power pin\n", __FUNCTION__);
+		return rc;
+	}
+	gpio_direction_input(GPIO_HD3_PWR_PIN);
+	val = gpio_get_value(GPIO_HD3_PWR_PIN);
+	gpio_direction_output(GPIO_HD3_PWR_PIN, val);
+#endif
+
+	rc = gpio_request(GPIO_HD0_GONE_PIN, LS400);
+	if (rc) {
+		printf("%s: can't get HD0 missing pin\n", __FUNCTION__);
+		return rc;
+	}
+	gpio_direction_input(GPIO_HD0_GONE_PIN);
+
+	rc = gpio_request(GPIO_HD1_GONE_PIN, LS400);
+	if (rc) {
+		printf("%s: can't get HD1 missing pin\n", __FUNCTION__);
+		return rc;
+	}
+	gpio_direction_input(GPIO_HD1_GONE_PIN);
+
+#if defined(CONFIG_TARGET_LS440)
+	rc = gpio_request(GPIO_HD2_GONE_PIN, LS400);
+	if (rc) {
+		printf("%s: can't get HD2 missing pin\n", __FUNCTION__);
+		return rc;
+	}
+	gpio_direction_input(GPIO_HD2_GONE_PIN);
+
+	rc = gpio_request(GPIO_HD3_GONE_PIN, LS400);
+	if (rc) {
+		printf("%s: can't get HD3 missing pin\n", __FUNCTION__);
+		return rc;
+	}
+	gpio_direction_input(GPIO_HD3_GONE_PIN);
+#endif
+
+	rc = gpio_request(GPIO_FAN_PIN0, LS400);
+	if (rc) {
+		printf("%s: can't get FAN pin 0\n", __FUNCTION__);
+		return rc;
+	}
+	rc = gpio_request(GPIO_FAN_PIN1, LS400);
+	if (rc) {
+		printf("%s: can't get FAN pin 1\n", __FUNCTION__);
+		return rc;
+	}
+	gpio_direction_output(GPIO_FAN_PIN0, GPIO_FAN_MED & 1);
+	gpio_direction_output(GPIO_FAN_PIN1, GPIO_FAN_MED & 2);
+
+	rc = gpio_request(POWER_BUTTON_PIN, LS400);
+	if (rc) {
+		printf("%s: can't get power button pin\n", __FUNCTION__);
+		return rc;
+	}
+	gpio_direction_input(POWER_BUTTON_PIN);
+
+	rc = gpio_request(FUNCTION_BUTTON_PIN, LS400);
+	if (rc) {
+		printf("%s: can't get function button pin\n", __FUNCTION__);
+		return rc;
+	}
+	gpio_direction_input(FUNCTION_BUTTON_PIN);
+
+	/* power on the hard disk if present otherwise power off */
+	hd0 = !gpio_get_value(GPIO_HD0_GONE_PIN);
+	hd1 = !gpio_get_value(GPIO_HD1_GONE_PIN);
+#if defined(CONFIG_TARGET_LS440)
+	hd2 = !gpio_get_value(GPIO_HD2_GONE_PIN);
+	hd3 = !gpio_get_value(GPIO_HD3_GONE_PIN);
+#endif
+	if (hd0)
+		gpio_set_value(GPIO_HD0_PWR_PIN, GPIO_POWER_ON);
+	else
+		gpio_set_value(GPIO_HD0_PWR_PIN, GPIO_POWER_OFF);
+	if (hd0 && hd1)
+		mdelay(HD_POWERUP_DELAY);
+	if (hd1)
+		gpio_set_value(GPIO_HD1_PWR_PIN, GPIO_POWER_ON);
+	else
+		gpio_set_value(GPIO_HD1_PWR_PIN, GPIO_POWER_OFF);
+#if defined(CONFIG_TARGET_LS420)
+	if (hd0 || hd1)
+		mdelay(HD_SPINUP_DELAY);
+#elif defined(CONFIG_TARGET_LS440)
+	if ((hd0 || hd1) && hd2)
+		mdelay(HD_POWERUP_DELAY);
+	if (hd2)
+		gpio_set_value(GPIO_HD2_PWR_PIN, GPIO_POWER_ON);
+	else
+		gpio_set_value(GPIO_HD2_PWR_PIN, GPIO_POWER_OFF);
+	if ((hd0 || hd1 || hd2) && hd3)
+		mdelay(HD_POWERUP_DELAY);
+	if (hd3)
+		gpio_set_value(GPIO_HD3_PWR_PIN, GPIO_POWER_ON);
+	else
+		gpio_set_value(GPIO_HD3_PWR_PIN, GPIO_POWER_OFF);
+	if (hd0 || hd1 || hd2 || hd3)
+		mdelay(HD_SPINUP_DELAY);
+#endif
+
+	gd->bd->bi_boot_params = gd->bd->bi_dram[0].start + CONFIG_SYS_PARAMS_PHYS;
+
+	return 0;
+}
+
+/* don't use netconsole until eth initialized */
+int overwrite_console (void)
+{
+	return 1;
+}
+
+int board_late_init (void)
+{
+	char *ethaddr = getenv("eth1addr");
+
+	if (!ethaddr) {
+		ethaddr = getenv("ethaddr");
+		if (!ethaddr)
+			ethaddr = __stringify(CONFIG_ETHADDR);
+	}
+	setenv("ethaddr", ethaddr);
+
+	return 0;
+}
+
+int last_stage_init (void)
+{
+#if defined(CONFIG_TARGET_LS440) && defined(CONFIG_PCI) && defined(CONFIG_USB_EHCI)
+	struct pci_device_id xhci_pci_ids[] = {
+		{0x1912, 0x0015},
+		{}
+	};
+	pci_dev_t pdev;
+	u8 *hccr, *hcor;
+#endif
+#ifdef CONFIG_NETCONSOLE
+	bool nc = false;
+	char *stdinname = getenv("stdin");
+	char *stdoutname = getenv("stdout");
+	char *stderrname = getenv("stderr");
+	struct udevice *eth;
+	u16 phyaddr;
+#endif
+
+#if defined(CONFIG_TARGET_LS440) && defined(CONFIG_PCI) && defined(CONFIG_USB_EHCI)
+	/*
+	 * It seems EHCI port is powered only after XHCI reset!
+	 */
+	pdev = pci_find_devices(xhci_pci_ids, 0);;
+	if (pdev > 0) {
+		hccr = pci_map_bar(pdev, PCI_BASE_ADDRESS_0, PCI_REGION_MEM);
+		hcor = hccr + USB_HC_LENGTH(readl(hccr));
+		setbits_le32(hcor, USB_CMD_RESET);
+	}
+#endif
+
+#ifdef CONFIG_NETCONSOLE
+	if (!strcmp(stdinname, "nc") || !strcmp(stdoutname, "nc") ||
+	    !strcmp(stdoutname, "nc"))
+		nc = true;
+
+	/* swap nc and serial if function switch pressed */
+	if (!gpio_get_value(FUNCTION_BUTTON_PIN)) {
+		if (!nc) {
+			stdinname = "nc";
+			stdoutname = "nc";
+			stderrname = "nc";
+		} else {
+			stdinname = "serial";
+			stdoutname = "serial";
+			stderrname = "serial";
+		}
+		nc = !nc;
+	}
+
+	if (nc) {
+		eth_init();
+
+		/* switch to serial if not linked up */
+		eth = eth_get_dev();
+		if (!eth || miiphy_read(eth->name, 0xEE, 0xEE, &phyaddr) ||
+		    !miiphy_link(eth->name, phyaddr)) {
+			nc = false;
+			stdinname = "serial";
+			stdoutname = "serial";
+			stderrname = "serial";
+		}
+	}
+
+	console_assign(stdin, stdinname);
+	console_assign(stdout, stdoutname);
+	console_assign(stderr, stderrname);
+
+	if (nc)
+		net_start_again();
+#endif
+
+	return 0;
+}
+
+void reset_phy (void)
+{
+	struct udevice *eth;
+	u16 phyaddr;
+	u16 reg;
+
+	eth = eth_get_dev();
+	if (!eth) {
+		printf("%s: can't get ETH device\n", __FUNCTION__);
+		return;
+	}
+
+	if (miiphy_read(eth->name, 0xEE, 0xEE, &phyaddr)) {
+		printf("%s: can't read PHY device address\n", __FUNCTION__);
+		return;
+	}
+
+	miiphy_write(eth->name, phyaddr, 22, 3);
+	/* set status[0] polarity to high active */
+	miiphy_read(eth->name, phyaddr, 17, &reg);
+	reg &= ~0x000f;
+	reg |= 0x0001;
+	miiphy_write(eth->name, phyaddr, 17, reg);
+	/* force INIT active */
+	miiphy_read(eth->name, phyaddr, 16, &reg);
+	reg &= ~0x0fff;
+	reg |= 0x091e;
+	miiphy_write(eth->name, phyaddr, 16, reg);
+	miiphy_write(eth->name, phyaddr, 22, 0);
+
+	/* reset the phy */
+	miiphy_reset(eth->name, phyaddr);
+}
+
+unsigned long do_go_exec (ulong (*entry)(int, char* const[]), int argc,
+				char* const argv[])
+{
+	eth_halt();
+	usb_stop();
+
+	cleanup_before_linux();
+
+	return entry(argc, argv);
+}
+
+void reset_misc (void)
+{
+	struct udevice *eth;
+	u16 phyaddr;
+	u16 reg;
+
+	eth = eth_get_dev();
+	if (!eth) {
+		printf("%s: can't get ETH device\n", __FUNCTION__);
+		return;
+	}
+
+	if (miiphy_read(eth->name, 0xEE, 0xEE, &phyaddr)) {
+		printf("%s: can't read PHY device address\n", __FUNCTION__);
+		return;
+	}
+
+	miiphy_write(eth->name, phyaddr, 0x16, 0x0003);
+	miiphy_read(eth->name, phyaddr, 0x10, &reg);
+	reg &= ~0x0f00;
+	reg |= 0x0900;
+	miiphy_write(eth->name, phyaddr, 0x10, reg);
+	miiphy_write(eth->name, phyaddr, 0x16, 0x0000);
+
+	mdelay(10);
+}
+
+int do_poweroff (cmd_tbl_t *cmdtp, int flag, int argc,
+			char* const argv[])
+{
+	struct udevice *eth;
+	u16 phyaddr;
+	u16 reg;
+
+	eth = eth_get_dev();
+	if (!eth) {
+		printf("%s: can't get ETH device\n", __FUNCTION__);
+		return -1;
+	}
+
+	if (miiphy_read(eth->name, 0xEE, 0xEE, &phyaddr)) {
+		printf("%s: can't read PHY device address\n", __FUNCTION__);
+		return -1;
+	}
+
+	miiphy_write(eth->name, phyaddr, 0x16, 0x0003);
+	miiphy_read(eth->name, phyaddr, 0x10, &reg);
+	reg &= ~0x0f00;
+	reg |= 0x0800;
+	miiphy_write(eth->name, phyaddr, 0x10, reg);
+	miiphy_write(eth->name, phyaddr, 0x16, 0x0000);
+
+	mdelay(10);
+
+	reset_cpu(0);
+
+	return CMD_RET_SUCCESS;
+}
+
+static int do_power (cmd_tbl_t *cmdtp, int flag, int argc, char* const argv[])
+{
+	if (argc == 3) {
+		u32 pin, val;
+
+		if (!strcasecmp(argv[1], "on"))
+			val = GPIO_POWER_ON;
+		else if (!strcasecmp(argv[1], "off"))
+			val = GPIO_POWER_OFF;
+		else
+			return CMD_RET_USAGE;
+
+		if (!strcasecmp(argv[2], "hd1"))
+			pin = GPIO_HD0_PWR_PIN;
+		else if (!strcasecmp(argv[2], "hd2"))
+			pin = GPIO_HD1_PWR_PIN;
+#if defined(CONFIG_TARGET_LS420)
+		else if (!strcasecmp(argv[2], "usb"))
+			pin = GPIO_USB_PWR_PIN;
+#elif defined(CONFIG_TARGET_LS440)
+		else if (!strcasecmp(argv[2], "hd3"))
+			pin = GPIO_HD2_PWR_PIN;
+		else if (!strcasecmp(argv[2], "hd4"))
+			pin = GPIO_HD3_PWR_PIN;
+#endif
+		else
+			return CMD_RET_USAGE;
+
+		gpio_set_value(pin, val);
+
+		return CMD_RET_SUCCESS;
+	}
+
+	return CMD_RET_USAGE;
+}
+
+#if defined(CONFIG_TARGET_LS420)
+U_BOOT_CMD(power, 3, 0, do_power, "power on/off disk/USB",
+	"<on|off> hd1    - turn on/off hard disk 1 power\n"
+	"power <on|off> hd2    - turn on/off hard disk 2 power\n"
+	"power <on|off> usb    - turn on/off USB power");
+#elif defined(CONFIG_TARGET_LS440)
+U_BOOT_CMD(power, 3, 0, do_power, "power on/off disk/USB",
+	"<on|off> hd1    - turn on/off hard disk 1 power\n"
+	"power <on|off> hd2    - turn on/off hard disk 2 power\n"
+	"power <on|off> hd3    - turn on/off hard disk 3 power\n"
+	"power <on|off> hd4    - turn on/off hard disk 4 power");
+#endif
+
+static int do_fan (cmd_tbl_t *cmdtp, int flag, int argc, char* const argv[])
+{
+	if (argc == 2) {
+		u32 val;
+
+		if (!strcasecmp(argv[1], "off"))
+			val = GPIO_FAN_OFF;
+		else if (!strcasecmp(argv[1], "low"))
+			val = GPIO_FAN_LO;
+		else if (!strcasecmp(argv[1], "med"))
+			val = GPIO_FAN_MED;
+		else if (!strcasecmp(argv[1], "high"))
+			val = GPIO_FAN_HI;
+		else
+			return CMD_RET_USAGE;
+
+		gpio_set_value(GPIO_FAN_PIN0, val & 1);
+		gpio_set_value(GPIO_FAN_PIN1, val & 2);
+
+		return CMD_RET_SUCCESS;
+	}
+
+	return CMD_RET_USAGE;
+}
+
+U_BOOT_CMD(fan, 2, 0, do_fan, "fan speed control",
+	"off       - turn off fan\n"
+	"fan low       - set low fan speed\n"
+	"fan med       - set medium fan speed\n"
+	"fan high      - set high fan speed");
+#endif
diff -aurN u-boot-2016.05.orig/board/buffalo/ls400/MAINTAINERS u-boot-2016.05-88fxxxx/board/buffalo/ls400/MAINTAINERS
--- u-boot-2016.05.orig/board/buffalo/ls400/MAINTAINERS	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/board/buffalo/ls400/MAINTAINERS	2016-05-21 11:33:32.840226195 -0700
@@ -0,0 +1,7 @@
+LS400 BOARD
+M:	Steve Shih <shihsung@gmail.com>
+S:	Maintained
+F:	board/buffalo/ls400/
+F:	include/configs/ls400.h
+F:	configs/ls420_defconfig
+F:	configs/ls440_defconfig
diff -aurN u-boot-2016.05.orig/board/buffalo/ls400/Makefile u-boot-2016.05-88fxxxx/board/buffalo/ls400/Makefile
--- u-boot-2016.05.orig/board/buffalo/ls400/Makefile	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/board/buffalo/ls400/Makefile	2016-05-21 11:33:32.840226195 -0700
@@ -0,0 +1,8 @@
+#
+# (C) Copyright 2014, 2016
+# Steve Shih <shihsung@gmail.com>
+#
+# SPDX-License-Identifier:	GPL-2.0+
+#
+
+obj-$(CONFIG_LS400) += ls400.o
diff -aurN u-boot-2016.05.orig/board/buffalo/lsgl/config.mk u-boot-2016.05-88fxxxx/board/buffalo/lsgl/config.mk
--- u-boot-2016.05.orig/board/buffalo/lsgl/config.mk	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/board/buffalo/lsgl/config.mk	2016-05-21 11:33:32.840226195 -0700
@@ -0,0 +1,12 @@
+#
+# (C) Copyright 2012
+# Steve Shih <shihsung@gmail.com>
+#
+# SPDX-License-Identifier:	GPL-2.0+
+#
+
+ifdef CONFIG_SYS_ROMBOOT
+CONFIG_BOARD_SIZE_LIMIT = 0x0003f000
+CONFIG_SYS_LDSCRIPT = $(TOPDIR)/board/$(BOARDDIR)/romboot.lds
+OBJCOPYFLAGS += -j .romboot
+endif
diff -aurN u-boot-2016.05.orig/board/buffalo/lsgl/Kconfig u-boot-2016.05-88fxxxx/board/buffalo/lsgl/Kconfig
--- u-boot-2016.05.orig/board/buffalo/lsgl/Kconfig	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/board/buffalo/lsgl/Kconfig	2016-05-21 11:33:32.840226195 -0700
@@ -0,0 +1,12 @@
+if TARGET_KUROPRO || TARGET_LSGL
+
+config SYS_BOARD
+	default "lsgl"
+
+config SYS_VENDOR
+	default "buffalo"
+
+config SYS_CONFIG_NAME
+	default "lsgl"
+
+endif
diff -aurN u-boot-2016.05.orig/board/buffalo/lsgl/lsgl.c u-boot-2016.05-88fxxxx/board/buffalo/lsgl/lsgl.c
--- u-boot-2016.05.orig/board/buffalo/lsgl/lsgl.c	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/board/buffalo/lsgl/lsgl.c	2016-05-21 11:33:32.840226195 -0700
@@ -0,0 +1,234 @@
+/*
+ * (C) Copyright 2012, 2016
+ * Steve Shih <shihsung@gmail.com>
+ *
+ * SPDX-License-Identifier:	GPL-2.0+
+ */
+
+#include <common.h>
+#include <console.h>
+#include <mtd/cfi_flash.h>
+#include <netdev.h>
+#include <miiphy.h>
+#include <usb.h>
+#include "micon.h"
+
+DECLARE_GLOBAL_DATA_PTR;
+
+int board_early_init_f (void)
+{
+	micon_init();
+	micon_boot();
+
+	return 0;
+}
+
+int board_init (void)
+{
+	gd->bd->bi_boot_params = gd->bd->bi_dram[0].start + CONFIG_SYS_PARAMS_PHYS;
+
+	mdelay(500);
+
+	return 0;
+}
+
+/* don't use netconsole until eth initialized */
+int overwrite_console (void)
+{
+	return 1;
+}
+
+int board_late_init (void)
+{
+	char *ethaddr = getenv("ethaddr");
+
+	if (!ethaddr)
+		setenv("ethaddr", __stringify(CONFIG_ETHADDR));
+
+	return 0;
+}
+
+int last_stage_init (void)
+{
+#ifdef CONFIG_NETCONSOLE
+	bool nc = false;
+	char *stdinname = getenv("stdin");
+	char *stdoutname = getenv("stdout");
+	char *stderrname = getenv("stderr");
+	struct eth_device *eth;
+	u16 phyaddr;
+#endif
+
+	micon_up();
+
+#ifdef CONFIG_NETCONSOLE
+	if (!strcmp(stdinname, "nc") || !strcmp(stdoutname, "nc") ||
+	    !strcmp(stdoutname, "nc"))
+		nc = true;
+
+	/*
+	 * swap nc and serial if init switch pressed and
+	 * read twice to make sure it's really pressed
+	 */
+	if (MICON_BUTTON_INIT(micon_buttons()) &&
+	    MICON_BUTTON_INIT(micon_buttons())) {
+		if (!nc) {
+			stdinname = "nc";
+			stdoutname = "nc";
+			stderrname = "nc";
+		} else {
+			stdinname = "serial";
+			stdoutname = "serial";
+			stderrname = "serial";
+		}
+		nc = !nc;
+	}
+
+	if (nc) {
+		eth_init();
+
+		/* switch to serial if not linked up */
+		eth = eth_get_dev();
+		if (!eth || miiphy_read(eth->name, 0xEE, 0xEE, &phyaddr) ||
+		    !miiphy_link(eth->name, phyaddr)) {
+			nc = false;
+			stdinname = "serial";
+			stdoutname = "serial";
+			stderrname = "serial";
+		}
+	}
+
+	console_assign(stdin, stdinname);
+	console_assign(stdout, stdoutname);
+	console_assign(stderr, stderrname);
+
+	if (nc)
+		net_start_again();
+#endif
+
+	return 0;
+}
+
+void enable_caches (void)
+{
+	icache_enable();
+	dcache_disable();
+}
+
+ulong board_flash_get_legacy (ulong base, int banknum, flash_info_t *info)
+{
+	ulong i;
+
+	if (banknum)
+		return 0;
+
+	/* SST 39VF020 256KB (64 uniform 4KB sectors) */
+	info->size = 0x00040000; /* 256K */
+	info->sector_count = CONFIG_SYS_MAX_FLASH_SECT;
+	info->flash_id = FLASH_MAN_CFI;
+	for (i = 0; i < info->sector_count; i++) {
+		info->start[i] = base + (i << 12);
+		info->protect[i] = 1;
+	}
+
+	info->portwidth = FLASH_CFI_8BIT;
+	info->chipwidth = FLASH_CFI_BY8;
+	info->buffer_size = 0;
+	info->erase_blk_tout = 25;	/* max sector erase time: 25 ms */
+	info->write_tout = 1;		/* max byte program time: 20 us */
+	info->buffer_write_tout = 0;
+	info->vendor = CFI_CMDSET_AMD_LEGACY;
+	info->cmd_reset = AMD_CMD_RESET;
+	info->cmd_erase_sector = AMD_CMD_ERASE_SECTOR;
+	info->interface = FLASH_CFI_X8;
+	info->legacy_unlock = 0;
+	info->manufacturer_id = (ushort)SST_MANUFACT;
+	info->device_id = (ushort)SST_ID_xF020;
+	info->device_id2 = 0;
+	info->ext_addr = 0;
+	info->cfi_version = 0;
+	info->cfi_offset = 0;
+	info->addr_unlock1 = 0x00005555;
+	info->addr_unlock2 = 0x00002aaa;
+	info->name = "SST 39VF020";
+
+	return 1;
+}
+
+int board_eth_init (bd_t *bis)
+{
+	struct eth_device *eth;
+	u16 phyaddr;
+	u16 id2, id3, reg;
+
+	if (mvgbe_initialize(bis))
+		return -1;
+
+	eth = eth_get_dev();
+	if (!eth) {
+		printf("%s: can't get ETH device\n", __FUNCTION__);
+		return -1;
+	}
+
+	if (miiphy_read(eth->name, 0xEE, 0xEE, &phyaddr)) {
+		printf("%s: can't get PHY device address\n", __FUNCTION__);
+		return -1;
+	}
+
+#ifdef CONFIG_SYS_RAMBOOT
+	/* workaround RAMBOOT netconsole issue loaded from BUFFALO U-Boot */
+	eth_halt();
+#endif
+
+	/* read PHY Ident 2 and 3 */
+	miiphy_read(eth->name, phyaddr, 2, &id2);
+	miiphy_read(eth->name, phyaddr, 3, &id3);
+	id3 &= 0xfff0;
+
+	/* Marvell 88E1111 (implies Linkstation Pro v1) */
+	if ((id2 == 0x0141) && (id3 == 0x0cc0)) {
+		/* PHY rx and tx delay */
+		miiphy_read(eth->name, phyaddr, 20, &reg);
+		reg |= 0x0082;
+		miiphy_write(eth->name, phyaddr, 20, reg);
+
+		/* Link and Activity LED (derived from PHY register dump) */
+		miiphy_write(eth->name, phyaddr, 24, 0x4111);
+
+		/* SATA Port 1 for Linkstation v1 */
+		ide_bus_offset[0] = ORION5X_SATA_PORT1_OFFSET;
+	/* Marvell 88E1118 (implies Linkstation Pro v2 or Kurobox-Pro) */
+	} else if ((id2 == 0x0141) && (id3 == 0x0e10)) {
+		/* Switch to page 3 */
+		miiphy_write(eth->name, phyaddr, 22, 3);
+
+		/* Link and Activity LED (derived from PHY register dump) */
+		miiphy_write(eth->name, phyaddr, 16, 0x0201);
+
+		/* switch back to page 0 */
+		miiphy_write(eth->name, phyaddr, 22, 0);
+	}
+
+	/* reset the phy */
+	miiphy_reset(eth->name, phyaddr);
+
+	return 0;
+}
+
+unsigned long do_go_exec (ulong (*entry)(int, char* const[]), int argc,
+				char* const argv[])
+{
+	eth_halt();
+	usb_stop();
+
+	cleanup_before_linux();
+
+	return entry(argc, argv);
+}
+
+int do_poweroff (cmd_tbl_t *cmdtp, int flag, int argc, char* const argv[])
+{
+	micon_off();
+
+	return CMD_RET_SUCCESS;
+}
diff -aurN u-boot-2016.05.orig/board/buffalo/lsgl/MAINTAINERS u-boot-2016.05-88fxxxx/board/buffalo/lsgl/MAINTAINERS
--- u-boot-2016.05.orig/board/buffalo/lsgl/MAINTAINERS	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/board/buffalo/lsgl/MAINTAINERS	2016-05-21 11:33:32.840226195 -0700
@@ -0,0 +1,10 @@
+LSGL BOARD
+M:	Steve Shih <shihsung@gmail.com>
+S:	Maintained
+F:	board/buffalo/lsgl/
+F:	include/configs/lsgl.h
+F:	configs/kuropro_defconfig
+F:	configs/kuropro_nand_defconfig
+F:	configs/kuropro_ram_defconfig
+F:	configs/lsgl_defconfig
+F:	configs/lsgl_ram_defconfig
diff -aurN u-boot-2016.05.orig/board/buffalo/lsgl/Makefile u-boot-2016.05-88fxxxx/board/buffalo/lsgl/Makefile
--- u-boot-2016.05.orig/board/buffalo/lsgl/Makefile	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/board/buffalo/lsgl/Makefile	2016-05-21 11:33:32.840226195 -0700
@@ -0,0 +1,10 @@
+#
+# (C) Copyright 2012
+# Steve Shih <shihsung@gmail.com>
+#
+# SPDX-License-Identifier:	GPL-2.0+
+#
+
+obj-y	:= $(BOARD).o micon.o
+
+obj-$(CONFIG_SYS_ROMBOOT) += romboot.o
diff -aurN u-boot-2016.05.orig/board/buffalo/lsgl/micon.c u-boot-2016.05-88fxxxx/board/buffalo/lsgl/micon.c
--- u-boot-2016.05.orig/board/buffalo/lsgl/micon.c	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/board/buffalo/lsgl/micon.c	2016-05-21 11:33:32.840226195 -0700
@@ -0,0 +1,351 @@
+/*
+ * (C) Copyright 2012
+ * Steve Shih <shihsung@gmail.com>
+ *
+ * SPDX-License-Identifier:	GPL-2.0+
+ */
+
+#include <common.h>
+#include <ns16550.h>
+#include <linux/ctype.h>
+#include "micon.h"
+
+#define MICON			((NS16550_t)ORION5X_UART1_BASE)
+#define MICON_DIV		(CONFIG_SYS_NS16550_CLK / 16 / 38400)
+#define MICON_LCR		(UART_LCR_WLS_8 | \
+				 UART_LCR_PEN | UART_LCR_EPS | \
+				 UART_LCR_STB)
+
+#define MICON_TIMEOUT		100
+#define MICON_MIN		4
+#define MICON_MAX		35
+
+#define MICON_READ		0x80
+#define MICON_LEN(c)		(((c) & 0x7f) + 3)
+
+#define MICON_HDR		0
+#define MICON_CMD		1
+#define MICON_DATA		2
+
+#define MICON_NOP		0xff
+
+#define MICON_BOOT_START	0x02
+#define MICON_BOOT_END		0x03
+#define MICON_DRAM_DATA_ERROR	0x04
+#define MICON_DRAM_ADDR_ERROR	0x05
+#define MICON_POWER_OFF		0x06
+#define MICON_DRAM_TEST		0x08
+#define MICON_SHUT_DOWN		0x0c
+#define MICON_SHUT_CANCEL	0x0d
+#define MICON_REBOOT		0x0e
+#define MICON_PHY_ERROR		0x28
+#define MICON_MOUNT_ERROR	0x29
+#define MICON_SYSTEM_ERROR	0x2a
+
+#define MICON_BUZ		0x30
+#define MICON_FAN		0x33
+#define MICON_WDT		0x35
+#define MICON_SW		0x36
+#define MICON_TEMP		0x37
+#define MICON_SPIN		0x38
+#define MICON_BRIGHT		0x3a
+#define MICON_HDD		0x3b
+#define MICON_STATUS		0x3c
+#define MICON_MODE		0x3e
+
+#define MICON_CTRL		0x50
+#define MICON_LED_ON		0x51	/* Power and Link LEDs */
+#define MICON_LED_BLINK		0x52	/* Power and Link LEDs */
+#define MICON_BUZ_FREQ		0x53
+#define MICON_LED		0x54	/* Info and Error LEDs */
+
+#define MICON_BUZ_SILENT	0x00
+#define MICON_BUZ_BOOT		0x01
+#define MICON_BUZ_ALARM		0x10
+
+#define MICON_FAN_STOP		0x00
+#define MICON_FAN_LOW		0x01
+#define MICON_FAN_MID		0x02
+#define MICON_FAN_HIGH		0x03
+
+#define MICON_LED_INFO_OFF	0x00
+#define MICON_LED_ERR_OFF	0x00
+#define MICON_LED_INFO_WARN	0x00
+#define MICON_LED_ERR_WARN	0x06
+
+static void micon_putc (char c)
+{
+	NS16550_putc(MICON, c);
+}
+
+static int micon_getc (void)
+{
+	int i;
+
+	for (i = 0; i < MICON_TIMEOUT; i++, mdelay(1))
+		if (NS16550_tstc(MICON))
+			break;
+
+	return NS16550_tstc(MICON) ? NS16550_getc(MICON) : -1;
+}
+
+static void micon_drain (void)
+{
+	while (micon_getc() > 0);
+}
+
+static void micon_tx (int len, char* buf)
+{
+	u8 sum;
+	int i;
+
+	for (sum = 0, i = 0; i < len; sum += buf[i++])
+		micon_putc(buf[i]);
+	NS16550_putc(MICON, -sum);
+	mdelay(1);
+}
+
+static int micon_rx (int len, char* buf)
+{
+	u8 sum;
+	int tmp, i;
+
+	if (len < MICON_MIN)
+		return -1;
+
+	if ((tmp = micon_getc()) < 0)
+		return -1;
+
+	if (MICON_LEN(tmp) > len || MICON_LEN(tmp) > MICON_MAX)
+		goto cleanup;
+
+	sum = buf[0] = tmp & 0xff;
+	for (i = 1, len = MICON_LEN(tmp); i < len; i++)
+		if ((tmp = micon_getc()) < 0)
+			goto cleanup;
+		else
+			sum += buf[i] = tmp & 0xff;
+
+	if (sum)
+		goto cleanup;
+
+	return len;
+
+cleanup:
+	micon_drain();
+	return -1;
+}
+
+static void micon_sync (void)
+{
+	int i;
+
+	for (i = 0; i < MICON_MAX; i++)
+		micon_putc(MICON_NOP);
+	micon_drain();
+}
+
+static void micon_cmd0 (bool read, char cmd)
+{
+	char tx[2] = {'\x00'};
+	char rx[MICON_MAX];
+
+	if (read)
+		tx[MICON_HDR] |= MICON_READ;
+	tx[MICON_CMD] = cmd;
+	micon_tx(sizeof(tx), tx);
+	if (!read)
+		micon_rx(sizeof(rx), rx);
+}
+
+static void micon_cmd1 (bool read, char cmd, char arg)
+{
+	char tx[3] = {'\x01'};
+	char rx[MICON_MAX];
+
+	if (read)
+		tx[MICON_HDR] |= MICON_READ;
+	tx[MICON_CMD] = cmd;
+	tx[MICON_DATA] = arg;
+	micon_tx(sizeof(tx), tx);
+	if (!read)
+		micon_rx(sizeof(rx), rx);
+}
+
+static void micon_cmd2 (bool read, char cmd, char arg0, char arg1)
+{
+	char tx[4] = {'\x02'};
+	char rx[MICON_MAX];
+
+	if (read)
+		tx[MICON_HDR] |= MICON_READ;
+	tx[MICON_CMD] = cmd;
+	tx[MICON_DATA] = arg0;
+	tx[MICON_DATA + 1] = arg1;
+	micon_tx(sizeof(tx), tx);
+	if (!read)
+		micon_rx(sizeof(rx), rx);
+}
+
+static void micon_shut (void)
+{
+	micon_cmd0(false, MICON_SHUT_DOWN);
+	micon_cmd1(false, MICON_BUZ, MICON_BUZ_SILENT);
+	micon_cmd2(false, MICON_LED, MICON_LED_INFO_OFF, MICON_LED_ERR_OFF);
+}
+
+void micon_init (void)
+{
+	MICON->ier = 0;
+	MICON->lcr = UART_LCR_DLAB | MICON_LCR;
+	MICON->dll = MICON_DIV & 0xff;
+	MICON->dlm = (MICON_DIV >> 8) & 0xff;
+	mdelay(1);
+	MICON->lcr = MICON_LCR;
+	MICON->mcr = UART_MCR_DTR | UART_MCR_RTS;
+	MICON->fcr = UART_FCR_FIFO_EN | UART_FCR_RXSR | UART_FCR_TXSR;
+
+	micon_sync();
+}
+
+void micon_boot (void)
+{
+	micon_cmd0(false, MICON_BOOT_START);
+	micon_cmd1(false, MICON_BUZ, MICON_BUZ_BOOT);
+}
+
+int micon_buttons (void)
+{
+	char rx[MICON_MAX];
+	int len;
+
+	micon_cmd0(true, MICON_SW);
+
+	len = micon_rx(sizeof(rx), rx);
+	if (len < 0) {
+		micon_sync();
+		return 0;
+	}
+
+	return rx[MICON_DATA];
+}
+
+bool micon_emergency (void)
+{
+	char cmd[] = {MICON_READ, MICON_MODE};
+	char ack[MICON_MAX];
+
+	micon_tx(sizeof(cmd), cmd);
+	if (micon_rx(sizeof(ack), ack) < 0)
+		return 0;
+
+	return ack[MICON_DATA];
+}
+
+void micon_hdd (bool on)
+{
+	micon_cmd1(false, MICON_HDD, on);
+}
+
+void micon_error (void)
+{
+	micon_cmd1(false, MICON_BUZ, MICON_BUZ_ALARM);
+	micon_cmd2(false, MICON_LED, MICON_LED_INFO_WARN, MICON_LED_ERR_WARN);
+}
+
+void micon_off (void)
+{
+	micon_shut();
+	micon_cmd0(false, MICON_POWER_OFF);
+}
+
+void micon_reboot (void)
+{
+	micon_shut();
+	micon_cmd0(false, MICON_REBOOT);
+}
+
+void micon_up (void)
+{
+	micon_cmd0(false, MICON_BOOT_END);
+	micon_cmd1(false, MICON_BUZ, MICON_BUZ_SILENT);
+	micon_cmd2(false, MICON_LED, MICON_LED_INFO_OFF, MICON_LED_ERR_OFF);
+	micon_cmd1(false, MICON_FAN, MICON_FAN_LOW);
+}
+
+static int do_micon (cmd_tbl_t *cmdtp, int flag, int argc, char* const argv[])
+{
+	char tx[MICON_MAX];
+	char rx[MICON_MAX];
+	int i, button, on, len = 0;
+
+	if (argc < 2 || argc > MICON_MAX + 1)
+		return CMD_RET_USAGE;
+
+	if (!strcasecmp(argv[1], "button")) {
+		button = micon_buttons();
+		printf("power: %s\ninit: %s\n",
+			MICON_BUTTON_POWER(button) ? "on" : "off",
+			MICON_BUTTON_INIT(button) ? "on" : "off");
+		return CMD_RET_SUCCESS;
+	} else if (!strcasecmp(argv[1], "error")) {
+		micon_error();
+		return CMD_RET_SUCCESS;
+	} else if (argc == 3 && !strcasecmp(argv[1], "hdd")) {
+		if (!strcasecmp(argv[2], "on"))
+			on = 1;
+		else if (!strcasecmp(argv[2], "off"))
+			on = 0;
+		else
+			return CMD_RET_USAGE;
+		micon_hdd(on);
+		return CMD_RET_SUCCESS;
+	} else if (!strcasecmp(argv[1], "mode")) {
+		printf("mode: %s\n",
+			micon_emergency() ? "emergency" : "normal");
+		return CMD_RET_SUCCESS;
+	} else if (!strcasecmp(argv[1], "off")) {
+		micon_off();
+		return CMD_RET_SUCCESS;
+	} else if (!strcasecmp(argv[1], "reboot")) {
+		micon_reboot();
+		return CMD_RET_SUCCESS;
+	} else if (!strcasecmp(argv[1], "sync")) {
+		micon_sync();
+		return CMD_RET_SUCCESS;
+	} else if (!strcasecmp(argv[1], "up")) {
+		micon_up();
+		return CMD_RET_SUCCESS;
+	} else if (argc > 2 && isxdigit(argv[1][0]) &&
+		   (argv[1][1] == '\0' || isxdigit(argv[1][1]))) {
+		for (i = 0; i < argc + 1; i++)
+			tx[i] = simple_strtoul(argv[i + 1], NULL, 16);
+		micon_tx(argc - 1, tx);
+		len = micon_rx(sizeof(rx), rx);
+		if (len < 0) {
+			micon_sync();
+			printf("communication error\n");
+			return CMD_RET_FAILURE;
+		}
+		printf("reply:");
+	} else
+		return CMD_RET_USAGE;
+
+	for (i = 0; i < len; i++)
+		printf(" %02x", rx[i]);
+	printf("\n");
+
+	return CMD_RET_SUCCESS;
+}
+
+U_BOOT_CMD(micon, MICON_MAX + 1, 0, do_micon,
+	"micro-controller commands",
+	"button             - show button status\n"
+	"micon error              - sound alarm and flash error LED\n"
+	"micon hdd <on|off>       - turn on/off hard disk power\n"
+	"micon mode               - show system mode\n"
+	"micon off                - power off system\n"
+	"micon reboot             - reboot system\n"
+	"micon sync               - synchronize\n"
+	"micon up                 - turn off all warning and turn on fan\n"
+	"micon <hex> ...          - issue command with raw byte stream");
diff -aurN u-boot-2016.05.orig/board/buffalo/lsgl/micon.h u-boot-2016.05-88fxxxx/board/buffalo/lsgl/micon.h
--- u-boot-2016.05.orig/board/buffalo/lsgl/micon.h	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/board/buffalo/lsgl/micon.h	2016-05-21 11:33:32.840226195 -0700
@@ -0,0 +1,24 @@
+/*
+ * (C) Copyright 2012
+ * Steve Shih <shihsung@gmail.com>
+ *
+ * SPDX-License-Identifier:	GPL-2.0+
+ */
+
+#ifndef __MICON_H
+#define __MICON_H
+
+#define MICON_BUTTON_POWER(button)	!((button) & 0x01)
+#define MICON_BUTTON_INIT(button)	!((button) & 0x08)
+
+void micon_init(void);
+void micon_boot(void);
+int micon_buttons(void);
+bool micon_emergency(void);
+void micon_hdd(bool on);
+void micon_error(void);
+void micon_off(void);
+void micon_reboot(void);
+void micon_up(void);
+
+#endif	/* __MICON_H */
diff -aurN u-boot-2016.05.orig/board/buffalo/lsgl/romboot.lds u-boot-2016.05-88fxxxx/board/buffalo/lsgl/romboot.lds
--- u-boot-2016.05.orig/board/buffalo/lsgl/romboot.lds	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/board/buffalo/lsgl/romboot.lds	2016-05-21 11:33:32.840226195 -0700
@@ -0,0 +1,119 @@
+/*
+ * (C) Copyright 2012
+ * Steve Shih <shihsung@gmail.com>
+ * Derived from arch/arm/cpu/u-boot.lds
+ *
+ * Copyright (c) 2004-2008 Texas Instruments
+ *
+ * (C) Copyright 2002
+ * Gary Jennejohn, DENX Software Engineering, <garyj@denx.de>
+ *
+ * SPDX-License-Identifier:	GPL-2.0+
+ */
+
+OUTPUT_FORMAT("elf32-littlearm", "elf32-littlearm", "elf32-littlearm")
+OUTPUT_ARCH(arm)
+ENTRY(_start)
+SECTIONS
+{
+	. = 0x00000000;
+
+	. = ALIGN(4);
+	.text :
+	{
+		*(.__image_copy_start)
+		*(.vectors)
+		CPUDIR/start.o (.text*)
+		*(.text*)
+	}
+
+	. = ALIGN(4);
+	.data : {
+		*(.data*)
+	}
+
+	. = ALIGN(4);
+
+	. = .;
+
+	. = ALIGN(4);
+	.u_boot_list : {
+		KEEP(*(SORT(.u_boot_list*)));
+	}
+
+	.romboot 0xffff0000 :
+	{
+		KEEP(*(.romboot));
+	}
+
+	. = ALIGN(4);
+	.rodata : { *(SORT_BY_ALIGNMENT(SORT_BY_NAME(.rodata*))) }
+
+	. = ALIGN(4);
+
+	.image_copy_end :
+	{
+		*(.__image_copy_end)
+	}
+
+	.rel_dyn_start :
+	{
+		*(.__rel_dyn_start)
+	}
+
+	.rel.dyn : {
+		*(.rel*)
+	}
+
+	.rel_dyn_end :
+	{
+		*(.__rel_dyn_end)
+	}
+
+	.end :
+	{
+		*(.__end)
+	}
+
+	_image_binary_end = .;
+
+	/*
+	 * Deprecated: this MMU section is used by pxa at present but
+	 * should not be used by new boards/CPUs.
+	 */
+	. = ALIGN(4096);
+	.mmutable : {
+		*(.mmutable)
+	}
+
+/*
+ * Compiler-generated __bss_start and __bss_end, see arch/arm/lib/bss.c
+ * __bss_base and __bss_limit are for linker only (overlay ordering)
+ */
+
+	.bss_start __rel_dyn_start (OVERLAY) : {
+		KEEP(*(.__bss_start));
+		__bss_base = .;
+	}
+
+	.bss __bss_base (OVERLAY) : {
+		*(.bss*)
+		 . = ALIGN(4);
+		 __bss_limit = .;
+	}
+
+	.bss_end __bss_limit (OVERLAY) : {
+		KEEP(*(.__bss_end));
+	}
+
+	.dynsym _image_binary_end : { *(.dynsym) }
+	.dynbss : { *(.dynbss) }
+	.dynstr : { *(.dynstr*) }
+	.dynamic : { *(.dynamic*) }
+	.plt : { *(.plt*) }
+	.interp : { *(.interp*) }
+	.gnu.hash : { *(.gnu.hash) }
+	.gnu : { *(.gnu*) }
+	.ARM.exidx : { *(.ARM.exidx*) }
+	.gnu.linkonce.armexidx : { *(.gnu.linkonce.armexidx.*) }
+}
diff -aurN u-boot-2016.05.orig/board/buffalo/lsgl/romboot.S u-boot-2016.05-88fxxxx/board/buffalo/lsgl/romboot.S
--- u-boot-2016.05.orig/board/buffalo/lsgl/romboot.S	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/board/buffalo/lsgl/romboot.S	2016-05-21 11:33:32.840226195 -0700
@@ -0,0 +1,21 @@
+/*
+ * (C) Copyright 2012
+ * Steve Shih <shihsung@gmail.com>
+ *
+ * SPDX-License-Identifier:	GPL-2.0+
+ */
+
+#include <common.h>
+
+.section	".romboot", #alloc, #execinstr
+
+_romboot:
+	/*
+	 * start DDR SDRAM initialization sequence here to workaround
+	 * "polling SDRAM operation until back in 'normal' mode" hangs.
+	 */
+	ldr	r0, =0xd0001480	/* DDR SDRAM initialization control register */
+	ldr	r1, =0x00000001	/* DDR SDRAM initialization enable */
+	str	r1, [r0]
+
+	b	_start
diff -aurN u-boot-2016.05.orig/board/cloudengines/pogo/Kconfig u-boot-2016.05-88fxxxx/board/cloudengines/pogo/Kconfig
--- u-boot-2016.05.orig/board/cloudengines/pogo/Kconfig	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/board/cloudengines/pogo/Kconfig	2016-05-21 11:33:32.840226195 -0700
@@ -0,0 +1,12 @@
+if TARGET_POGO1 || TARGET_POGO2 || TARGET_POGO4 || TARGET_POGOM || TARGET_SHEEVA
+
+config SYS_BOARD
+	default "pogo"
+
+config SYS_VENDOR
+	default "cloudengines"
+
+config SYS_CONFIG_NAME
+	default "pogo"
+
+endif
diff -aurN u-boot-2016.05.orig/board/cloudengines/pogo/kwbimage-pogo1.cfg u-boot-2016.05-88fxxxx/board/cloudengines/pogo/kwbimage-pogo1.cfg
--- u-boot-2016.05.orig/board/cloudengines/pogo/kwbimage-pogo1.cfg	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/board/cloudengines/pogo/kwbimage-pogo1.cfg	2016-05-21 11:33:32.840226195 -0700
@@ -0,0 +1,147 @@
+#
+# (C) Copyright 2013
+# Steve Shih <shihsung@gmail.com>
+#
+# (C) Copyright 2009
+# Marvell Semiconductor <www.marvell.com>
+# Written-by: Prafulla Wadaskar <prafulla@marvell.com>
+#
+# SPDX-License-Identifier:	GPL-2.0+
+#
+# Refer to doc/README.kwbimage for more details about how-to
+# configure and create kirkwood boot images.
+#
+
+# Boot Media configurations
+BOOT_FROM	nand
+NAND_ECC_MODE	default
+NAND_PAGE_SIZE	0x0800
+
+# SOC registers configuration using bootrom header extension
+# Maximum KWBIMAGE_MAX_CONFIG configurations allowed
+
+# Configure RGMII-0 interface pad voltage to 1.8V
+DATA 0xFFD100e0 0x1b1b1b9b
+
+#Dram initalization for SINGLE x16 CL=5 @ 400MHz
+DATA 0xFFD01400 0x43000c30	# DDR Configuration register
+# bit13-0:  0xc30 (3120 DDR2 clks refresh rate)
+# bit23-14: zero
+# bit24: 1= enable exit self refresh mode on DDR access
+# bit25: 1 required
+# bit29-26: zero
+# bit31-30: 01
+
+DATA 0xFFD01404 0x37543000	# DDR Controller Control Low
+# bit 4:    0=addr/cmd in smame cycle
+# bit 5:    0=clk is driven during self refresh, we don't care for APX
+# bit 6:    0=use recommended falling edge of clk for addr/cmd
+# bit14:    0=input buffer always powered up
+# bit18:    1=cpu lock transaction enabled
+# bit23-20: 5=recommended value for CL=5 and STARTBURST_DEL disabled bit31=0
+# bit27-24: 7= CL+2, STARTBURST sample stages, for freqs 400MHz, unbuffered DIMM
+# bit30-28: 3 required
+# bit31:    0=no additional STARTBURST delay
+
+DATA 0xFFD01408 0x22125451	# DDR Timing (Low) (active cycles value +1)
+# bit3-0:   TRAS lsbs
+# bit7-4:   TRCD
+# bit11- 8: TRP
+# bit15-12: TWR
+# bit19-16: TWTR
+# bit20:    TRAS msb
+# bit23-21: 0x0
+# bit27-24: TRRD
+# bit31-28: TRTP
+
+DATA 0xFFD0140C 0x00000a33	#  DDR Timing (High)
+# bit6-0:   TRFC
+# bit8-7:   TR2R
+# bit10-9:  TR2W
+# bit12-11: TW2W
+# bit31-13: zero required
+
+DATA 0xFFD01410 0x000000cc	#  DDR Address Control
+# bit1-0:   00, Cs0width=x8
+# bit3-2:   11, Cs0size=1Gb
+# bit5-4:   00, Cs1width=x8
+# bit7-6:   11, Cs1size=1Gb
+# bit9-8:   00, Cs2width=nonexistent
+# bit11-10: 00, Cs2size =nonexistent
+# bit13-12: 00, Cs3width=nonexistent
+# bit15-14: 00, Cs3size =nonexistent
+# bit16:    0,  Cs0AddrSel
+# bit17:    0,  Cs1AddrSel
+# bit18:    0,  Cs2AddrSel
+# bit19:    0,  Cs3AddrSel
+# bit31-20: 0 required
+
+DATA 0xFFD01414 0x00000000	#  DDR Open Pages Control
+# bit0:    0,  OpenPage enabled
+# bit31-1: 0 required
+
+DATA 0xFFD01418 0x00000000	#  DDR Operation
+# bit3-0:   0x0, DDR cmd
+# bit31-4:  0 required
+
+DATA 0xFFD0141C 0x00000C52	#  DDR Mode
+# bit2-0:   2, BurstLen=2 required
+# bit3:     0, BurstType=0 required
+# bit6-4:   4, CL=5
+# bit7:     0, TestMode=0 normal
+# bit8:     0, DLL reset=0 normal
+# bit11-9:  6, auto-precharge write recovery ????????????
+# bit12:    0, PD must be zero
+# bit31-13: 0 required
+
+DATA 0xFFD01420 0x00000040	#  DDR Extended Mode
+# bit0:    0,  DDR DLL enabled
+# bit1:    0,  DDR drive strenght normal
+# bit2:    0,  DDR ODT control lsd (disabled)
+# bit5-3:  000, required
+# bit6:    1,  DDR ODT control msb, (disabled)
+# bit9-7:  000, required
+# bit10:   0,  differential DQS enabled
+# bit11:   0, required
+# bit12:   0, DDR output buffer enabled
+# bit31-13: 0 required
+
+DATA 0xFFD01424 0x0000F17F	#  DDR Controller Control High
+# bit2-0:  111, required
+# bit3  :  1  , MBUS Burst Chop disabled
+# bit6-4:  111, required
+# bit7  :  0
+# bit8  :  1  , add writepath sample stage, must be 1 for DDR freq >= 300MHz
+# bit9  :  0  , no half clock cycle addition to dataout
+# bit10 :  0  , 1/4 clock cycle skew enabled for addr/ctl signals
+# bit11 :  0  , 1/4 clock cycle skew disabled for write mesh
+# bit15-12: 1111 required
+# bit31-16: 0    required
+
+DATA 0xFFD01428 0x00085520	# DDR2 ODT Read Timing (default values)
+DATA 0xFFD0147C 0x00008552	# DDR2 ODT Write Timing (default values)
+
+DATA 0xFFD01500 0x00000000	# CS[0]n Base address to 0x0
+DATA 0xFFD01504 0x0FFFFFF1	# CS[0]n Size 256Mb Window enabled for CS0
+# bit0:    1,  Window enabled
+# bit1:    0,  Write Protect disabled
+# bit3-2:  00, CS0 hit selected
+# bit23-4: ones, required
+# bit31-24: 0x0F, Size (i.e. 256MB)
+
+DATA 0xFFD0150C 0x00000000	# CS[1]n Size, window disabled
+DATA 0xFFD01514 0x00000000	# CS[2]n Size, window disabled
+DATA 0xFFD0151C 0x00000000	# CS[3]n Size, window disabled
+
+DATA 0xFFD01494 0x00030000	#  DDR ODT Control (Low)
+DATA 0xFFD01498 0x00000000	#  DDR ODT Control (High)
+# bit1-0:  00, ODT0 controlled by ODT Control (low) register above
+# bit3-2:  01, ODT1 active NEVER!
+# bit31-4: zero, required
+
+DATA 0xFFD0149C 0x0000E803	# CPU ODT Control
+DATA 0xFFD01480 0x00000001	# DDR Initialization Control
+#bit0=1, enable DDR init upon this register write
+
+# End of Header extension
+DATA 0x0 0x0
diff -aurN u-boot-2016.05.orig/board/cloudengines/pogo/kwbimage-pogo4.cfg u-boot-2016.05-88fxxxx/board/cloudengines/pogo/kwbimage-pogo4.cfg
--- u-boot-2016.05.orig/board/cloudengines/pogo/kwbimage-pogo4.cfg	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/board/cloudengines/pogo/kwbimage-pogo4.cfg	2016-05-21 11:33:32.840226195 -0700
@@ -0,0 +1,145 @@
+#
+# (C) Copyright 2013
+# Steve Shih <shihsung@gmail.com>
+#
+# (C) Copyright 2009
+# Marvell Semiconductor <www.marvell.com>
+# Written-by: Prafulla Wadaskar <prafulla@marvell.com>
+#
+# SPDX-License-Identifier:	GPL-2.0+
+#
+# Refer to doc/README.kwbimage for more details about how-to
+# configure and create kirkwood boot images.
+#
+
+# Boot Media configurations
+BOOT_FROM	nand
+NAND_ECC_MODE	default
+NAND_PAGE_SIZE	0x0800
+
+# SOC registers configuration using bootrom header extension
+# Maximum KWBIMAGE_MAX_CONFIG configurations allowed
+
+#Dram initalization for SINGLE x16 CL=5 @ 400MHz
+DATA 0xFFD01400 0x43000618	# DDR Configuration register
+# bit13-0:  0xc30 (3120 DDR2 clks refresh rate)
+# bit23-14: zero
+# bit24: 1= enable exit self refresh mode on DDR access
+# bit25: 1 required
+# bit29-26: zero
+# bit31-30: 01
+
+DATA 0xFFD01404 0x34143000	# DDR Controller Control Low
+# bit 4:    0=addr/cmd in smame cycle
+# bit 5:    0=clk is driven during self refresh, we don't care for APX
+# bit 6:    0=use recommended falling edge of clk for addr/cmd
+# bit14:    0=input buffer always powered up
+# bit18:    1=cpu lock transaction enabled
+# bit23-20: 5=recommended value for CL=5 and STARTBURST_DEL disabled bit31=0
+# bit27-24: 7= CL+2, STARTBURST sample stages, for freqs 400MHz, unbuffered DIMM
+# bit30-28: 3 required
+# bit31:    0=no additional STARTBURST delay
+
+DATA 0xFFD01408 0x11012227	# DDR Timing (Low) (active cycles value +1)
+# bit3-0:   TRAS lsbs
+# bit7-4:   TRCD
+# bit11- 8: TRP
+# bit15-12: TWR
+# bit19-16: TWTR
+# bit20:    TRAS msb
+# bit23-21: 0x0
+# bit27-24: TRRD
+# bit31-28: TRTP
+
+DATA 0xFFD0140C 0x00000819	#  DDR Timing (High)
+# bit6-0:   TRFC
+# bit8-7:   TR2R
+# bit10-9:  TR2W
+# bit12-11: TW2W
+# bit31-13: zero required
+
+DATA 0xFFD01410 0x000000CD	#  DDR Address Control
+# bit1-0:   00, Cs0width=x8
+# bit3-2:   11, Cs0size=1Gb
+# bit5-4:   00, Cs1width=x8
+# bit7-6:   11, Cs1size=1Gb
+# bit9-8:   00, Cs2width=nonexistent
+# bit11-10: 00, Cs2size =nonexistent
+# bit13-12: 00, Cs3width=nonexistent
+# bit15-14: 00, Cs3size =nonexistent
+# bit16:    0,  Cs0AddrSel
+# bit17:    0,  Cs1AddrSel
+# bit18:    0,  Cs2AddrSel
+# bit19:    0,  Cs3AddrSel
+# bit31-20: 0 required
+
+DATA 0xFFD01414 0x00000000	#  DDR Open Pages Control
+# bit0:    0,  OpenPage enabled
+# bit31-1: 0 required
+
+DATA 0xFFD01418 0x00000000	#  DDR Operation
+# bit3-0:   0x0, DDR cmd
+# bit31-4:  0 required
+
+DATA 0xFFD0141C 0x00000632	#  DDR Mode
+# bit2-0:   2, BurstLen=2 required
+# bit3:     0, BurstType=0 required
+# bit6-4:   4, CL=5
+# bit7:     0, TestMode=0 normal
+# bit8:     0, DLL reset=0 normal
+# bit11-9:  6, auto-precharge write recovery ????????????
+# bit12:    0, PD must be zero
+# bit31-13: 0 required
+
+DATA 0xFFD01420 0x00000006	#  DDR Extended Mode
+# bit0:    0,  DDR DLL enabled
+# bit1:    0,  DDR drive strenght normal
+# bit2:    0,  DDR ODT control lsd (disabled)
+# bit5-3:  000, required
+# bit6:    1,  DDR ODT control msb, (disabled)
+# bit9-7:  000, required
+# bit10:   0,  differential DQS enabled
+# bit11:   0, required
+# bit12:   0, DDR output buffer enabled
+# bit31-13: 0 required
+
+DATA 0xFFD01424 0x0000F07F	#  DDR Controller Control High
+# bit2-0:  111, required
+# bit3  :  1  , MBUS Burst Chop disabled
+# bit6-4:  111, required
+# bit7  :  0
+# bit8  :  1  , add writepath sample stage, must be 1 for DDR freq >= 300MHz
+# bit9  :  0  , no half clock cycle addition to dataout
+# bit10 :  0  , 1/4 clock cycle skew enabled for addr/ctl signals
+# bit11 :  0  , 1/4 clock cycle skew disabled for write mesh
+# bit15-12: 1111 required
+# bit31-16: 0    required
+
+DATA 0xFFD01500 0x00000000	# CS[0]n Base address to 0x0
+DATA 0xFFD01504 0x07FFFFF1	# CS[0]n Size 128Mb Window enabled for CS0
+# bit0:    1,  Window enabled
+# bit1:    0,  Write Protect disabled
+# bit3-2:  00, CS0 hit selected
+# bit23-4: ones, required
+# bit31-24: 0x07, Size (i.e. 128MB)
+
+DATA 0xFFD0150C 0x00000000	# CS[1]n Size, window disabled
+DATA 0xFFD01514 0x00000000	# CS[2]n Size, window disabled
+DATA 0xFFD0151C 0x00000000	# CS[3]n Size, window disabled
+
+DATA 0xFFD01494 0x00010000	#  DDR ODT Control (Low)
+DATA 0xFFD01498 0x00000000	#  DDR ODT Control (High)
+# bit1-0:  00, ODT0 controlled by ODT Control (low) register above
+# bit3-2:  01, ODT1 active NEVER!
+# bit31-4: zero, required
+
+DATA 0xFFD0149C 0x0000E403	# CPU ODT Control
+DATA 0xFFD01480 0x00000001	# DDR Initialization Control
+#bit0=1, enable DDR init upon this register write
+
+DATA 0xFFD01620 0x00465000
+DATA 0xFFD20134 0x66666666
+DATA 0xFFD20138 0x66666666
+
+# End of Header extension
+DATA 0x0 0x0
diff -aurN u-boot-2016.05.orig/board/cloudengines/pogo/kwbimage-sheeva.cfg u-boot-2016.05-88fxxxx/board/cloudengines/pogo/kwbimage-sheeva.cfg
--- u-boot-2016.05.orig/board/cloudengines/pogo/kwbimage-sheeva.cfg	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/board/cloudengines/pogo/kwbimage-sheeva.cfg	2016-05-21 11:33:32.844226195 -0700
@@ -0,0 +1,149 @@
+#
+# (C) Copyright 2013
+# Steve Shih <shihsung@gmail.com>
+#
+# (C) Copyright 2009
+# Marvell Semiconductor <www.marvell.com>
+# Written-by: Prafulla Wadaskar <prafulla@marvell.com>
+#
+# SPDX-License-Identifier:	GPL-2.0+
+#
+# Refer to doc/README.kwbimage for more details about how-to
+# configure and create kirkwood boot images.
+#
+
+# Boot Media configurations
+BOOT_FROM	nand
+NAND_ECC_MODE	default
+NAND_PAGE_SIZE	0x0800
+
+# SOC registers configuration using bootrom header extension
+# Maximum KWBIMAGE_MAX_CONFIG configurations allowed
+
+# Configure RGMII-0 interface pad voltage to 1.8V
+DATA 0xFFD100e0 0x1b1b1b9b
+
+#Dram initalization for SINGLE x16 CL=5 @ 400MHz
+DATA 0xFFD01400 0x43000c30	# DDR Configuration register
+# bit13-0:  0xc30 (3120 DDR2 clks refresh rate)
+# bit23-14: zero
+# bit24: 1= enable exit self refresh mode on DDR access
+# bit25: 1 required
+# bit29-26: zero
+# bit31-30: 01
+
+DATA 0xFFD01404 0x37543000	# DDR Controller Control Low
+# bit 4:    0=addr/cmd in smame cycle
+# bit 5:    0=clk is driven during self refresh, we don't care for APX
+# bit 6:    0=use recommended falling edge of clk for addr/cmd
+# bit14:    0=input buffer always powered up
+# bit18:    1=cpu lock transaction enabled
+# bit23-20: 5=recommended value for CL=5 and STARTBURST_DEL disabled bit31=0
+# bit27-24: 7= CL+2, STARTBURST sample stages, for freqs 400MHz, unbuffered DIMM
+# bit30-28: 3 required
+# bit31:    0=no additional STARTBURST delay
+
+DATA 0xFFD01408 0x22125451	# DDR Timing (Low) (active cycles value +1)
+# bit3-0:   TRAS lsbs
+# bit7-4:   TRCD
+# bit11- 8: TRP
+# bit15-12: TWR
+# bit19-16: TWTR
+# bit20:    TRAS msb
+# bit23-21: 0x0
+# bit27-24: TRRD
+# bit31-28: TRTP
+
+DATA 0xFFD0140C 0x00000a33	#  DDR Timing (High)
+# bit6-0:   TRFC
+# bit8-7:   TR2R
+# bit10-9:  TR2W
+# bit12-11: TW2W
+# bit31-13: zero required
+
+DATA 0xFFD01410 0x000000cc	#  DDR Address Control
+# bit1-0:   00, Cs0width=x8
+# bit3-2:   11, Cs0size=1Gb
+# bit5-4:   00, Cs1width=x8
+# bit7-6:   11, Cs1size=1Gb
+# bit9-8:   00, Cs2width=nonexistent
+# bit11-10: 00, Cs2size =nonexistent
+# bit13-12: 00, Cs3width=nonexistent
+# bit15-14: 00, Cs3size =nonexistent
+# bit16:    0,  Cs0AddrSel
+# bit17:    0,  Cs1AddrSel
+# bit18:    0,  Cs2AddrSel
+# bit19:    0,  Cs3AddrSel
+# bit31-20: 0 required
+
+DATA 0xFFD01414 0x00000000	#  DDR Open Pages Control
+# bit0:    0,  OpenPage enabled
+# bit31-1: 0 required
+
+DATA 0xFFD01418 0x00000000	#  DDR Operation
+# bit3-0:   0x0, DDR cmd
+# bit31-4:  0 required
+
+DATA 0xFFD0141C 0x00000C52	#  DDR Mode
+# bit2-0:   2, BurstLen=2 required
+# bit3:     0, BurstType=0 required
+# bit6-4:   4, CL=5
+# bit7:     0, TestMode=0 normal
+# bit8:     0, DLL reset=0 normal
+# bit11-9:  6, auto-precharge write recovery ????????????
+# bit12:    0, PD must be zero
+# bit31-13: 0 required
+
+DATA 0xFFD01420 0x00000040	#  DDR Extended Mode
+# bit0:    0,  DDR DLL enabled
+# bit1:    0,  DDR drive strenght normal
+# bit2:    0,  DDR ODT control lsd (disabled)
+# bit5-3:  000, required
+# bit6:    1,  DDR ODT control msb, (disabled)
+# bit9-7:  000, required
+# bit10:   0,  differential DQS enabled
+# bit11:   0, required
+# bit12:   0, DDR output buffer enabled
+# bit31-13: 0 required
+
+DATA 0xFFD01424 0x0000F17F	#  DDR Controller Control High
+# bit2-0:  111, required
+# bit3  :  1  , MBUS Burst Chop disabled
+# bit6-4:  111, required
+# bit7  :  0
+# bit8  :  1  , add writepath sample stage, must be 1 for DDR freq >= 300MHz
+# bit9  :  0  , no half clock cycle addition to dataout
+# bit10 :  0  , 1/4 clock cycle skew enabled for addr/ctl signals
+# bit11 :  0  , 1/4 clock cycle skew disabled for write mesh
+# bit15-12: 1111 required
+# bit31-16: 0    required
+
+DATA 0xFFD01428 0x00085520	# DDR2 ODT Read Timing (default values)
+DATA 0xFFD0147C 0x00008552	# DDR2 ODT Write Timing (default values)
+
+DATA 0xFFD01500 0x00000000	# CS[0]n Base address to 0x0
+DATA 0xFFD01504 0x0FFFFFF1	# CS[0]n Size 256Mb Window enabled for CS0
+# bit0:    1,  Window enabled
+# bit1:    0,  Write Protect disabled
+# bit3-2:  00, CS0 hit selected
+# bit23-4: ones, required
+# bit31-24: 0x0F, Size (i.e. 256MB)
+
+DATA 0xFFD01508 0x10000000	# CS[1]n Base address to 256Mb
+DATA 0xFFD0150C 0x0ffffff5	# CS[1]n Size 256Mb Window enabled for CS1
+
+DATA 0xFFD01514 0x00000000	# CS[2]n Size, window disabled
+DATA 0xFFD0151C 0x00000000	# CS[3]n Size, window disabled
+
+DATA 0xFFD01494 0x00030000	#  DDR ODT Control (Low)
+DATA 0xFFD01498 0x00000000	#  DDR ODT Control (High)
+# bit1-0:  00, ODT0 controlled by ODT Control (low) register above
+# bit3-2:  01, ODT1 active NEVER!
+# bit31-4: zero, required
+
+DATA 0xFFD0149C 0x0000E803	# CPU ODT Control
+DATA 0xFFD01480 0x00000001	# DDR Initialization Control
+#bit0=1, enable DDR init upon this register write
+
+# End of Header extension
+DATA 0x0 0x0
diff -aurN u-boot-2016.05.orig/board/cloudengines/pogo/MAINTAINERS u-boot-2016.05-88fxxxx/board/cloudengines/pogo/MAINTAINERS
--- u-boot-2016.05.orig/board/cloudengines/pogo/MAINTAINERS	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/board/cloudengines/pogo/MAINTAINERS	2016-05-21 11:33:32.844226195 -0700
@@ -0,0 +1,10 @@
+POGO BOARD
+M:	Steve Shih <shihsung@gmail.com>
+S:	Maintained
+F:	board/cloudengines/pogo/
+F:	include/configs/pogo.h
+F:	configs/pogo1_defconfig
+F:	configs/pogo2_defconfig
+F:	configs/pogo4_defconfig
+F:	configs/pogom_defconfig
+F:	configs/sheeva_defconfig
diff -aurN u-boot-2016.05.orig/board/cloudengines/pogo/Makefile u-boot-2016.05-88fxxxx/board/cloudengines/pogo/Makefile
--- u-boot-2016.05.orig/board/cloudengines/pogo/Makefile	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/board/cloudengines/pogo/Makefile	2016-05-21 11:33:32.844226195 -0700
@@ -0,0 +1,11 @@
+# (C) Copyright 2013
+# Steve Shih <shihsung@gmail.com>
+#
+# (C) Copyright 2009
+# Marvell Semiconductor <www.marvell.com>
+# Written-by: Prafulla Wadaskar <prafulla@marvell.com>
+#
+# SPDX-License-Identifier:	GPL-2.0+
+#
+
+obj-y	:= $(BOARD).o
diff -aurN u-boot-2016.05.orig/board/cloudengines/pogo/pogo.c u-boot-2016.05-88fxxxx/board/cloudengines/pogo/pogo.c
--- u-boot-2016.05.orig/board/cloudengines/pogo/pogo.c	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/board/cloudengines/pogo/pogo.c	2016-05-21 11:33:32.844226195 -0700
@@ -0,0 +1,189 @@
+/*
+ * (C) Copyright 2013, 2016
+ * Steve Shih <shihsung@gmail.com>
+ *
+ * (C) Copyright 2009
+ * Marvell Semiconductor <www.marvell.com>
+ * Written-by: Prafulla Wadaskar <prafulla@marvell.com>
+ *
+ * SPDX-License-Identifier:	GPL-2.0+
+ */
+
+#include <common.h>
+#include <console.h>
+#include <asm/io.h>
+#include <asm/arch/cpu.h>
+#include <asm/arch/gpio.h>
+#include <netdev.h>
+#include <miiphy.h>
+#include <usb.h>
+#include "pogo.h"
+
+DECLARE_GLOBAL_DATA_PTR;
+
+int board_early_init_f(void)
+{
+	struct kwgpio_registers *gpio0reg =
+		(struct kwgpio_registers *)MVEBU_GPIO0_BASE;
+	struct kwgpio_registers *gpio1reg =
+		(struct kwgpio_registers *)MVEBU_GPIO1_BASE;
+	u32 *mpp_ctrl = (u32 *)KW_MPP_BASE;
+
+	mvebu_config_gpio(KIRKWOOD_GPIO_LO_OUT_VAL, KIRKWOOD_GPIO_HI_OUT_VAL,
+			KIRKWOOD_GPIO_LO_OUT_ENA, KIRKWOOD_GPIO_HI_OUT_ENA);
+	writel(KIRKWOOD_GPIO_LO_BLINK_ENA, &gpio0reg->blink_en);
+	writel(KIRKWOOD_GPIO_HI_BLINK_ENA, &gpio1reg->blink_en);
+
+	writel(KIRKWOOD_MPP_0_7, &mpp_ctrl[0]);
+	writel(KIRKWOOD_MPP_8_15, &mpp_ctrl[1]);
+	writel(KIRKWOOD_MPP_16_23, &mpp_ctrl[2]);
+	writel(KIRKWOOD_MPP_24_31, &mpp_ctrl[3]);
+	writel(KIRKWOOD_MPP_32_39, &mpp_ctrl[4]);
+	writel(KIRKWOOD_MPP_40_47, &mpp_ctrl[5]);
+	writel(KIRKWOOD_MPP_48_49, &mpp_ctrl[6]);
+
+#if defined(CONFIG_POGO_EJECT_BUTTON)
+	kw_gpio_set_valid(CONFIG_POGO_EJECT_BUTTON, GPIO_INPUT_OK);
+	kw_gpio_direction_input(CONFIG_POGO_EJECT_BUTTON);
+#endif
+
+	return 0;
+}
+
+int board_init(void)
+{
+	gd->bd->bi_boot_params = gd->bd->bi_dram[0].start + CONFIG_SYS_PARAMS_PHYS;
+
+	return 0;
+}
+
+/* don't use netconsole until eth initialized */
+int overwrite_console (void)
+{
+	return 1;
+}
+
+int board_late_init (void)
+{
+	char *ethaddr = getenv("ethaddr");
+
+	if (!ethaddr)
+		setenv("ethaddr", __stringify(CONFIG_ETHADDR));
+
+	return 0;
+}
+
+int last_stage_init (void)
+{
+#ifdef CONFIG_NETCONSOLE
+	bool nc = false;
+	char *stdinname = getenv("stdin");
+	char *stdoutname = getenv("stdout");
+	char *stderrname = getenv("stderr");
+	struct eth_device *eth;
+	u16 phyaddr;
+
+	if (!strcmp(stdinname, "nc") || !strcmp(stdoutname, "nc") ||
+	    !strcmp(stdoutname, "nc"))
+		nc = true;
+
+#ifdef CONFIG_POGO_EJECT_BUTTON
+	/* swap nc and serial if init switch pressed */
+	if (!kw_gpio_get_value(CONFIG_POGO_EJECT_BUTTON)) {
+		if (!nc) {
+			stdinname = "nc";
+			stdoutname = "nc";
+			stderrname = "nc";
+		} else {
+			stdinname = "serial";
+			stdoutname = "serial";
+			stderrname = "serial";
+		}
+		nc = !nc;
+	}
+#endif
+
+	if (nc) {
+		eth_init();
+
+		/* switch to serial if not linked up */
+		eth = eth_get_dev();
+		if (!eth || miiphy_read(eth->name, 0xEE, 0xEE, &phyaddr) ||
+		    !miiphy_link(eth->name, phyaddr)) {
+			nc = false;
+			stdinname = "serial";
+			stdoutname = "serial";
+			stderrname = "serial";
+		}
+	}
+
+	console_assign(stdin, stdinname);
+	console_assign(stdout, stdoutname);
+	console_assign(stderr, stderrname);
+
+	if (nc)
+		net_start_again();
+#endif
+
+	return 0;
+}
+
+void enable_caches (void)
+{
+	icache_enable();
+	dcache_disable();
+}
+
+int board_eth_init (bd_t *bis)
+{
+	if (mvgbe_initialize(bis))
+		return -1;
+
+	/* workaround RAMBOOT netconsole issue loaded from CE U-Boot */
+	eth_halt();
+
+	return 0;
+}
+
+/* Configure and enable MV88E1116 PHY */
+void reset_phy(void)
+{
+	struct eth_device *eth;
+	u16 phyaddr;
+	u16 reg;
+
+	eth = eth_get_dev();
+	if (!eth) {
+		printf("%s: can't get ETH device\n", __FUNCTION__);
+		return;
+	}
+
+	if (miiphy_read(eth->name, 0xEE, 0xEE, &phyaddr)) {
+		printf("%s: can't read PHY device address\n", __FUNCTION__);
+		return;
+	}
+
+	/*
+	 * Enable RGMII delay on Tx and Rx for CPU port
+	 * Ref: sec 4.7.2 of chip datasheet
+	 */
+	miiphy_write(eth->name, phyaddr, MV88E1116_PGADR_REG, 2);
+	miiphy_read(eth->name, phyaddr, MV88E1116_MAC_CTRL_REG, &reg);
+	reg |= (MV88E1116_RGMII_RXTM_CTRL | MV88E1116_RGMII_TXTM_CTRL);
+	miiphy_write(eth->name, phyaddr, MV88E1116_MAC_CTRL_REG, reg);
+	miiphy_write(eth->name, phyaddr, MV88E1116_PGADR_REG, 0);
+
+	/* reset the phy */
+	miiphy_reset(eth->name, phyaddr);
+}
+
+unsigned long do_go_exec (ulong (*entry)(int, char* const[]), int argc,
+				char* const argv[])
+{
+	eth_halt();
+	usb_stop();
+
+	cleanup_before_linux();
+
+	return entry(argc, argv);
+}
diff -aurN u-boot-2016.05.orig/board/cloudengines/pogo/pogo.h u-boot-2016.05-88fxxxx/board/cloudengines/pogo/pogo.h
--- u-boot-2016.05.orig/board/cloudengines/pogo/pogo.h	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/board/cloudengines/pogo/pogo.h	2016-05-21 11:33:32.844226195 -0700
@@ -0,0 +1,23 @@
+/*
+ * (C) Copyright 2013
+ * Steve Shih <shihsung@gmail.com>
+ *
+ * (C) Copyright 2009
+ * Marvell Semiconductor <www.marvell.com>
+ * Written-by: Prafulla Wadaskar <prafulla@marvell.com>
+ *
+ * SPDX-License-Identifier:	GPL-2.0+
+ */
+
+#ifndef __POGO_H
+#define __POGO_H
+
+/* PHY related */
+#define MV88E1116_LED_FCTRL_REG		10
+#define MV88E1116_CPRSP_CR3_REG		21
+#define MV88E1116_MAC_CTRL_REG		21
+#define MV88E1116_PGADR_REG		22
+#define MV88E1116_RGMII_TXTM_CTRL	(1 << 4)
+#define MV88E1116_RGMII_RXTM_CTRL	(1 << 5)
+
+#endif /* __POGO_H */
diff -aurN u-boot-2016.05.orig/cmd/read.c u-boot-2016.05-88fxxxx/cmd/read.c
--- u-boot-2016.05.orig/cmd/read.c	2016-05-16 07:40:32.000000000 -0700
+++ u-boot-2016.05-88fxxxx/cmd/read.c	2016-05-21 11:33:32.844226195 -0700
@@ -66,7 +66,7 @@
 		return 1;
 	}
 
-	if (blk_read(dev_desc, offset + blk, cnt, addr) < 0) {
+	if (blk_dread(dev_desc, offset + blk, cnt, addr) < 0) {
 		printf("Error reading blocks\n");
 		return 1;
 	}
diff -aurN u-boot-2016.05.orig/configs/kuropro_defconfig u-boot-2016.05-88fxxxx/configs/kuropro_defconfig
--- u-boot-2016.05.orig/configs/kuropro_defconfig	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/configs/kuropro_defconfig	2016-05-21 11:33:32.844226195 -0700
@@ -0,0 +1,33 @@
+CONFIG_ARM=y
+CONFIG_ORION5X=y
+CONFIG_TARGET_KUROPRO=y
+CONFIG_SYS_EXTRA_OPTIONS="SYS_ROMBOOT"
+CONFIG_SYS_PROMPT="KUROPRO> "
+CONFIG_SYS_NS16550=y
+
+# CONFIG_BOOTM_NETBSD is not set
+# CONFIG_BOOTM_PLAN9 is not set
+# CONFIG_BOOTM_RTEMS is not set
+# CONFIG_BOOTM_VXWORKS is not set
+
+# CONFIG_CMD_CRC32 is not set
+# CONFIG_CMD_EXPORTENV is not set
+# CONFIG_CMD_IMPORTENV is not set
+
+# CONFIG_CMD_BDI is not set
+# CONFIG_CMD_ECHO is not set
+# CONFIG_CMD_FPGA is not set
+# CONFIG_CMD_ITEST is not set
+# CONFIG_CMD_IMLS is not set
+# CONFIG_CMD_LOADB is not set
+# CONFIG_CMD_LOADS is not set
+# CONFIG_CMD_MISC is not set
+# CONFIG_CMD_NFS is not set
+# CONFIG_CMD_SETGETDCR is not set
+# CONFIG_CMD_SOURCE is not set
+# CONFIG_CMD_XIMG is not set
+
+# CONFIG_CMD_ELF is not set
+# CONFIG_CMD_SETEXPR is not set
+
+# CONFIG_REGEX is not set
diff -aurN u-boot-2016.05.orig/configs/kuropro_nand_defconfig u-boot-2016.05-88fxxxx/configs/kuropro_nand_defconfig
--- u-boot-2016.05.orig/configs/kuropro_nand_defconfig	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/configs/kuropro_nand_defconfig	2016-05-21 11:33:32.844226195 -0700
@@ -0,0 +1,6 @@
+CONFIG_ARM=y
+CONFIG_ORION5X=y
+CONFIG_TARGET_KUROPRO=y
+CONFIG_SYS_EXTRA_OPTIONS="SYS_RAMBOOT,ENV_IS_IN_NAND"
+CONFIG_SYS_PROMPT="KUROPRO> "
+CONFIG_SYS_NS16550=y
diff -aurN u-boot-2016.05.orig/configs/kuropro_ram_defconfig u-boot-2016.05-88fxxxx/configs/kuropro_ram_defconfig
--- u-boot-2016.05.orig/configs/kuropro_ram_defconfig	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/configs/kuropro_ram_defconfig	2016-05-21 11:33:32.844226195 -0700
@@ -0,0 +1,6 @@
+CONFIG_ARM=y
+CONFIG_ORION5X=y
+CONFIG_TARGET_KUROPRO=y
+CONFIG_SYS_EXTRA_OPTIONS="SYS_RAMBOOT"
+CONFIG_SYS_PROMPT="KUROPRO> "
+CONFIG_SYS_NS16550=y
diff -aurN u-boot-2016.05.orig/configs/ls420_defconfig u-boot-2016.05-88fxxxx/configs/ls420_defconfig
--- u-boot-2016.05.orig/configs/ls420_defconfig	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/configs/ls420_defconfig	2016-05-21 11:33:32.844226195 -0700
@@ -0,0 +1,14 @@
+CONFIG_ARM=y
+CONFIG_ARCH_MVEBU=y
+CONFIG_SYS_MALLOC_F_LEN=0x2000
+CONFIG_TARGET_LS420=y
+CONFIG_SPL_SYS_MALLOC_SIMPLE=y
+CONFIG_DEFAULT_DEVICE_TREE="armada-370-buffalo-ls421de"
+CONFIG_SPL=y
+CONFIG_SYS_PROMPT="LS420> "
+# CONFIG_CMD_IMLS is not set
+CONFIG_SPL_OF_TRANSLATE=y
+CONFIG_SPI_FLASH=y
+CONFIG_SPI_FLASH_MACRONIX=y
+CONFIG_SYS_NS16550=y
+CONFIG_OF_LIBFDT=y
diff -aurN u-boot-2016.05.orig/configs/ls440_defconfig u-boot-2016.05-88fxxxx/configs/ls440_defconfig
--- u-boot-2016.05.orig/configs/ls440_defconfig	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/configs/ls440_defconfig	2016-05-21 11:33:32.844226195 -0700
@@ -0,0 +1,14 @@
+CONFIG_ARM=y
+CONFIG_ARCH_MVEBU=y
+CONFIG_SYS_MALLOC_F_LEN=0x2000
+CONFIG_TARGET_LS440=y
+CONFIG_SPL_SYS_MALLOC_SIMPLE=y
+CONFIG_DEFAULT_DEVICE_TREE="armada-370-buffalo-ls441de"
+CONFIG_SPL=y
+CONFIG_SYS_PROMPT="LS440> "
+# CONFIG_CMD_IMLS is not set
+CONFIG_SPL_OF_TRANSLATE=y
+CONFIG_SPI_FLASH=y
+CONFIG_SPI_FLASH_MACRONIX=y
+CONFIG_SYS_NS16550=y
+CONFIG_OF_LIBFDT=y
diff -aurN u-boot-2016.05.orig/configs/lsgl_defconfig u-boot-2016.05-88fxxxx/configs/lsgl_defconfig
--- u-boot-2016.05.orig/configs/lsgl_defconfig	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/configs/lsgl_defconfig	2016-05-21 11:33:32.844226195 -0700
@@ -0,0 +1,33 @@
+CONFIG_ARM=y
+CONFIG_ORION5X=y
+CONFIG_TARGET_LSGL=y
+CONFIG_SYS_EXTRA_OPTIONS="SYS_ROMBOOT"
+CONFIG_SYS_PROMPT="LSGL> "
+CONFIG_SYS_NS16550=y
+
+# CONFIG_BOOTM_NETBSD is not set
+# CONFIG_BOOTM_PLAN9 is not set
+# CONFIG_BOOTM_RTEMS is not set
+# CONFIG_BOOTM_VXWORKS is not set
+
+# CONFIG_CMD_CRC32 is not set
+# CONFIG_CMD_EXPORTENV is not set
+# CONFIG_CMD_IMPORTENV is not set
+
+# CONFIG_CMD_BDI is not set
+# CONFIG_CMD_ECHO is not set
+# CONFIG_CMD_FPGA is not set
+# CONFIG_CMD_ITEST is not set
+# CONFIG_CMD_IMLS is not set
+# CONFIG_CMD_LOADB is not set
+# CONFIG_CMD_LOADS is not set
+# CONFIG_CMD_MISC is not set
+# CONFIG_CMD_NFS is not set
+# CONFIG_CMD_SETGETDCR is not set
+# CONFIG_CMD_SOURCE is not set
+# CONFIG_CMD_XIMG is not set
+
+# CONFIG_CMD_ELF is not set
+# CONFIG_CMD_SETEXPR is not set
+
+# CONFIG_REGEX is not set
diff -aurN u-boot-2016.05.orig/configs/lsgl_ram_defconfig u-boot-2016.05-88fxxxx/configs/lsgl_ram_defconfig
--- u-boot-2016.05.orig/configs/lsgl_ram_defconfig	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/configs/lsgl_ram_defconfig	2016-05-21 11:33:32.844226195 -0700
@@ -0,0 +1,6 @@
+CONFIG_ARM=y
+CONFIG_ORION5X=y
+CONFIG_TARGET_LSGL=y
+CONFIG_SYS_EXTRA_OPTIONS="SYS_RAMBOOT"
+CONFIG_SYS_PROMPT="LSGL> "
+CONFIG_SYS_NS16550=y
diff -aurN u-boot-2016.05.orig/configs/pogo1_defconfig u-boot-2016.05-88fxxxx/configs/pogo1_defconfig
--- u-boot-2016.05.orig/configs/pogo1_defconfig	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/configs/pogo1_defconfig	2016-05-21 11:33:32.844226195 -0700
@@ -0,0 +1,8 @@
+CONFIG_ARM=y
+CONFIG_KIRKWOOD=y
+CONFIG_TARGET_POGO1=y
+CONFIG_SYS_EXTRA_OPTIONS="POGO1"
+CONFIG_SYS_PROMPT="POGO1> "
+# CONFIG_CMD_IMLS is not set
+CONFIG_SYS_NS16550=y
+CONFIG_OF_LIBFDT=y
diff -aurN u-boot-2016.05.orig/configs/pogo2_defconfig u-boot-2016.05-88fxxxx/configs/pogo2_defconfig
--- u-boot-2016.05.orig/configs/pogo2_defconfig	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/configs/pogo2_defconfig	2016-05-21 11:33:32.844226195 -0700
@@ -0,0 +1,8 @@
+CONFIG_ARM=y
+CONFIG_KIRKWOOD=y
+CONFIG_TARGET_POGO2=y
+CONFIG_SYS_EXTRA_OPTIONS="POGO2"
+CONFIG_SYS_PROMPT="POGO2> "
+# CONFIG_CMD_IMLS is not set
+CONFIG_SYS_NS16550=y
+CONFIG_OF_LIBFDT=y
diff -aurN u-boot-2016.05.orig/configs/pogo4_defconfig u-boot-2016.05-88fxxxx/configs/pogo4_defconfig
--- u-boot-2016.05.orig/configs/pogo4_defconfig	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/configs/pogo4_defconfig	2016-05-21 11:33:32.844226195 -0700
@@ -0,0 +1,8 @@
+CONFIG_ARM=y
+CONFIG_KIRKWOOD=y
+CONFIG_TARGET_POGO4=y
+CONFIG_SYS_EXTRA_OPTIONS="POGO4"
+CONFIG_SYS_PROMPT="POGO4> "
+# CONFIG_CMD_IMLS is not set
+CONFIG_SYS_NS16550=y
+CONFIG_OF_LIBFDT=y
diff -aurN u-boot-2016.05.orig/configs/pogom_defconfig u-boot-2016.05-88fxxxx/configs/pogom_defconfig
--- u-boot-2016.05.orig/configs/pogom_defconfig	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/configs/pogom_defconfig	2016-05-21 11:33:32.844226195 -0700
@@ -0,0 +1,8 @@
+CONFIG_ARM=y
+CONFIG_KIRKWOOD=y
+CONFIG_TARGET_POGOM=y
+CONFIG_SYS_EXTRA_OPTIONS="POGOM"
+CONFIG_SYS_PROMPT="POGOM> "
+# CONFIG_CMD_IMLS is not set
+CONFIG_SYS_NS16550=y
+CONFIG_OF_LIBFDT=y
diff -aurN u-boot-2016.05.orig/configs/sheeva_defconfig u-boot-2016.05-88fxxxx/configs/sheeva_defconfig
--- u-boot-2016.05.orig/configs/sheeva_defconfig	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/configs/sheeva_defconfig	2016-05-21 11:33:32.844226195 -0700
@@ -0,0 +1,8 @@
+CONFIG_ARM=y
+CONFIG_KIRKWOOD=y
+CONFIG_TARGET_SHEEVA=y
+CONFIG_SYS_EXTRA_OPTIONS="SHEEVA"
+CONFIG_SYS_PROMPT="SHEEVA> "
+# CONFIG_CMD_IMLS is not set
+CONFIG_SYS_NS16550=y
+CONFIG_OF_LIBFDT=y
diff -aurN u-boot-2016.05.orig/drivers/block/mvsata_ide.c u-boot-2016.05-88fxxxx/drivers/block/mvsata_ide.c
--- u-boot-2016.05.orig/drivers/block/mvsata_ide.c	2016-05-16 07:40:32.000000000 -0700
+++ u-boot-2016.05-88fxxxx/drivers/block/mvsata_ide.c	2016-05-21 11:33:32.844226195 -0700
@@ -21,7 +21,9 @@
 struct mvsata_port_registers {
 	u32 reserved0[10];
 	u32 edma_cmd;
-	u32 reserved1[181];
+	u32 reserved1[2];
+	u32 iordy_timeout;
+	u32 reserved2[178];
 	/* offset 0x300 : ATA Interface registers */
 	u32 sstatus;
 	u32 serror;
@@ -29,7 +31,7 @@
 	u32 ltmode;
 	u32 phymode3;
 	u32 phymode4;
-	u32 reserved2[5];
+	u32 reserved3[5];
 	u32 phymode1;
 	u32 phymode2;
 	u32 bist_cr;
@@ -91,6 +93,43 @@
 #define MVSATA_STATUS_OK	0
 #define MVSATA_STATUS_TIMEOUT	-1
 
+#if defined(CONFIG_ARMADA_370)
+/*
+ * From Linux: mv_soc_65n_phy_errata() in drivers/ata/sata_mv.c
+ */
+static void mvsata_soc_65n_phy_errata(struct mvsata_port_registers *port)
+{
+	u8 *port_mmio = (u8*)port;
+	u32 reg;
+
+	reg = readl(&port->phymode3);
+	reg &= ~(0x3 << 27);	/* SELMUPF (bits 28:27) to 1 */
+	reg |= (0x1 << 27);
+	reg &= ~(0x3 << 29);	/* SELMUPI (bits 30:29) to 1 */
+	reg |= (0x1 << 29);
+	writel(reg, &port->phymode3);
+
+	reg = readl(&port->phymode4);
+	reg &= ~0x1;	/* SATU_OD8 (bit 0) to 0, reserved bit 16 must be set */
+	reg |= (0x1 << 16);
+	writel(reg, &port->phymode4);
+
+#define PHY_MODE9_GEN2			0x0398
+	reg = readl(port_mmio + PHY_MODE9_GEN2);
+	reg &= ~0xf;	/* TXAMP[3:0] (bits 3:0) to 8 */
+	reg |= 0x8;
+	reg &= ~(0x1 << 14);	/* TXAMP[4] (bit 14) to 0 */
+	writel(reg, port_mmio + PHY_MODE9_GEN2);
+
+#define PHY_MODE9_GEN1			0x039c
+	reg = readl(port_mmio + PHY_MODE9_GEN1);
+	reg &= ~0xf;	/* TXAMP[3:0] (bits 3:0) to 8 */
+	reg |= 0x8;
+	reg &= ~(0x1 << 14);	/* TXAMP[4] (bit 14) to 0 */
+	writel(reg, port_mmio + PHY_MODE9_GEN1);
+}
+#endif
+
 /*
  * Registers for SATA MBUS memory windows
  */
@@ -144,6 +183,14 @@
 	udelay(25); /* taken from original marvell port */
 	writel(0, &port->edma_cmd);
 
+#if defined(CONFIG_ARMADA_370)
+	mvsata_soc_65n_phy_errata(port);
+#endif
+
+	/* SATA IORdy time-out is 1250 ns */
+	writel(DIV_ROUND_UP(CONFIG_SYS_TCLK / 1000000 * 1250, 1000),
+		&port->iordy_timeout);
+
 	/* Set control IPM to 3 (no low power) and DET to 1 (initialize) */
 	control = readl(&port->scontrol);
 	control = (control & ~MVSATA_SCONTROL_MASK) | MVSATA_PORT_INIT;
diff -aurN u-boot-2016.05.orig/drivers/ddr/marvell/axp/ddr3_a370_config.h u-boot-2016.05-88fxxxx/drivers/ddr/marvell/axp/ddr3_a370_config.h
--- u-boot-2016.05.orig/drivers/ddr/marvell/axp/ddr3_a370_config.h	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/drivers/ddr/marvell/axp/ddr3_a370_config.h	2016-05-21 11:33:32.844226195 -0700
@@ -0,0 +1,65 @@
+/*
+ * Copyright (C) Marvell International Ltd. and its affiliates
+ *
+ * SPDX-License-Identifier:	GPL-2.0
+ *
+ * Taken from u-boot-armada-2011.12
+ */
+
+#ifndef __DDR3_A370_CONFIG_H
+#define __DDR3_A370_CONFIG_H
+
+/*
+ * DDR3_LOG_LEVEL Information
+ *
+ * Level 0: Provides an error code in a case of failure, RL, WL errors
+ *          and other algorithm failure
+ * Level 1: Provides the D-Unit setup (SPD/Static configuration)
+ * Level 2: Provides the windows margin as a results of DQS centeralization
+ * Level 3: Provides the windows margin of each DQ as a results of DQS
+ *          centeralization
+ */
+#ifdef CONFIG_DDR_LOG_LEVEL
+#define DDR3_LOG_LEVEL	CONFIG_DDR_LOG_LEVEL
+#else
+#define DDR3_LOG_LEVEL  0
+#endif
+
+#define DDR3_PBS        0
+
+/* This flag allows the execution of SW WL/RL upon HW failure */
+#define DDR3_RUN_SW_WHEN_HW_FAIL	1
+
+/*
+ * General Configurations
+ *
+ * The following parameters are required for proper setup:
+ *
+ * DRAM_ECC            - Must be set to FALSE
+ * DQS_CLK_ALIGNED     - Set this if CLK and DQS signals are aligned on board
+ * DDR3_TRAINING_DEBUG - debug prints of internal code
+ * A370_A0             - must be defined if using Marvell DB board and A370 A0 device
+ *                       otherwise dont care
+ */
+#define DUNIT_STATIC
+#define DRAM_ECC			0
+#undef DQS_CLK_ALIGNED
+#define DDR3_TRAINING_DEBUG		0
+#undef A370_A0
+#define REG_DIMM_SKIP_WL		1
+
+/* Marvell boards specific configurations */
+#if defined(DB_88F6710_PCAC)
+#define STATIC_TRAINING
+#endif
+
+#if defined(DB_88F6710) && !defined(A370_A0)
+#define AUTO_DETECTION_SUPPORT
+#define SPD_SUPPORT
+#define DRAM_2T				0x0
+#define DIMM_CS_BITMAP			0xF
+#define DUNIT_SPD
+#undef DUNIT_STATIC
+#endif
+
+#endif /* __DDR3_A370_CONFIG_H */
diff -aurN u-boot-2016.05.orig/drivers/ddr/marvell/axp/ddr3_a370.h u-boot-2016.05-88fxxxx/drivers/ddr/marvell/axp/ddr3_a370.h
--- u-boot-2016.05.orig/drivers/ddr/marvell/axp/ddr3_a370.h	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/drivers/ddr/marvell/axp/ddr3_a370.h	2016-05-21 11:33:32.844226195 -0700
@@ -0,0 +1,448 @@
+/*
+ * Copyright (C) Marvell International Ltd. and its affiliates
+ *
+ * SPDX-License-Identifier:	GPL-2.0
+ *
+ * Taken from u-boot-armada-2011.12
+ */
+
+#ifndef __DDR3_A370_H
+#define __DDR3_A370_H
+
+#define SAR_DDR3_FREQ_MASK		0xFF800
+#define SAR_CPU_FAB_GET(cpu, fab)	(((cpu & 0xF) << 11) | ((fab & 0x1F) << 15))
+
+#define MAX_CS				4
+
+#define MIN_DIMM_ADDR			0x50
+#define FAR_END_DIMM_ADDR		0x56
+#define MAX_DIMM_ADDR			0x60
+
+#ifndef CONFIG_DDR_FIXED_SIZE
+#define SDRAM_CS_SIZE			0xFFFFFFF
+#else
+#define SDRAM_CS_SIZE			(CONFIG_DDR_FIXED_SIZE - 1)
+#endif
+#define SDRAM_CS_BASE			0x0
+#define SDRAM_DIMM_SIZE			0x80000000
+
+#define CPU_CONFIGURATION_REG(id)	(0x21800 + (id * 0x100))
+#define CPU_MRVL_ID_OFFSET		0x10
+#define SAR1_CPU_CORE_MASK		0x00000018
+#define SAR1_CPU_CORE_OFFSET		3
+
+#define BUS_WIDTH			16
+#define ECC_SUPPORT_TWSI_ADDR		0
+#define EEPROM_MODULE_ADDR		0x21
+#define CFG_MULTI_CS_MODE(cfg)		((cfg >> 6) & 0x1)
+
+#define MV_6710_DEV_ID			0x6710
+#define MV_MAX_DDR3_STATIC_SIZE		50
+#define MV_DDR3_MODES_NUMBER		15
+
+/*
+ * Registers offset
+ */
+
+#define REG_SAMPLE_RESET_LOW_ADDR		0x18230
+#define REG_SAMPLE_RESET_HIGH_ADDR		0x18234
+#define	REG_SAMPLE_RESET_CPU_FREQ_OFFS		11
+#define	REG_SAMPLE_RESET_CPU_FREQ_MASK		0x7800
+#define	REG_SAMPLE_RESET_FAB_OFFS		15
+#define	REG_SAMPLE_RESET_FAB_MASK		0xF8000
+#define	REG_SAMPLE_RESET_TCLK_OFFS		20
+#define	REG_SAMPLE_RESET_CPU_ARCH_OFFS		21
+#define REG_SAMPLE_RESET_HIGH_CPU_FREQ_OFFS	0
+
+/* MISC */
+#define INTER_REGS_BASE				0xD0000000
+
+/* DDR */
+#define REG_SDRAM_CONFIG_ADDR			0x1400
+#define REG_SDRAM_CONFIG_MASK			0x9FFFFFFF
+#define REG_SDRAM_CONFIG_RFRS_MASK		0x3FFF
+#define REG_SDRAM_CONFIG_WIDTH_OFFS		15
+#define REG_SDRAM_CONFIG_REGDIMM_OFFS		17
+#define REG_SDRAM_CONFIG_ECC_OFFS		18
+#define REG_SDRAM_CONFIG_IERR_OFFS		19
+#define REG_SDRAM_CONFIG_PUPRSTDIV_OFFS		28
+#define REG_SDRAM_CONFIG_RSTRD_OFFS		30
+
+#define REG_DUNIT_CTRL_LOW_ADDR			0x1404
+#define REG_DUNIT_CTRL_LOW_2T_OFFS		3
+#define REG_DUNIT_CTRL_LOW_DPDE_OFFS		14
+#define REG_DUNIT_CTRL_LOW_2T_MASK		0x3
+#define REG_DUNIT_CTRL_LOW_DPDE_OFFS		14
+
+#define REG_SDRAM_TIMING_LOW_ADDR		0x1408
+
+#define REG_SDRAM_TIMING_LOW_ADDR		0x1408
+
+#define REG_SDRAM_TIMING_HIGH_ADDR		0x140C
+#define REG_SDRAM_TIMING_H_R2R_OFFS		7
+#define REG_SDRAM_TIMING_H_R2R_MASK		0x3
+#define REG_SDRAM_TIMING_H_R2W_W2R_OFFS		9
+#define REG_SDRAM_TIMING_H_R2W_W2R_MASK		0x3
+#define REG_SDRAM_TIMING_H_W2W_OFFS		11
+#define REG_SDRAM_TIMING_H_W2W_MASK		0x1F
+#define REG_SDRAM_TIMING_H_R2R_H_OFFS		19
+#define REG_SDRAM_TIMING_H_R2R_H_MASK		0x7
+#define REG_SDRAM_TIMING_H_R2W_W2R_H_OFFS	22
+#define REG_SDRAM_TIMING_H_R2W_W2R_H_MASK	0x7
+
+#define REG_SDRAM_ADDRESS_CTRL_ADDR		0x1410
+#define REG_SDRAM_ADDRESS_SIZE_OFFS		2
+#define REG_SDRAM_ADDRESS_SIZE_HIGH_OFFS	18
+#define REG_SDRAM_ADDRESS_CTRL_STRUCT_OFFS	4
+
+#define REG_SDRAM_OPEN_PAGES_ADDR		0x1414
+#define REG_SDRAM_OPERATION_CS_OFFS		8
+
+#define REG_SDRAM_OPERATION_ADDR		0x1418
+#define REG_SDRAM_OPERATION_CWA_DELAY_SEL_OFFS	24
+#define REG_SDRAM_OPERATION_CWA_DATA_OFFS	20
+#define REG_SDRAM_OPERATION_CWA_DATA_MASK	0xF
+#define REG_SDRAM_OPERATION_CWA_RC_OFFS		16
+#define REG_SDRAM_OPERATION_CWA_RC_MASK		0xF
+#define REG_SDRAM_OPERATION_CMD_MR0		0xF03
+#define REG_SDRAM_OPERATION_CMD_MR1		0xF04
+#define REG_SDRAM_OPERATION_CMD_MR2		0xF08
+#define REG_SDRAM_OPERATION_CMD_MR3		0xF09
+#define REG_SDRAM_OPERATION_CMD_RFRS		0xF02
+#define REG_SDRAM_OPERATION_CMD_CWA		0xF0E
+#define REG_SDRAM_OPERATION_CMD_RFRS_DONE	0xF
+#define REG_SDRAM_OPERATION_CMD_MASK		0xF
+#define REG_SDRAM_OPERATION_CS_OFFS		8
+
+
+#define REG_SDRAM_OPERATION_CMD_MR0		0xF03
+#define REG_SDRAM_OPERATION_CMD_MR1		0xF04
+#define REG_SDRAM_OPERATION_CMD_MR2		0xF08
+#define REG_SDRAM_OPERATION_CMD_MR3		0xF09
+#define REG_SDRAM_OPERATION_CMD_RFRS		0xF02
+#define REG_SDRAM_OPERATION_CMD_RFRS_DONE	0xF
+#define REG_SDRAM_OPERATION_CS_OFFS		8
+
+#define REG_OUDDR3_TIMING_ADDR			0x142C
+
+#define REG_SDRAM_MODE_ADDR			0x141C
+
+#define REG_SDRAM_EXT_MODE_ADDR			0x1420
+
+#define REG_DDR_CONT_HIGH_ADDR			0x1424
+
+#define REG_ODT_TIME_LOW_ADDR			0x1428
+#define REG_ODT_ON_CTL_RD_OFFS			12
+#define REG_ODT_OFF_CTL_RD_OFFS			16
+
+#define REG_SDRAM_ERROR_ADDR			0x1454
+#define REG_SDRAM_AUTO_PWR_SAVE_ADDR		0x1474
+#define REG_ODT_TIME_HIGH_ADDR			0x147C
+
+#define REG_SDRAM_INIT_CTRL_ADDR		0x1480
+#define REG_SDRAM_INIT_CTRL_OFFS		0
+
+#define REG_MBUS_CPU_BLOCK_ADDR			0x1484
+
+#define REG_SDRAM_ODT_CTRL_LOW_ADDR		0x1494
+
+#define REG_SDRAM_ODT_CTRL_HIGH_ADDR		0x1498
+#define REG_SDRAM_ODT_CTRL_HIGH_OVRD_MASK	0x0
+#define REG_SDRAM_ODT_CTRL_HIGH_OVRD_ENA	0x3
+
+
+#define REG_DUNIT_ODT_CTRL_ADDR			0x149C
+#define REG_DUNIT_ODT_CTRL_OVRD_OFFS		8
+#define REG_DUNIT_ODT_CTRL_OVRD_VAL_OFFS	9
+
+#define REG_DRAM_AXI_CTRL_ADDR			0x14A8
+
+#define REG_METAL_MASK_ADDR			0x14B0
+#define REG_METAL_MASK_MASK			0xDFFFFFFF
+/* #define REG_METAL_MASK_DUNIT_CROSS_OFFS	0 */
+#define REG_METAL_MASK_RETRY_OFFS		0
+
+#define REG_DRAM_ADDR_CTRL_DRIVE_STRENGTH_ADDR	0x14C0
+#define REG_DRAM_DATA_DQS_DRIVE_STRENGTH_ADDR	0x14C4
+
+#define REG_CS_SIZE_SCRATCH_ADDR 		0x1504
+
+#define REG_DDR_IO_ADDR				0x1524
+#define REG_DDR_IO_CLK_RATIO_OFFS		15
+
+#define REG_DFS_ADDR				0x1528
+#define REG_DFS_DLLNEXTSTATE_OFFS		0
+#define REG_DFS_BLOCK_OFFS			1
+#define REG_DFS_SR_OFFS				2
+#define REG_DFS_ATSR_OFFS			3
+#define REG_DFS_RECONF_OFFS			4
+
+#define REG_READ_DATA_SAMPLE_DELAYS_ADDR	0x1538
+#define REG_READ_DATA_SAMPLE_DELAYS_MASK	0x1F
+#define REG_READ_DATA_SAMPLE_DELAYS_OFFS	8
+
+#define REG_READ_DATA_READY_DELAYS_ADDR		0x153C
+#define REG_READ_DATA_READY_DELAYS_MASK		0x1F
+#define REG_READ_DATA_READY_DELAYS_OFFS		8
+
+#define START_BURST_IN_ADDR			1
+
+#define REG_DRAM_TRAINING_SHADOW_ADDR		0x18488
+#define REG_DRAM_TRAINING_ADDR			0x15B0
+#define REG_DRAM_TRAINING_LOW_FREQ_OFFS		0
+#define REG_DRAM_TRAINING_PATTERNS_OFFS		4
+#define REG_DRAM_TRAINING_MED_FREQ_OFFS		2
+#define REG_DRAM_TRAINING_WL_OFFS		3
+#define REG_DRAM_TRAINING_RL_OFFS		6
+#define REG_DRAM_TRAINING_CS_OFFS		20
+#define REG_DRAM_TRAINING_RETEST_OFFS		24
+#define REG_DRAM_TRAINING_DFS_FREQ_OFFS		27
+#define REG_DRAM_TRAINING_DFS_REQ_OFFS		29
+#define REG_DRAM_TRAINING_ERROR_OFFS		30
+#define REG_DRAM_TRAINING_AUTO_OFFS		31
+#define REG_DRAM_TRAINING_RETEST_PAR		0x3
+#define REG_DRAM_TRAINING_RETEST_MASK		0xF8FFFFFF
+#define REG_DRAM_TRAINING_CS_MASK		0xFF0FFFFF
+#define REG_DRAM_TRAINING_PATTERNS_MASK 	0xFF0F0000
+
+#define REG_DRAM_TRAINING_1_ADDR		0x15B4
+#define REG_DRAM_TRAINING_1_TRNBPOINT_OFFS	16
+
+#define REG_DRAM_TRAINING_2_ADDR		0x15B8
+#define REG_DRAM_TRAINING_2_OVERRUN_OFFS	17
+#define REG_DRAM_TRAINING_2_FIFO_RST_OFFS	4
+#define REG_DRAM_TRAINING_2_RL_MODE_OFFS	3
+#define REG_DRAM_TRAINING_2_WL_MODE_OFFS	2
+#define REG_DRAM_TRAINING_2_ECC_MUX_OFFS	1
+#define REG_DRAM_TRAINING_2_SW_OVRD_OFFS	0
+
+#define REG_DRAM_TRAINING_PATTERN_BASE_ADDR	0x15BC
+#define REG_DRAM_TRAINING_PATTERN_BASE_OFFS	3
+
+#define REG_TRAINING_DEBUG_2_ADDR		0x15C4
+#define REG_TRAINING_DEBUG_2_OFFS		16
+#define REG_TRAINING_DEBUG_2_MASK		0x3
+
+#define REG_TRAINING_DEBUG_3_ADDR		0x15C8
+#define REG_TRAINING_DEBUG_3_OFFS		3
+#define REG_TRAINING_DEBUG_3_MASK		0x7
+
+
+#define	REG_DDR3_MR0_ADDR			0x15D0
+#define	REG_DDR3_MR0_CS_ADDR			0x1870
+#define REG_DDR3_MR0_MASK			0x74
+#define	REG_DDR3_MR0_CL_OFFS			2
+#define	REG_DDR3_MR0_CL_HIGH_OFFS		3
+#define	CL_MASK					0xF
+
+#define	REG_DDR3_MR1_ADDR			0x15D4
+#define REG_DDR3_MR1_RTT_MASK			0xFFFFFDBB
+#define REG_DDR3_MR1_DLL_ENA_OFFS		0
+#define REG_DDR3_MR1_RTT_DISABLED		0x0
+#define REG_DDR3_MR1_RTT_RZQ2			0x40
+#define REG_DDR3_MR1_RTT_RZQ4			0x2
+#define REG_DDR3_MR1_RTT_RZQ6			0x42
+#define REG_DDR3_MR1_RTT_RZQ8			0x202
+#define REG_DDR3_MR1_RTT_RZQ12			0x4
+#define REG_DDR3_MR1_OUTBUF_WL_MASK		0xFFFFEF7F	/* WL-disabled,OB-enabled */
+#define REG_DDR3_MR1_OUTBUF_DIS_OFFS		12	/* Output Buffer Disabled */
+#define REG_DDR3_MR1_WL_ENA_OFFS		7
+#define REG_DDR3_MR1_WL_ENA			0x80	/* WL Enabled */
+#define REG_DDR3_MR1_ODT_MASK			0xFFFFFDBB
+
+#define	REG_DDR3_MR2_ADDR			0x15D8
+#define	REG_DDR3_MR2_CS_ADDR			0x1878
+#define	REG_DDR3_MR2_CWL_OFFS			3
+#define	REG_DDR3_MR2_CWL_MASK			0x7
+#define REG_DDR3_MR2_ODT_MASK			0xFFFFF9FF
+#define	REG_DDR3_MR3_ADDR			0x15DC
+
+#define REG_DDR3_RANK_CTRL_ADDR			0x15E0
+#define REG_DDR3_RANK_CTRL_CS_ENA_MASK		0xF
+#define REG_DDR3_RANK_CTRL_MIRROR_OFFS		4
+
+#define REG_ZQC_CONF_ADDR			0x15E4
+
+#define REG_DRAM_PHY_CONFIG_ADDR		0x15EC
+#define REG_DRAM_PHY_CONFIG_MASK		0x3FFFFFFF
+
+#define REG_ODPG_CNTRL_ADDR			0x1600
+#define REG_ODPG_CNTRL_OFFS			21
+
+#define REG_PHY_LOCK_MASK_ADDR			0x1670
+#define REG_PHY_LOCK_MASK_MASK			0xFFFFF000
+
+#define REG_PHY_LOCK_STATUS_ADDR		0x1674
+#define REG_PHY_LOCK_STATUS_LOCK_OFFS		9
+#define REG_PHY_LOCK_STATUS_LOCK_MASK		0xFFF
+#define REG_PHY_LOCK_APLL_ADLL_STATUS_MASK	0x7FF
+
+#define REG_PHY_REGISTRY_FILE_ACCESS_ADDR	0x16A0
+#define REG_PHY_REGISTRY_FILE_ACCESS_OP_WR	0xC0000000
+#define REG_PHY_REGISTRY_FILE_ACCESS_OP_RD	0x80000000
+#define REG_PHY_REGISTRY_FILE_ACCESS_OP_DONE	0x80000000
+#define REG_PHY_BC_OFFS				27
+#define REG_PHY_CNTRL_OFFS			26
+#define REG_PHY_CS_OFFS				16
+#define REG_PHY_DQS_REF_DLY_OFFS		10
+#define REG_PHY_PHASE_OFFS			8
+#define REG_PHY_PUP_OFFS			22
+
+#define REG_TRAINING_WL_ADDR			0x16AC
+#define REG_TRAINING_WL_CS_MASK			0xFFFFFFFC
+#define REG_TRAINING_WL_UPD_OFFS		2
+#define REG_TRAINING_WL_CS_DONE_OFFS		3
+#define REG_TRAINING_WL_RATIO_MASK		0xFFFFFF0F
+#define REG_TRAINING_WL_1TO1			0x50
+#define REG_TRAINING_WL_2TO1			0x10
+#define REG_TRAINING_WL_DELAYEXP_MASK		0x20000000
+#define REG_TRAINING_WL_RESULTS_MASK		0x000001FF
+#define REG_TRAINING_WL_RESULTS_OFFS		20
+
+/* CPU    */
+#define REG_MPP_CONTROL_ADDR			0x18000
+#define REG_MPP_CONTROL_TWSI_OFFS		8
+#define REG_MPP_CONTROL_TWSI_VALUE		0x11
+
+#define REG_BOOTROM_ROUTINE_ADDR		0x182D0
+#define REG_BOOTROM_ROUTINE_DRAM_INIT_OFFS	12
+
+#define REG_DRAM_INIT_CTRL_STATUS_ADDR		0x18488
+#define REG_DRAM_INIT_CTRL_TRN_CLK_OFFS		16
+#define REG_CPU_DIV_CLK_CTRL_0_NEW_RATIO	0x000200FF
+#define REG_DRAM_INIT_CTRL_STATUS_2_ADDR	0x1488
+
+#define REG_CPU_DIV_CLK_CTRL_0_ADDR		0x18700
+
+#define REG_CPU_DIV_CLK_CTRL_1_ADDR		0x18704
+#define REG_CPU_DIV_CLK_CTRL_2_ADDR		0x18708
+
+#define REG_CPU_DIV_CLK_CTRL_3_ADDR		0x1870C
+#define REG_CPU_DIV_CLK_CTRL_3_FREQ_MASK	0xFFFFC0FF
+#define REG_CPU_DIV_CLK_CTRL_3_FREQ_OFFS	8
+
+#define REG_CPU_DIV_CLK_CTRL_4_ADDR		0x18710
+
+#define REG_CPU_DIV_CLK_STATUS_0_ADDR		0x18718
+#define REG_CPU_DIV_CLK_ALL_STABLE_OFFS		8
+
+#define REG_CPU_PLL_CTRL_0_ADDR			0x1871C
+#define REG_CPU_PLL_STATUS_0_ADDR		0x18724
+#define REG_CORE_DIV_CLK_CTRL_ADDR		0x18740
+#define REG_CORE_DIV_CLK_STATUS_ADDR		0x18744
+#define REG_DDRPHY_APLL_CTRL_ADDR		0x18780
+
+#define REG_DDRPHY_APLL_CTRL_2_ADDR		0x18784
+
+/* DRAM Windows */
+#define REG_XBAR_WIN_19_CTRL_ADDR		0x200e8
+#define REG_XBAR_WIN_4_CTRL_ADDR		0x20040
+#define REG_XBAR_WIN_4_BASE_ADDR		0x20044
+#define REG_XBAR_WIN_4_REMAP_ADDR		0x20048
+#define REG_FASTPATH_WIN_0_CTRL_ADDR		0x20184
+#define REG_XBAR_WIN_7_REMAP_ADDR		0x20078
+
+/* SRAM */
+#define REG_CDI_CONFIG_ADDR			0x20220
+#define REG_SRAM_WINDOW_0_ADDR			0x20240
+#define REG_SRAM_WINDOW_0_ENA_OFFS		0
+#define REG_SRAM_WINDOW_1_ADDR			0x20244
+#define REG_SRAM_L2_ENA_ADDR			0x8500
+#define REG_SRAM_CLEAN_BY_WAY_ADDR		0x87BC
+
+/* Timers */
+#define REG_TIMERS_CTRL_ADDR			0x20300
+#define REG_TIMERS_EVENTS_ADDR			0x20304
+#define REG_TIMER0_VALUE_ADDR			0x20314
+#define REG_TIMER1_VALUE_ADDR			0x2031C
+#define REG_TIMER0_ENABLE_MASK			0x1
+
+/* PMU */
+#define REG_PMU_I_F_CTRL_ADDR			0x1C090
+#define REG_PMU_DUNIT_BLK_OFFS			16
+#define REG_PMU_DUNIT_RFRS_OFFS			20
+#define REG_PMU_DUNIT_ACK_OFFS			24
+
+/* Controller revision info */
+#define PCI_CLASS_CODE_AND_REVISION_ID		0x008
+#define PCCRIR_REVID_OFFS			0	/* Revision ID */
+#define PCCRIR_REVID_MASK			(0xff << PCCRIR_REVID_OFFS)
+
+/*  Power Management Clock Gating Control Register	*/
+#define MV_PEX_IF_REGS_OFFSET(pexIf)		(0x40000 + (pexIf * 0x40000))
+#define MV_PEX_IF_REGS_BASE(unit) 		(MV_PEX_IF_REGS_OFFSET(unit))
+#define POWER_MNG_CTRL_REG			0x18220
+#define PEX_DEVICE_AND_VENDOR_ID		0x000
+#define PEX_CFG_DIRECT_ACCESS(pexIf, cfgReg)	(MV_PEX_IF_REGS_BASE(pexIf) + (cfgReg))
+#define PMC_PEXSTOPCLOCK_OFFS(port)		((port) < 8 ? (5 + (port)) : (18 + (port)))
+#define PMC_PEXSTOPCLOCK_MASK(port)		(1 << PMC_PEXSTOPCLOCK_OFFS(port))
+#define PMC_PEXSTOPCLOCK_EN(port)		(1 << PMC_PEXSTOPCLOCK_OFFS(port))
+#define PMC_PEXSTOPCLOCK_STOP(port)		(0 << PMC_PEXSTOPCLOCK_OFFS(port))
+
+/* TWSI */
+#define TWSI_DATA_ADDR_MASK			0x7
+#define TWSI_DATA_ADDR_OFFS			1
+
+/* General */
+#define MAX_CS					4
+
+/* Frequencies */
+#define FAB_OPT					21
+#define CLK_CPU					8
+#define CLK_VCO					(2 * CLK_CPU)
+#define CLK_DDR					6
+
+/* Cpu Frequencies: */
+#define CLK_CPU_400				0
+#define CLK_CPU_533				1
+#define CLK_CPU_667				2
+#define CLK_CPU_800				3
+#define CLK_CPU_1000				4
+#define CLK_CPU_1067				5
+#define CLK_CPU_1200				6
+#define CLK_CPU_1333				7
+
+/* DDR3 Frequencies: */
+#define DDR_100					0
+#define DDR_300					1
+#define DDR_333					1
+#define DDR_400					2
+#define DDR_500					3
+#define DDR_533					3
+#define DDR_600					4
+#define DDR_666					5
+#define DDR_S					9
+#define DDR_S_1TO1				10
+#define MARGIN_FREQ 				DDR_400
+#define DFS_MARGIN				DDR_100
+
+#define ODT_OPT					16
+#define ODT20					0x200
+#define ODT30					0x204
+#define ODT40					0x44
+#define ODT60					0x4
+#define ODT120					0x40
+#define ODT120D					0x400
+
+#define MRS_DELAY				100
+
+#define SDRAM_WL_SW_OFFS			0x100
+#define SDRAM_RL_OFFS				0x0
+
+#define SDRAM_PBS_I_OFFS			0x30
+#define SDRAM_PBS_II_OFFS			0x50
+#define SDRAM_PBS_NEXT_OFFS			(SDRAM_PBS_II_OFFS - SDRAM_PBS_I_OFFS)
+
+/* #define SDRAM_PBS_RX_OFFS			0x30 */
+/* #define SDRAM_PBS_TX_OFFS			384 */
+/* #define SDRAM_PBS_TX_DM_OFFS			576 */
+#define SDRAM_DQS_RX_OFFS			1024
+#define SDRAM_DQS_TX_OFFS			2048
+#define SDRAM_DQS_RX_SPECIAL_OFFS		5120
+
+#define LEN_STD_PATTERN				4
+#define LEN_KILLER_PATTERN			128
+#define LEN_SPECIAL_PATTERN			128
+#define LEN_PBS_PATTERN				4
+
+#endif /* __DDR3_A370_H */
diff -aurN u-boot-2016.05.orig/drivers/ddr/marvell/axp/ddr3_a370_mc_static.h u-boot-2016.05-88fxxxx/drivers/ddr/marvell/axp/ddr3_a370_mc_static.h
--- u-boot-2016.05.orig/drivers/ddr/marvell/axp/ddr3_a370_mc_static.h	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/drivers/ddr/marvell/axp/ddr3_a370_mc_static.h	2016-05-21 11:33:32.848226195 -0700
@@ -0,0 +1,156 @@
+/*
+ * Copyright (C) Marvell International Ltd. and its affiliates
+ *
+ * SPDX-License-Identifier:	GPL-2.0
+ *
+ * Taken from u-boot-armada-2011.12
+ */
+
+#ifndef __A370_MC_STATIC_H
+#define __A370_MC_STATIC_H
+
+/* MC - DUNIT registers values - relevant only if SPD_SUPPORT is NOT defined */
+MV_DRAM_MC_INIT ddr3_A0_db_400[MV_MAX_DDR3_STATIC_SIZE] =
+{
+	{0x00001400, 0x73004926}, 	/*DDR SDRAM Configuration Register */
+	{0x00001404, 0x3630B800},	/*Dunit Control Low Register */
+	{0x00001408, 0x33137663},	/*DDR SDRAM Timing (Low) Register */
+	{0x0000140C, 0x38000C30},	/*DDR SDRAM Timing (High) Register */
+	{0x00001410, 0x020F0000}, 	/*DDR SDRAM Open Pages Control Register */
+	{0x00001414, 0x00000000},	/*DDR SDRAM Open Pages Control Register */
+	{0x00001418, 0x00000e00},	/*DDR SDRAM Operation Register */
+	{0x00001420, 0x00000004},	/*DDR SDRAM Extended Mode Register */
+	{0x00001424, 0x0100D1FF},	/*Dunit Control High Register */
+	{0x00001428, 0x000D6720},	/*Dunit Control High Register */
+	{0x0000142C, 0x014C2F38},	/*Dunit Control High Register */
+	{0x0000147C, 0x00006571},
+
+	{0x00001484, 0x0000E907},	/* AXI/Mbus Arbiter  */
+
+	{0x00001494, 0x00010000},	/*DDR SDRAM ODT Control (Low) Register */
+	{0x00001498, 0x00000000},	/*DDR SDRAM ODT Control (High) Register */
+	{0x0000149C, 0x00000001},	/*DDR Dunit ODT Control Register */
+
+	{0x000014C0, 0x192424C8}, 	/* DRAM address and Control Driving Strenght  */
+	{0x000014C4, 0xEFB24C8}, 	/* DRAM Data and DQS Driving Strenght  */
+
+	{0x000200e8, 0x1FFF0E01}, 	/* DO NOT Modify - Open Mbus Window - 2G - Mbus is required for the training sequence*/
+ 	{0x00020184, 0x1FFFFFE0}, 	/* DO NOT Modify - Close fast path Window to - 2G */
+
+	{0x0001504, 0x1FFFFFF1}, 	/* CS0 Size */
+	{0x000150C, 0x00000000}, 	/* CS1 Size */
+	{0x0001514, 0x00000000}, 	/* CS2 Size */
+	{0x000151C, 0x00000000}, 	/* CS3 Size */
+ 
+	{0x00001538, 0x00000006},	/*Read Data Sample Delays Register */
+	{0x0000153C, 0x00000007},	/*Read Data Ready Delay Register */
+
+	{0x000015D0, 0x00000610},	/*MR0 */
+	{0x000015D4, 0x00000046},	/*MR1 */
+	{0x000015D8, 0x00000008},	/*MR2 */
+	{0x000015DC, 0x00000000},	/*MR3 */
+
+	{0x000015E4, 0x00203c18},	/*ZQDS Configuration Register */
+	{0x000015EC, 0xDE000025},	/*DDR PHY */
+	{0x0, 0x0}
+};
+
+/* MC - DUNIT registers values - relevant only if SPD_SUPPORT is NOT defined */
+MV_DRAM_MC_INIT ddr3_A0_db_600[MV_MAX_DDR3_STATIC_SIZE] =
+{
+	{0x00001400, 0x73014924}, 	/*DDR SDRAM Configuration Register */
+	{0x00001404, 0x3000B800}, 	/*Dunit Control Low Register */
+	{0x00001408, 0x45148886}, 	/*DDR SDRAM Timing (Low) Register */
+	{0x0000140C, 0x3A000C5F}, 	/*DDR SDRAM Timing (High) Register */
+
+	{0x00001410, 0x040F0000}, 	/*DDR SDRAM Open Pages Control Register */
+
+	{0x00001414, 0x00000000}, 	/*DDR SDRAM Open Pages Control Register */
+	{0x00001418, 0x00000e00}, 	/*DDR SDRAM Operation Register */
+	{0x00001420, 0x00000004}, 	/*DDR SDRAM Extended Mode Register */
+	{0x00001424, 0x0100D1FF}, 	/*Dunit Control High Register */
+	{0x00001428, 0x000E7830}, 	/*Dunit Control High Register */
+	{0x0000142C, 0x214C2F38}, 	/*Dunit Control High Register */
+	{0x0000147C, 0x0000c671},
+
+	{0x00001484, 0x0000E907},	/* AXI/Mbus Arbiter  */
+	{0x000014a8, 0x00000100}, 	/* AXI Control */
+
+	{0x00001494, 0x00010000}, 	/*DDR SDRAM ODT Control (Low) Register */
+	{0x00001498, 0x00000000}, 	/*DDR SDRAM ODT Control (High) Register */
+	{0x0000149C, 0x00000001}, 	/*DDR Dunit ODT Control Register */
+
+	{0x000014C0, 0x192424C8}, 	/* DRAM address and Control Driving Strenght  */
+	{0x000014C4, 0xEFB24C8}, 	/* DRAM Data and DQS Driving Strenght  */
+
+	{0x000200e8, 0x1FFF0E01}, 	/*Open Mbus Window - 2G */
+	{0x00020184, 0x1FFFFFE0}, 	/*Close fast path Window to - 2G */
+
+	{0x00001504, 0x1FFFFFF1}, 	/*Dummy */
+	{0x0000150C, 0x00000000}, 	/*Close all 15xx registers they are not used */
+	{0x00001514, 0x00000000}, 	/*Close all 15xx registers they are not used */
+	{0x0000151C, 0x00000000}, 	/*Close all 15xx registers they are not used */
+
+	{0x00001538, 0x0000000b}, 	/*Read Data Sample Delays Register */
+	{0x0000153C, 0x0000000d}, 	/*Read Data Ready Delay Register */
+
+	{0x000015D0, 0x00000640}, 	/*MR0 */
+	{0x000015D4, 0x00000046}, 	/*MR1 */
+	{0x000015D8, 0x00000010}, 	/*MR2 */
+	{0x000015DC, 0x00000000}, 	/*MR3 */
+
+	{0x000015E4, 0x00203c18}, 	/*ZQC Configuration Register */
+	{0x000015EC, 0xDE000025}, 	/*DDR PHY */
+	{0x0, 0x0}
+};
+
+/* MC - DUNIT registers values - relevant only if SPD_SUPPORT is NOT defined */
+MV_DRAM_MC_INIT ddr3_A0_rd_600[MV_MAX_DDR3_STATIC_SIZE] =
+{
+	{0x00001400, 0x73014924}, 	/*DDR SDRAM Configuration Register */
+	{0x00001404, 0x3000B810}, 	/*Dunit Control Low Register */
+	{0x00001408, 0x45148886}, 	/*DDR SDRAM Timing (Low) Register */
+	{0x0000140C, 0x3A000C5F}, 	/*DDR SDRAM Timing (High) Register */
+
+	{0x00001410, 0x040F0000}, 	/*DDR SDRAM Open Pages Control Register */
+
+	{0x00001414, 0x00000000}, 	/*DDR SDRAM Open Pages Control Register */
+	{0x00001418, 0x00000e00}, 	/*DDR SDRAM Operation Register */
+	{0x00001420, 0x00000004}, 	/*DDR SDRAM Extended Mode Register */
+	{0x00001424, 0x0100D1FF}, 	/*Dunit Control High Register */
+	{0x00001428, 0x000E7830}, 	/*Dunit Control High Register */
+	{0x0000142C, 0x214C2F38}, 	/*Dunit Control High Register */
+	{0x0000147C, 0x0000c671},
+
+	{0x00001484, 0x0000E907},	/* AXI/Mbus Arbiter  */
+	{0x000014a8, 0x00000100}, 	/* AXI Control */
+
+	{0x00001494, 0x00010000}, 	/*DDR SDRAM ODT Control (Low) Register */
+	{0x00001498, 0x00000000}, 	/*DDR SDRAM ODT Control (High) Register */
+	{0x0000149C, 0x00000001}, 	/*DDR Dunit ODT Control Register */
+
+	{0x000014C0, 0x192424C8}, 	/* DRAM address and Control Driving Strenght  */
+	{0x000014C4, 0xEFB24C8}, 	/* DRAM Data and DQS Driving Strenght  */
+
+	{0x000200e8, 0x1FFF0E01}, 	/*Open Mbus Window - 2G */
+	{0x00020184, 0x1FFFFFE0}, 	/*Close fast path Window to - 2G */
+
+	{0x00001504, 0x1FFFFFF1}, 	/*Dummy */
+	{0x0000150C, 0x00000000}, 	/*Close all 15xx registers they are not used */
+	{0x00001514, 0x00000000}, 	/*Close all 15xx registers they are not used */
+	{0x0000151C, 0x00000000}, 	/*Close all 15xx registers they are not used */
+
+	{0x00001538, 0x0000000b}, 	/*Read Data Sample Delays Register */
+	{0x0000153C, 0x0000000d}, 	/*Read Data Ready Delay Register */
+
+	{0x000015D0, 0x00000640}, 	/*MR0 */
+	{0x000015D4, 0x00000046}, 	/*MR1 */
+	{0x000015D8, 0x00000010}, 	/*MR2 */
+	{0x000015DC, 0x00000000}, 	/*MR3 */
+
+	{0x000015E4, 0x00203c18}, 	/*ZQC Configuration Register */
+	{0x000015EC, 0xDE000025}, 	/*DDR PHY */
+	{0x0, 0x0}
+};
+
+#endif /* __A370_MC_STATIC_H */
diff -aurN u-boot-2016.05.orig/drivers/ddr/marvell/axp/ddr3_a370_training_static.h u-boot-2016.05-88fxxxx/drivers/ddr/marvell/axp/ddr3_a370_training_static.h
--- u-boot-2016.05.orig/drivers/ddr/marvell/axp/ddr3_a370_training_static.h	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/drivers/ddr/marvell/axp/ddr3_a370_training_static.h	2016-05-21 11:33:32.848226195 -0700
@@ -0,0 +1,96 @@
+/*
+ * Copyright (C) Marvell International Ltd. and its affiliates
+ *
+ * SPDX-License-Identifier:	GPL-2.0
+ *
+ * Taken from u-boot-armada-2011.12
+ */
+
+#ifndef __A370_TRAINING_STATIC_H
+#define __A370_TRAINING_STATIC_H
+
+/*
+ * STATIC_TRAINING - Set only if static parameters for training are set and
+ * required
+ */
+/*#define STATIC_TRAINING */
+
+/* DDR3 Training static parameters - relevant only if STATIC_TRAINING is defined */
+MV_DRAM_TRAINING_INIT ddr3_db_400[MV_MAX_DDR3_STATIC_SIZE] =
+{
+	/* Read Leveling */
+	/*PUP	RdSampleDly (+CL)	Phase	RL ADLL value */
+	/*0		2		3		1 */
+	{0x000016A0, 0xC0020015},
+	/*1		2		2		6 */
+	{0x000016A0, 0xC0420019},
+
+	/* Write Leveling */
+	/*0 */
+	{0x000016A0, 0xC0006C0E},
+	/*1 */
+	{0x000016A0, 0xC0409418},
+
+	/*center DQS on read cycle */
+	{0x000016A0, 0xC803000F},
+
+	{0x00001538, 0x00000007}, 	/*Read Data Sample Delays Register */
+	{0x0000153C, 0x00000009}, 	/*Read Data Ready Delay Register */
+	/*init DRAM */
+	{0x00001480, 0x00000001},
+	{0x0, 0x0}
+};
+
+/* DDR3 Training static parameters - relevant only if STATIC_TRAINING is defined */
+MV_DRAM_TRAINING_INIT ddr3_db_600[MV_MAX_DDR3_STATIC_SIZE] =
+{
+	/* Read Leveling */
+	/*PUP	RdSampleDly (+CL)	Phase	RL ADLL value */
+	/*0		2		3		1 */
+	{0x000016A0, 0xC0020119},
+	/*1		2		2		6 */
+	{0x000016A0, 0xC042011E},
+
+	/* Write Leveling */
+	/*0 */
+	{0x000016A0, 0xC0005508},
+	/*1 */
+	{0x000016A0, 0xC0409117},
+
+	/*center DQS on read cycle */
+	{0x000016A0, 0xC803000F},
+
+	{0x00001538, 0x0000000B}, 	/*Read Data Sample Delays Register */
+	{0x0000153C, 0x0000000F}, 	/*Read Data Ready Delay Register */
+	/*init DRAM */
+	{0x00001480, 0x00000001},
+	{0x0, 0x0}
+};
+
+/* DDR3 Training static parameters - relevant only if STATIC_TRAINING is defined */
+MV_DRAM_TRAINING_INIT ddr3_pcac_600[MV_MAX_DDR3_STATIC_SIZE] =
+{
+	/* Read Leveling */
+	/*PUP	RdSampleDly (+CL)	Phase	RL ADLL value */
+	/*0		2		3		1 */
+	{0x000016A0, 0xC0020006},
+	/*1		2		2		6 */
+ 	{0x000016A0, 0xC0420009},
+
+	/* Write Leveling */
+	/*0 */
+	{0x000016A0, 0xC0005809},
+	/*1 */
+	{0x000016A0, 0xC040680D},
+
+	/*center DQS on read cycle */
+	{0x000016A0, 0xC803000F},
+
+	{0x00001538, 0x0000000A}, 	/*Read Data Sample Delays Register */
+	{0x0000153C, 0x0000000C}, 	/*Read Data Ready Delay Register */
+	/*init DRAM */
+	{0x00001480, 0x00000001},
+	{0x0, 0x0}
+};
+
+#endif /* __A370_TRAINING_STATIC_H */
diff -aurN u-boot-2016.05.orig/drivers/ddr/marvell/axp/ddr3_a370_vars.h u-boot-2016.05-88fxxxx/drivers/ddr/marvell/axp/ddr3_a370_vars.h
--- u-boot-2016.05.orig/drivers/ddr/marvell/axp/ddr3_a370_vars.h	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/drivers/ddr/marvell/axp/ddr3_a370_vars.h	2016-05-21 11:33:32.848226195 -0700
@@ -0,0 +1,201 @@
+/*
+ * Copyright (C) Marvell International Ltd. and its affiliates
+ * 
+ * SPDX-License-Identifier:	GPL-2.0
+ *
+ * Taken from u-boot-armada-2011.12
+ */
+
+#ifndef __A370_VARS_H
+#define __A370_VARS_H
+
+#include "ddr3_a370_config.h"
+#include "ddr3_a370_mc_static.h"
+#include "ddr3_a370_training_static.h"
+
+MV_DRAM_MODES ddr_modes[MV_DDR3_MODES_NUMBER] =
+{
+	/*	Conf name		CPUFreq		FabFreq		Chip ID		Chip/Board	MC regs		Training Values */
+	/* db board values: */
+	{"db_1200-300-600", 0x6, 0x5, 0x0, A0, ddr3_A0_db_600, ddr3_db_600},
+	{"pcac_1200-300-600", 0x6, 0x5, 0x0, A0_PCAC, ddr3_A0_db_600, ddr3_pcac_600},
+	{"rd_1200-300-600", 0x6, 0x5, 0x0, A0_RD, ddr3_A0_rd_600, NULL},
+};
+
+/*
+ * ODT settings - if needed update the following tables:
+ * (ODT_OPT - represents the CS configuration bitmap)
+ */
+
+u16 odt_static[ODT_OPT][MAX_CS] =
+{
+	{0, 0, 0, 0}, 	/* 0000 */
+	{ODT60,	0, 0, 0}, 	/* 0001 */
+	{0, 0, 0, 0}, 	/* 0010 */
+	{ODT60,	0, 0, 0}, 	/* 0011 */
+	{0, 0, 0, 0}, 	/* 0100 */
+	{0, 0, 0, 0}, 	/* 0101 */
+	{0, 0, 0, 0}, 	/* 0110 */
+	{0, 0, 0, 0}, 	/* 0111 */
+	{0, 0, 0, 0}, 	/* 1000 */
+	{0, 0, 0, 0}, 	/* 1001 */
+	{0, 0, 0, 0}, 	/* 1010 */
+	{0, 0, 0, 0}, 	/* 1011 */
+	{0, 0, 0, 0}, 	/* 1100 */
+	{0, 0, 0, 0}, 	/* 1101 */
+	{0, 0, 0, 0}, 	/* 1110 */
+	{0, 0, 0, 0}, 	/* 1111 */
+};
+
+u16 odt_dynamic[ODT_OPT][MAX_CS] =
+{
+	{0, 0, 0, 0}, 	/* 0000 */
+	{0, 0, 0, 0}, 	/* 0001 */
+	{0, 0, 0, 0}, 	/* 0010 */
+	{0, 0, 0, 0}, 	/* 0011 */
+	{0, 0, 0, 0}, 	/* 0100 */
+	{0, 0, 0, 0}, 	/* 0101 */
+	{0, 0, 0, 0}, 	/* 0110 */
+	{0, 0, 0, 0}, 	/* 0111 */
+	{0, 0, 0, 0}, 	/* 1000 */
+	{0, 0, 0, 0}, 	/* 1001 */
+	{0, 0, 0, 0}, 	/* 1010 */
+	{0, 0, 0, 0}, 	/* 1011 */
+	{0, 0, 0, 0}, 	/* 1100 */
+	{0, 0, 0, 0}, 	/* 1101 */
+	{0, 0, 0, 0}, 	/* 1110 */
+	{0, 0, 0, 0} 	/* 1111 */
+};
+
+u32 odt_config[ODT_OPT] = {
+	0, 0x00010000, 0, 0x00030000, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
+};
+
+/*
+ * User can manually set SPD values (in case SPD is not available on DIMM/System).
+ * SPD Values can simplify calculating the DUNIT registers values
+ */	
+u8 spd_data[SPD_SIZE] = 
+{ 
+	/* A370 DB Board DIMM SPD Values - manually set */
+	/* 2 CS */
+	0x92, 0x10, 0x0B, 0x2, 0x3, 0x19, 0x0, 0x9, 0x09, 0x52, 0x1, 0x8, 0x0C,
+	0x0, 0x7E, 0x0, 0x69, 0x78,
+	/* 1 CS */
+/*	0x92, 0x10, 0x0B, 0x2, 0x3, 0x19, 0x0, 0x1, 0x09, 0x52, 0x1, 0x8, 0x0C,
+	0x0, 0x7E, 0x0, 0x69, 0x78, */
+	0x69, 0x30, 0x69, 0x11, 0x20, 0x89, 0x0, 0x5, 0x3C, 0x3C, 0x0, 0xF0, 0x82,
+	0x5, 0x80, 0x0, 0x0, 0x0,
+	0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
+	0x0, 0x0, 0x0, 0x0, 0x0,
+	0x0, 0x0, 0x0, 0x0, 0x0F, 0x1, 0x3, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
+	0x0, 0x0, 0x0, 0x0, 0x0,
+	0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
+	0x0, 0x0, 0x0, 0x0, 0x0,
+	0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
+	0x0, 0x0, 0x0, 0x0, 0x0,
+	0x0, 0x80, 0x2C, 0x1, 0x10, 0x23, 0x35, 0x28, 0xEB, 0xCA, 0x19, 0x8F
+};
+
+/*
+ * Controller Specific configurations Starts Here - DO NOT MODIFY
+ */
+
+/* Frequency - values are 1/HCLK in ps */
+u32 cpu_fab_clk_to_hclk[FAB_OPT][CLK_CPU] =
+/* CPU Frequency:
+	400		533		667		800		1000	1067	1200	1333	Fabric */
+{
+	{0, 0, 0, 0, 0, 0, 0, 0},
+	{0, 0, 3330, 2500, 0, 0, 0, 0},
+	{0, 0, 0, 0, 0, 0, 0, 0},
+	{0, 0, 0, 0, 0, 0, 0, 0},
+	{0, 0, 0, 0, 0, 0, 0, 0},
+	{0, 0, 0, 0, 4000, 3750, 3000, 3000},
+	{0, 0, 0, 0, 0, 0, 0, 0},
+	{0, 0, 0, 0, 0, 0, 2500, 0},
+	{0, 0, 0, 0, 0, 0, 0, 0},
+	{0, 0, 0, 0, 0, 0, 0, 0},
+ 	{0, 0, 0, 0, 0, 0, 0, 0},
+	{0, 0, 0, 0, 0, 0, 0, 0},
+	{0, 0, 0, 0, 0, 0, 0, 0},
+	{0, 0, 0, 0, 0, 0, 0, 0},
+	{0, 0, 0, 0, 0, 0, 0, 0},
+	{0, 0, 0, 0, 0, 0, 0, 0},
+	{0, 0, 0, 0, 0, 0, 0, 0},
+	{0, 0, 0, 0, 0, 0, 0, 0},
+	{0, 0, 0, 0, 0, 0, 0, 0},
+	{0, 3750, 3000, 0, 0, 0, 0, 0},
+	{2500, 0, 0, 3750, 3000, 0, 0, 0}
+};
+
+u32 cpu_ddr_ratios[FAB_OPT][CLK_CPU] =
+/* CPU Frequency:
+	400		533		667		800		1000	1067	1200	1333	Fabric */
+{
+	{0, 0, 0, 0, 0, 0, 0, 0},
+	{0, 0, DDR_333, DDR_400, 0, 0, 0, 0},	
+ 	{0, 0, 0, 0, 0, 0, 0, 0},
+	{0, 0, 0, 0, 0, 0, 0, 0},
+	{0, 0, 0, 0, 0, 0, 0, 0},
+ 	{0, 0, 0, 0, DDR_500, DDR_533, DDR_600, DDR_666},
+	{0, 0, 0, 0, 0, 0, 0, 0},
+ 	{0, 0, 0, 0, 0, 0, DDR_400, 0},
+	{0, 0, 0, 0, 0, 0, 0, 0},
+ 	{0, 0, 0, 0, 0, 0, 0, 0},
+ 	{0, 0, 0, 0, 0, 0, 0, 0},
+	{0, 0, 0, 0, 0, 0, 0, 0},
+	{0, 0, 0, 0, 0, 0, 0, 0},
+	{0, 0, 0, 0, 0, 0, 0, 0},
+	{0, 0, 0, 0, 0, 0, 0, 0},
+	{0, 0, 0, 0, 0, 0, 0, 0},
+	{0, 0, 0, 0, 0, 0, 0, 0},
+	{0, 0, 0, 0, 0, 0, 0, 0},
+	{0, 0, 0, 0, 0, 0, 0, 0},
+	{0, DDR_533, DDR_666, 0, 0, 0, 0, 0},
+	{DDR_400, 0, 0, DDR_533, DDR_666, 0, 0, 0}
+};
+
+u8 div_ratio1to1[CLK_VCO][CLK_DDR] =
+/* DDR Frequency:
+	100		333	400 533	600	666		*/
+{ 	{0x4,  0, 1, 0, 0, 0},	/*	1:1	CLK_CPU_400 = VCO  */
+	{0x5,  0, 0, 1,	0, 0},	/*	1:1	CLK_CPU_533 = VCO   */
+	{0x6,  2, 0, 0,	0, 0},	/*	1:1	CLK_CPU_667 = VCO   */
+	{0x8,  0, 2, 0,	0, 0},	/*	1:1	CLK_CPU_800 = VCO   */
+	{0xA,  3, 5, 2,	0, 0},	/*	1:1	CLK_CPU_1000 = VCO   */
+	{0xA,  0, 0, 2,	0, 0},	/*	1:1	CLK_CPU_1067 = VCO   */
+	{0xC,  0, 6, 0,	2, 0},	/*	1:1	CLK_CPU_1200 = VCO   */
+	{0xD,  0, 0, 0,	0, 2},	/*	1:1	CLK_CPU_1333 = VCO   */
+	{0x8,  0, 0, 0,	0, 0},	/*	1:1	CLK_CPU_400 VCO_800  */
+	{0xA,  0, 0, 0,	0, 0},	/*	1:1	CLK_CPU_533 VCO_1066 */
+	{0xC,  0, 0, 0,	0, 0},	/*	1:1	CLK_CPU_667 VCO_1333 */
+	{0x10, 0, 0, 0,	0, 0},	/*	1:1	CLK_CPU_800 VCO_1600 */
+	{0x14, 0, 0, 0,	0, 0},	/*	1:1	CLK_CPU_1000 VCO_2000  */
+	{0x15, 0, 0, 0,	0, 0},	/*	1:1	CLK_CPU_1067 VCO_2133 */
+	{0x18, 0, 6, 0,	0, 0},	/*	1:1	CLK_CPU_1200 VCO_2400 */
+	{0x1A, 0, 0, 0,	0, 0}	/*	1:1	CLK_CPU_1333 VCO_2666 */
+};
+
+u8 div_ratio2to1[CLK_VCO][CLK_DDR] =
+/* DDR Frequency:
+	100		333	400	533	600	666		*/
+{ 	{0, 0, 0, 0, 0,	0},	/*	2:1	CLK_CPU_400 = VCO  */
+	{0, 0, 0, 1, 0,	0},	/*	2:1	CLK_CPU_533 = VCO  */
+	{0, 2, 0, 0, 0, 1},	/*	2:1	CLK_CPU_667 = VCO  */
+	{0, 0, 2, 3, 0,	0},	/*	2:1	CLK_CPU_800 = VCO  */
+	{0, 0, 5, 2, 0,	0},	/*	2:1	CLK_CPU_1000 = VCO  */
+	{0, 0, 0, 2, 0,	5},	/*	2:1	CLK_CPU_1067 = VCO  */
+	{0, 0, 3, 0, 2,	0},	/*	2:1	CLK_CPU_1200 = VCO  */
+	{0, 0, 0, 5, 0,	2},	/*	2:1	CLK_CPU_1333 = VCO  */
+	{0, 0, 0, 0, 0,	0},	/*	2:1	CLK_CPU_400 VCO_800  */
+	{0, 0, 0, 0, 0,	0},	/*	2:1	CLK_CPU_533 VCO_1066 */
+	{0, 0, 0, 0, 0,	0},	/*	2:1	CLK_CPU_667 VCO_1333 */
+	{0, 0, 0, 3, 0,	0},	/*	2:1	CLK_CPU_800 VCO_1600 */
+	{0, 0, 0, 0, 0,	3},	/*	2:1	CLK_CPU_1000 VCO_2000  */
+	{0, 0, 0, 0, 0,	0},	/*	2:1	CLK_CPU_1067 VCO_2133 */
+	{0, 0, 6, 0, 0,	0},	/*	2:1	CLK_CPU_1200 VCO_2400 */
+	{0, 0, 0, 0, 0,	0}	/*	2:1	CLK_CPU_1333 VCO_2666 */
+};
+
+#endif /* __A370_VARS_H */
diff -aurN u-boot-2016.05.orig/drivers/ddr/marvell/axp/ddr3_hw_training.h u-boot-2016.05-88fxxxx/drivers/ddr/marvell/axp/ddr3_hw_training.h
--- u-boot-2016.05.orig/drivers/ddr/marvell/axp/ddr3_hw_training.h	2016-05-16 07:40:32.000000000 -0700
+++ u-boot-2016.05-88fxxxx/drivers/ddr/marvell/axp/ddr3_hw_training.h	2016-05-21 11:33:32.848226195 -0700
@@ -296,11 +296,17 @@
 
 /* Board/Soc revisions define */
 enum board_rev {
+#ifdef CONFIG_ARMADA_370
+	A0,
+	A0_PCAC,
+	A0_RD
+#else
 	Z1,
 	Z1_PCAC,
 	Z1_RD_SLED,
 	A0,
 	A0_AMC
+#endif
 };
 
 typedef struct dram_modes {
diff -aurN u-boot-2016.05.orig/drivers/ddr/marvell/axp/ddr3_patterns_16bit.h u-boot-2016.05-88fxxxx/drivers/ddr/marvell/axp/ddr3_patterns_16bit.h
--- u-boot-2016.05.orig/drivers/ddr/marvell/axp/ddr3_patterns_16bit.h	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/drivers/ddr/marvell/axp/ddr3_patterns_16bit.h	2016-05-21 11:33:32.848226195 -0700
@@ -0,0 +1,877 @@
+/*
+ * Copyright (C) Marvell International Ltd. and its affiliates
+ *
+ * (C) Copyright 2016
+ * Steve Shih <shihsung@gmail.com>
+ * Extracted from ddr3_training.lib and bin_hdr.elf
+ *
+ * SPDX-License-Identifier:	GPL-2.0+
+ */
+
+#ifndef __DDR3_PATTERNS_16_H
+#define __DDR3_PATTERNS_16_H
+
+/*
+ * Patterns Declerations
+ */
+
+u32 rl_pattern[LEN_STD_PATTERN] __aligned(32) = {
+	0x00000000, 0x00000000, 0x00000000, 0x01010101
+};
+
+u32 wl_pattern[LEN_STD_PATTERN] __aligned(32) = {
+	0x00000000, 0x00000000, 0x00000000, 0x01010101
+};
+
+u32 wl_inv_pattern[LEN_STD_PATTERN] __aligned(32) = {
+	0xffffffff, 0xffffffff, 0xffffffff, 0xfefefefe
+};
+
+u32 pbs_pattern[2][LEN_PBS_PATTERN] __aligned(32) = {
+	{
+		0x5555aaaa, 0x5555aaaa, 0x5555aaaa, 0x5555aaaa
+	},
+	{
+		0xaaaa5555, 0xaaaa5555, 0xaaaa5555, 0xaaaa5555
+	}
+};
+
+u32 killer_pattern[DQ_NUM][LEN_KILLER_PATTERN] __aligned(32) = {
+	{
+		0x01010101, 0x01010101, 0x00000000, 0x00000000,
+		0x01010101, 0x01010101, 0xffffffff, 0xffffffff,
+		0xfefefefe, 0xfefefefe, 0xfefefefe, 0xfefefefe,
+		0x01010101, 0x01010101, 0xfefefefe, 0xfefefefe,
+		0x01010101, 0x01010101, 0xfefefefe, 0xfefefefe,
+		0x01010101, 0x01010101, 0x01010101, 0x01010101,
+		0xfefefefe, 0xfefefefe, 0x01010101, 0x01010101,
+		0xfefefefe, 0xfefefefe, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x01010101, 0x01010101,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0xfefefefe, 0xfefefefe,
+		0xfefefefe, 0xfefefefe, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0xfefefefe, 0xfefefefe, 0x00000000, 0x00000000,
+		0xfefefefe, 0xfefefefe, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0xffffffff, 0xffffffff, 0x01010101, 0x01010101,
+		0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
+		0x01010101, 0x01010101, 0x00000000, 0x00000000,
+		0x01010101, 0x01010101, 0xffffffff, 0xffffffff,
+		0xfefefefe, 0xfefefefe, 0xfefefefe, 0xfefefefe
+	},
+	{
+		0x02020202, 0x02020202, 0x00000000, 0x00000000,
+		0x02020202, 0x02020202, 0xffffffff, 0xffffffff,
+		0xfdfdfdfd, 0xfdfdfdfd, 0xfdfdfdfd, 0xfdfdfdfd,
+		0x02020202, 0x02020202, 0xfdfdfdfd, 0xfdfdfdfd,
+		0x02020202, 0x02020202, 0xfdfdfdfd, 0xfdfdfdfd,
+		0x02020202, 0x02020202, 0x02020202, 0x02020202,
+		0xfdfdfdfd, 0xfdfdfdfd, 0x02020202, 0x02020202,
+		0xfdfdfdfd, 0xfdfdfdfd, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x02020202, 0x02020202,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0xfdfdfdfd, 0xfdfdfdfd,
+		0xfdfdfdfd, 0xfdfdfdfd, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0xfdfdfdfd, 0xfdfdfdfd, 0x00000000, 0x00000000,
+		0xfdfdfdfd, 0xfdfdfdfd, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0xffffffff, 0xffffffff, 0x02020202, 0x02020202,
+		0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
+		0x02020202, 0x02020202, 0x00000000, 0x00000000,
+		0x02020202, 0x02020202, 0xffffffff, 0xffffffff,
+		0xfdfdfdfd, 0xfdfdfdfd, 0xfdfdfdfd, 0xfdfdfdfd
+	},
+	{
+		0x04040404, 0x04040404, 0x00000000, 0x00000000,
+		0x04040404, 0x04040404, 0xffffffff, 0xffffffff,
+		0xfbfbfbfb, 0xfbfbfbfb, 0xfbfbfbfb, 0xfbfbfbfb,
+		0x04040404, 0x04040404, 0xfbfbfbfb, 0xfbfbfbfb,
+		0x04040404, 0x04040404, 0xfbfbfbfb, 0xfbfbfbfb,
+		0x04040404, 0x04040404, 0x04040404, 0x04040404,
+		0xfbfbfbfb, 0xfbfbfbfb, 0x04040404, 0x04040404,
+		0xfbfbfbfb, 0xfbfbfbfb, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x04040404, 0x04040404,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0xfbfbfbfb, 0xfbfbfbfb,
+		0xfbfbfbfb, 0xfbfbfbfb, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0xfbfbfbfb, 0xfbfbfbfb, 0x00000000, 0x00000000,
+		0xfbfbfbfb, 0xfbfbfbfb, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0xffffffff, 0xffffffff, 0x04040404, 0x04040404,
+		0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
+		0x04040404, 0x04040404, 0x00000000, 0x00000000,
+		0x04040404, 0x04040404, 0xffffffff, 0xffffffff,
+		0xfbfbfbfb, 0xfbfbfbfb, 0xfbfbfbfb, 0xfbfbfbfb
+	},
+	{
+		0x08080808, 0x08080808, 0x00000000, 0x00000000,
+		0x08080808, 0x08080808, 0xffffffff, 0xffffffff,
+		0xf7f7f7f7, 0xf7f7f7f7, 0xf7f7f7f7, 0xf7f7f7f7,
+		0x08080808, 0x08080808, 0xf7f7f7f7, 0xf7f7f7f7,
+		0x08080808, 0x08080808, 0xf7f7f7f7, 0xf7f7f7f7,
+		0x08080808, 0x08080808, 0x08080808, 0x08080808,
+		0xf7f7f7f7, 0xf7f7f7f7, 0x08080808, 0x08080808,
+		0xf7f7f7f7, 0xf7f7f7f7, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x08080808, 0x08080808,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0xf7f7f7f7, 0xf7f7f7f7,
+		0xf7f7f7f7, 0xf7f7f7f7, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0xf7f7f7f7, 0xf7f7f7f7, 0x00000000, 0x00000000,
+		0xf7f7f7f7, 0xf7f7f7f7, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0xffffffff, 0xffffffff, 0x08080808, 0x08080808,
+		0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
+		0x08080808, 0x08080808, 0x00000000, 0x00000000,
+		0x08080808, 0x08080808, 0xffffffff, 0xffffffff,
+		0xf7f7f7f7, 0xf7f7f7f7, 0xf7f7f7f7, 0xf7f7f7f7
+	},
+	{
+		0x10101010, 0x10101010, 0x00000000, 0x00000000,
+		0x10101010, 0x10101010, 0xffffffff, 0xffffffff,
+		0xefefefef, 0xefefefef, 0xefefefef, 0xefefefef,
+		0x10101010, 0x10101010, 0xefefefef, 0xefefefef,
+		0x10101010, 0x10101010, 0xefefefef, 0xefefefef,
+		0x10101010, 0x10101010, 0x10101010, 0x10101010,
+		0xefefefef, 0xefefefef, 0x10101010, 0x10101010,
+		0xefefefef, 0xefefefef, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x10101010, 0x10101010,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0xefefefef, 0xefefefef,
+		0xefefefef, 0xefefefef, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0xefefefef, 0xefefefef, 0x00000000, 0x00000000,
+		0xefefefef, 0xefefefef, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0xffffffff, 0xffffffff, 0x10101010, 0x10101010,
+		0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
+		0x10101010, 0x10101010, 0x00000000, 0x00000000,
+		0x10101010, 0x10101010, 0xffffffff, 0xffffffff,
+		0xefefefef, 0xefefefef, 0xefefefef, 0xefefefef
+	},
+	{
+		0x20202020, 0x20202020, 0x00000000, 0x00000000,
+		0x20202020, 0x20202020, 0xffffffff, 0xffffffff,
+		0xdfdfdfdf, 0xdfdfdfdf, 0xdfdfdfdf, 0xdfdfdfdf,
+		0x20202020, 0x20202020, 0xdfdfdfdf, 0xdfdfdfdf,
+		0x20202020, 0x20202020, 0xdfdfdfdf, 0xdfdfdfdf,
+		0x20202020, 0x20202020, 0x20202020, 0x20202020,
+		0xdfdfdfdf, 0xdfdfdfdf, 0x20202020, 0x20202020,
+		0xdfdfdfdf, 0xdfdfdfdf, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x20202020, 0x20202020,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0xdfdfdfdf, 0xdfdfdfdf,
+		0xdfdfdfdf, 0xdfdfdfdf, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0xdfdfdfdf, 0xdfdfdfdf, 0x00000000, 0x00000000,
+		0xdfdfdfdf, 0xdfdfdfdf, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0xffffffff, 0xffffffff, 0x20202020, 0x20202020,
+		0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
+		0x20202020, 0x20202020, 0x00000000, 0x00000000,
+		0x20202020, 0x20202020, 0xffffffff, 0xffffffff,
+		0xdfdfdfdf, 0xdfdfdfdf, 0xdfdfdfdf, 0xdfdfdfdf
+	},
+	{
+		0x40404040, 0x40404040, 0x00000000, 0x00000000,
+		0x40404040, 0x40404040, 0xffffffff, 0xffffffff,
+		0xbfbfbfbf, 0xbfbfbfbf, 0xbfbfbfbf, 0xbfbfbfbf,
+		0x40404040, 0x40404040, 0xbfbfbfbf, 0xbfbfbfbf,
+		0x40404040, 0x40404040, 0xbfbfbfbf, 0xbfbfbfbf,
+		0x40404040, 0x40404040, 0x40404040, 0x40404040,
+		0xbfbfbfbf, 0xbfbfbfbf, 0x40404040, 0x40404040,
+		0xbfbfbfbf, 0xbfbfbfbf, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x40404040, 0x40404040,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0xbfbfbfbf, 0xbfbfbfbf,
+		0xbfbfbfbf, 0xbfbfbfbf, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0xbfbfbfbf, 0xbfbfbfbf, 0x00000000, 0x00000000,
+		0xbfbfbfbf, 0xbfbfbfbf, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0xffffffff, 0xffffffff, 0x40404040, 0x40404040,
+		0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
+		0x40404040, 0x40404040, 0x00000000, 0x00000000,
+		0x40404040, 0x40404040, 0xffffffff, 0xffffffff,
+		0xbfbfbfbf, 0xbfbfbfbf, 0xbfbfbfbf, 0xbfbfbfbf
+	},
+	{
+		0x80808080, 0x80808080, 0x00000000, 0x00000000,
+		0x80808080, 0x80808080, 0xffffffff, 0xffffffff,
+		0x7f7f7f7f, 0x7f7f7f7f, 0x7f7f7f7f, 0x7f7f7f7f,
+		0x80808080, 0x80808080, 0x7f7f7f7f, 0x7f7f7f7f,
+		0x80808080, 0x80808080, 0x7f7f7f7f, 0x7f7f7f7f,
+		0x80808080, 0x80808080, 0x80808080, 0x80808080,
+		0x7f7f7f7f, 0x7f7f7f7f, 0x80808080, 0x80808080,
+		0x7f7f7f7f, 0x7f7f7f7f, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x80808080, 0x80808080,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0x7f7f7f7f, 0x7f7f7f7f,
+		0x7f7f7f7f, 0x7f7f7f7f, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x7f7f7f7f, 0x7f7f7f7f, 0x00000000, 0x00000000,
+		0x7f7f7f7f, 0x7f7f7f7f, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0xffffffff, 0xffffffff, 0x80808080, 0x80808080,
+		0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
+		0x80808080, 0x80808080, 0x00000000, 0x00000000,
+		0x80808080, 0x80808080, 0xffffffff, 0xffffffff,
+		0x7f7f7f7f, 0x7f7f7f7f, 0x7f7f7f7f, 0x7f7f7f7f
+	}
+};
+
+u32 special_pattern[DQ_NUM][LEN_SPECIAL_PATTERN] __aligned(32) = {
+	{
+		0x00000000, 0x00000000, 0x01010101, 0x01010101,
+		0xffffffff, 0xffffffff, 0xfefefefe, 0xfefefefe,
+		0xfefefefe, 0xfefefefe, 0x01010101, 0x01010101,
+		0xfefefefe, 0xfefefefe, 0x01010101, 0x01010101,
+		0xfefefefe, 0xfefefefe, 0x01010101, 0x01010101,
+		0x01010101, 0x01010101, 0xfefefefe, 0xfefefefe,
+		0x01010101, 0x01010101, 0xfefefefe, 0xfefefefe,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x01010101, 0x01010101, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0xfefefefe, 0xfefefefe, 0xfefefefe, 0xfefefefe,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0xfefefefe, 0xfefefefe,
+		0x00000000, 0x00000000, 0xfefefefe, 0xfefefefe,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
+		0x01010101, 0x01010101, 0xffffffff, 0xffffffff,
+		0xffffffff, 0xffffffff, 0x01010101, 0x01010101,
+		0x00000000, 0x00000000, 0x01010101, 0x01010101,
+		0xffffffff, 0xffffffff, 0xfefefefe, 0xfefefefe,
+		0xfefefefe, 0xfefefefe, 0x00000000, 0x00000000,
+	},
+	{
+		0x00000000, 0x00000000, 0x02020202, 0x02020202,
+		0xffffffff, 0xffffffff, 0xfdfdfdfd, 0xfdfdfdfd,
+		0xfdfdfdfd, 0xfdfdfdfd, 0x02020202, 0x02020202,
+		0xfdfdfdfd, 0xfdfdfdfd, 0x02020202, 0x02020202,
+		0xfdfdfdfd, 0xfdfdfdfd, 0x02020202, 0x02020202,
+		0x02020202, 0x02020202, 0xfdfdfdfd, 0xfdfdfdfd,
+		0x02020202, 0x02020202, 0xfdfdfdfd, 0xfdfdfdfd,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x02020202, 0x02020202, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0xfdfdfdfd, 0xfdfdfdfd, 0xfdfdfdfd, 0xfdfdfdfd,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0xfdfdfdfd, 0xfdfdfdfd,
+		0x00000000, 0x00000000, 0xfdfdfdfd, 0xfdfdfdfd,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
+		0x02020202, 0x02020202, 0xffffffff, 0xffffffff,
+		0xffffffff, 0xffffffff, 0x02020202, 0x02020202,
+		0x00000000, 0x00000000, 0x02020202, 0x02020202,
+		0xffffffff, 0xffffffff, 0xfdfdfdfd, 0xfdfdfdfd,
+		0xfdfdfdfd, 0xfdfdfdfd, 0x00000000, 0x00000000,
+	},
+	{
+		0x00000000, 0x00000000, 0x04040404, 0x04040404,
+		0xffffffff, 0xffffffff, 0xfbfbfbfb, 0xfbfbfbfb,
+		0xfbfbfbfb, 0xfbfbfbfb, 0x04040404, 0x04040404,
+		0xfbfbfbfb, 0xfbfbfbfb, 0x04040404, 0x04040404,
+		0xfbfbfbfb, 0xfbfbfbfb, 0x04040404, 0x04040404,
+		0x04040404, 0x04040404, 0xfbfbfbfb, 0xfbfbfbfb,
+		0x04040404, 0x04040404, 0xfbfbfbfb, 0xfbfbfbfb,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x04040404, 0x04040404, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0xfbfbfbfb, 0xfbfbfbfb, 0xfbfbfbfb, 0xfbfbfbfb,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0xfbfbfbfb, 0xfbfbfbfb,
+		0x00000000, 0x00000000, 0xfbfbfbfb, 0xfbfbfbfb,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
+		0x04040404, 0x04040404, 0xffffffff, 0xffffffff,
+		0xffffffff, 0xffffffff, 0x04040404, 0x04040404,
+		0x00000000, 0x00000000, 0x04040404, 0x04040404,
+		0xffffffff, 0xffffffff, 0xfbfbfbfb, 0xfbfbfbfb,
+		0xfbfbfbfb, 0xfbfbfbfb, 0x00000000, 0x00000000,
+	},
+	{
+		0x00000000, 0x00000000, 0x08080808, 0x08080808,
+		0xffffffff, 0xffffffff, 0xf7f7f7f7, 0xf7f7f7f7,
+		0xf7f7f7f7, 0xf7f7f7f7, 0x08080808, 0x08080808,
+		0xf7f7f7f7, 0xf7f7f7f7, 0x08080808, 0x08080808,
+		0xf7f7f7f7, 0xf7f7f7f7, 0x08080808, 0x08080808,
+		0x08080808, 0x08080808, 0xf7f7f7f7, 0xf7f7f7f7,
+		0x08080808, 0x08080808, 0xf7f7f7f7, 0xf7f7f7f7,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x08080808, 0x08080808, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0xf7f7f7f7, 0xf7f7f7f7, 0xf7f7f7f7, 0xf7f7f7f7,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0xf7f7f7f7, 0xf7f7f7f7,
+		0x00000000, 0x00000000, 0xf7f7f7f7, 0xf7f7f7f7,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
+		0x08080808, 0x08080808, 0xffffffff, 0xffffffff,
+		0xffffffff, 0xffffffff, 0x08080808, 0x08080808,
+		0x00000000, 0x00000000, 0x08080808, 0x08080808,
+		0xffffffff, 0xffffffff, 0xf7f7f7f7, 0xf7f7f7f7,
+		0xf7f7f7f7, 0xf7f7f7f7, 0x00000000, 0x00000000,
+	},
+	{
+		0x00000000, 0x00000000, 0x10101010, 0x10101010,
+		0xffffffff, 0xffffffff, 0xefefefef, 0xefefefef,
+		0xefefefef, 0xefefefef, 0x10101010, 0x10101010,
+		0xefefefef, 0xefefefef, 0x10101010, 0x10101010,
+		0xefefefef, 0xefefefef, 0x10101010, 0x10101010,
+		0x10101010, 0x10101010, 0xefefefef, 0xefefefef,
+		0x10101010, 0x10101010, 0xefefefef, 0xefefefef,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x10101010, 0x10101010, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0xefefefef, 0xefefefef, 0xefefefef, 0xefefefef,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0xefefefef, 0xefefefef,
+		0x00000000, 0x00000000, 0xefefefef, 0xefefefef,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
+		0x10101010, 0x10101010, 0xffffffff, 0xffffffff,
+		0xffffffff, 0xffffffff, 0x10101010, 0x10101010,
+		0x00000000, 0x00000000, 0x10101010, 0x10101010,
+		0xffffffff, 0xffffffff, 0xefefefef, 0xefefefef,
+		0xefefefef, 0xefefefef, 0x00000000, 0x00000000,
+	},
+	{
+		0x00000000, 0x00000000, 0x20202020, 0x20202020,
+		0xffffffff, 0xffffffff, 0xdfdfdfdf, 0xdfdfdfdf,
+		0xdfdfdfdf, 0xdfdfdfdf, 0x20202020, 0x20202020,
+		0xdfdfdfdf, 0xdfdfdfdf, 0x20202020, 0x20202020,
+		0xdfdfdfdf, 0xdfdfdfdf, 0x20202020, 0x20202020,
+		0x20202020, 0x20202020, 0xdfdfdfdf, 0xdfdfdfdf,
+		0x20202020, 0x20202020, 0xdfdfdfdf, 0xdfdfdfdf,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x20202020, 0x20202020, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0xdfdfdfdf, 0xdfdfdfdf, 0xdfdfdfdf, 0xdfdfdfdf,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0xdfdfdfdf, 0xdfdfdfdf,
+		0x00000000, 0x00000000, 0xdfdfdfdf, 0xdfdfdfdf,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
+		0x20202020, 0x20202020, 0xffffffff, 0xffffffff,
+		0xffffffff, 0xffffffff, 0x20202020, 0x20202020,
+		0x00000000, 0x00000000, 0x20202020, 0x20202020,
+		0xffffffff, 0xffffffff, 0xdfdfdfdf, 0xdfdfdfdf,
+		0xdfdfdfdf, 0xdfdfdfdf, 0x00000000, 0x00000000,
+	},
+	{
+		0x00000000, 0x00000000, 0x40404040, 0x40404040,
+		0xffffffff, 0xffffffff, 0xbfbfbfbf, 0xbfbfbfbf,
+		0xbfbfbfbf, 0xbfbfbfbf, 0x40404040, 0x40404040,
+		0xbfbfbfbf, 0xbfbfbfbf, 0x40404040, 0x40404040,
+		0xbfbfbfbf, 0xbfbfbfbf, 0x40404040, 0x40404040,
+		0x40404040, 0x40404040, 0xbfbfbfbf, 0xbfbfbfbf,
+		0x40404040, 0x40404040, 0xbfbfbfbf, 0xbfbfbfbf,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x40404040, 0x40404040, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0xbfbfbfbf, 0xbfbfbfbf, 0xbfbfbfbf, 0xbfbfbfbf,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0xbfbfbfbf, 0xbfbfbfbf,
+		0x00000000, 0x00000000, 0xbfbfbfbf, 0xbfbfbfbf,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
+		0x40404040, 0x40404040, 0xffffffff, 0xffffffff,
+		0xffffffff, 0xffffffff, 0x40404040, 0x40404040,
+		0x00000000, 0x00000000, 0x40404040, 0x40404040,
+		0xffffffff, 0xffffffff, 0xbfbfbfbf, 0xbfbfbfbf,
+		0xbfbfbfbf, 0xbfbfbfbf, 0x00000000, 0x00000000,
+	},
+	{
+		0x00000000, 0x00000000, 0x80808080, 0x80808080,
+		0xffffffff, 0xffffffff, 0x7f7f7f7f, 0x7f7f7f7f,
+		0x7f7f7f7f, 0x7f7f7f7f, 0x80808080, 0x80808080,
+		0x7f7f7f7f, 0x7f7f7f7f, 0x80808080, 0x80808080,
+		0x7f7f7f7f, 0x7f7f7f7f, 0x80808080, 0x80808080,
+		0x80808080, 0x80808080, 0x7f7f7f7f, 0x7f7f7f7f,
+		0x80808080, 0x80808080, 0x7f7f7f7f, 0x7f7f7f7f,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0x80808080, 0x80808080, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0x7f7f7f7f, 0x7f7f7f7f, 0x7f7f7f7f, 0x7f7f7f7f,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0x7f7f7f7f, 0x7f7f7f7f,
+		0x00000000, 0x00000000, 0x7f7f7f7f, 0x7f7f7f7f,
+		0x00000000, 0x00000000, 0x00000000, 0x00000000,
+		0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
+		0x80808080, 0x80808080, 0xffffffff, 0xffffffff,
+		0xffffffff, 0xffffffff, 0x80808080, 0x80808080,
+		0x00000000, 0x00000000, 0x80808080, 0x80808080,
+		0xffffffff, 0xffffffff, 0x7f7f7f7f, 0x7f7f7f7f,
+		0x7f7f7f7f, 0x7f7f7f7f, 0x00000000, 0x00000000,
+	}
+};
+
+/* taken from ddr3_patterns_64bit.h */
+u32 wl_sup_pattern[LEN_WL_SUP_PATTERN] __aligned(32) = {
+	0x04030201, 0x08070605, 0x0c0b0a09, 0x100f0e0d,
+	0x14131211, 0x18171615, 0x1c1b1a19, 0x201f1e1d,
+	0x24232221, 0x28272625, 0x2c2b2a29, 0x302f2e2d,
+	0x34333231, 0x38373635, 0x3c3b3a39, 0x403f3e3d,
+	0x44434241, 0x48474645, 0x4c4b4a49, 0x504f4e4d,
+	0x54535251, 0x58575655, 0x5c5b5a59, 0x605f5e5d,
+	0x64636261, 0x68676665, 0x6c6b6a69, 0x706f6e6d,
+	0x74737271, 0x78777675, 0x7c7b7a79, 0x807f7e7d
+};
+
+/* taken from ddr3_patterns_64bit.h */
+u32 killer_pattern_32b[DQ_NUM][LEN_KILLER_PATTERN] __aligned(32) = {
+	{
+		0x01010101, 0x00000000, 0x01010101, 0xFFFFFFFF,
+		0x01010101, 0x00000000, 0x01010101, 0xFFFFFFFF,
+		0xFEFEFEFE, 0xFEFEFEFE, 0x01010101, 0xFEFEFEFE,
+		0xFEFEFEFE, 0xFEFEFEFE, 0x01010101, 0xFEFEFEFE,
+		0x01010101, 0xFEFEFEFE, 0x01010101, 0x01010101,
+		0x01010101, 0xFEFEFEFE, 0x01010101, 0x01010101,
+		0xFEFEFEFE, 0x01010101, 0xFEFEFEFE, 0x00000000,
+		0xFEFEFEFE, 0x01010101, 0xFEFEFEFE, 0x00000000,
+		0xFFFFFFFF, 0x00000000, 0xFFFFFFFF, 0x00000000,
+		0xFFFFFFFF, 0x00000000, 0xFFFFFFFF, 0x00000000,
+		0xFFFFFFFF, 0x00000000, 0xFFFFFFFF, 0x01010101,
+		0xFFFFFFFF, 0x00000000, 0xFFFFFFFF, 0x01010101,
+		0x00000000, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0x00000000, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0x00000000, 0x00000000, 0x00000000, 0xFEFEFEFE,
+		0x00000000, 0x00000000, 0x00000000, 0xFEFEFEFE,
+		0xFEFEFEFE, 0xFFFFFFFF, 0x00000000, 0x00000000,
+		0xFEFEFEFE, 0xFFFFFFFF, 0x00000000, 0x00000000,
+		0xFFFFFFFF, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0xFFFFFFFF, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x00000000,
+		0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x00000000,
+		0x00000000, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0x00000000, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0xFEFEFEFE, 0x00000000, 0xFEFEFEFE, 0x00000000,
+		0xFEFEFEFE, 0x00000000, 0xFEFEFEFE, 0x00000000,
+		0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x01010101,
+		0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x01010101,
+		0xFFFFFFFF, 0xFFFFFFFF, 0x01010101, 0x00000000,
+		0xFFFFFFFF, 0xFFFFFFFF, 0x01010101, 0x00000000,
+		0x01010101, 0xFFFFFFFF, 0xFEFEFEFE, 0xFEFEFEFE,
+		0x01010101, 0xFFFFFFFF, 0xFEFEFEFE, 0xFEFEFEFE
+	},
+	{
+		0x02020202, 0x00000000, 0x02020202, 0xFFFFFFFF,
+		0x02020202, 0x00000000, 0x02020202, 0xFFFFFFFF,
+		0xFDFDFDFD, 0xFDFDFDFD, 0x02020202, 0xFDFDFDFD,
+		0xFDFDFDFD, 0xFDFDFDFD, 0x02020202, 0xFDFDFDFD,
+		0x02020202, 0xFDFDFDFD, 0x02020202, 0x02020202,
+		0x02020202, 0xFDFDFDFD, 0x02020202, 0x02020202,
+		0xFDFDFDFD, 0x02020202, 0xFDFDFDFD, 0x00000000,
+		0xFDFDFDFD, 0x02020202, 0xFDFDFDFD, 0x00000000,
+		0xFFFFFFFF, 0x00000000, 0xFFFFFFFF, 0x00000000,
+		0xFFFFFFFF, 0x00000000, 0xFFFFFFFF, 0x00000000,
+		0xFFFFFFFF, 0x00000000, 0xFFFFFFFF, 0x02020202,
+		0xFFFFFFFF, 0x00000000, 0xFFFFFFFF, 0x02020202,
+		0x00000000, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0x00000000, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0x00000000, 0x00000000, 0x00000000, 0xFDFDFDFD,
+		0x00000000, 0x00000000, 0x00000000, 0xFDFDFDFD,
+		0xFDFDFDFD, 0xFFFFFFFF, 0x00000000, 0x00000000,
+		0xFDFDFDFD, 0xFFFFFFFF, 0x00000000, 0x00000000,
+		0xFFFFFFFF, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0xFFFFFFFF, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x00000000,
+		0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x00000000,
+		0x00000000, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0x00000000, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0xFDFDFDFD, 0x00000000, 0xFDFDFDFD, 0x00000000,
+		0xFDFDFDFD, 0x00000000, 0xFDFDFDFD, 0x00000000,
+		0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x02020202,
+		0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x02020202,
+		0xFFFFFFFF, 0xFFFFFFFF, 0x02020202, 0x00000000,
+		0xFFFFFFFF, 0xFFFFFFFF, 0x02020202, 0x00000000,
+		0x02020202, 0xFFFFFFFF, 0xFDFDFDFD, 0xFDFDFDFD,
+		0x02020202, 0xFFFFFFFF, 0xFDFDFDFD, 0xFDFDFDFD
+	},
+	{
+		0x04040404, 0x00000000, 0x04040404, 0xFFFFFFFF,
+		0x04040404, 0x00000000, 0x04040404, 0xFFFFFFFF,
+		0xFBFBFBFB, 0xFBFBFBFB, 0x04040404, 0xFBFBFBFB,
+		0xFBFBFBFB, 0xFBFBFBFB, 0x04040404, 0xFBFBFBFB,
+		0x04040404, 0xFBFBFBFB, 0x04040404, 0x04040404,
+		0x04040404, 0xFBFBFBFB, 0x04040404, 0x04040404,
+		0xFBFBFBFB, 0x04040404, 0xFBFBFBFB, 0x00000000,
+		0xFBFBFBFB, 0x04040404, 0xFBFBFBFB, 0x00000000,
+		0xFFFFFFFF, 0x00000000, 0xFFFFFFFF, 0x00000000,
+		0xFFFFFFFF, 0x00000000, 0xFFFFFFFF, 0x00000000,
+		0xFFFFFFFF, 0x00000000, 0xFFFFFFFF, 0x04040404,
+		0xFFFFFFFF, 0x00000000, 0xFFFFFFFF, 0x04040404,
+		0x00000000, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0x00000000, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0x00000000, 0x00000000, 0x00000000, 0xFBFBFBFB,
+		0x00000000, 0x00000000, 0x00000000, 0xFBFBFBFB,
+		0xFBFBFBFB, 0xFFFFFFFF, 0x00000000, 0x00000000,
+		0xFBFBFBFB, 0xFFFFFFFF, 0x00000000, 0x00000000,
+		0xFFFFFFFF, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0xFFFFFFFF, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x00000000,
+		0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x00000000,
+		0x00000000, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0x00000000, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0xFBFBFBFB, 0x00000000, 0xFBFBFBFB, 0x00000000,
+		0xFBFBFBFB, 0x00000000, 0xFBFBFBFB, 0x00000000,
+		0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x04040404,
+		0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x04040404,
+		0xFFFFFFFF, 0xFFFFFFFF, 0x04040404, 0x00000000,
+		0xFFFFFFFF, 0xFFFFFFFF, 0x04040404, 0x00000000,
+		0x04040404, 0xFFFFFFFF, 0xFBFBFBFB, 0xFBFBFBFB,
+		0x04040404, 0xFFFFFFFF, 0xFBFBFBFB, 0xFBFBFBFB
+	},
+	{
+		0x08080808, 0x00000000, 0x08080808, 0xFFFFFFFF,
+		0x08080808, 0x00000000, 0x08080808, 0xFFFFFFFF,
+		0xF7F7F7F7, 0xF7F7F7F7, 0x08080808, 0xF7F7F7F7,
+		0xF7F7F7F7, 0xF7F7F7F7, 0x08080808, 0xF7F7F7F7,
+		0x08080808, 0xF7F7F7F7, 0x08080808, 0x08080808,
+		0x08080808, 0xF7F7F7F7, 0x08080808, 0x08080808,
+		0xF7F7F7F7, 0x08080808, 0xF7F7F7F7, 0x00000000,
+		0xF7F7F7F7, 0x08080808, 0xF7F7F7F7, 0x00000000,
+		0xFFFFFFFF, 0x00000000, 0xFFFFFFFF, 0x00000000,
+		0xFFFFFFFF, 0x00000000, 0xFFFFFFFF, 0x00000000,
+		0xFFFFFFFF, 0x00000000, 0xFFFFFFFF, 0x08080808,
+		0xFFFFFFFF, 0x00000000, 0xFFFFFFFF, 0x08080808,
+		0x00000000, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0x00000000, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0x00000000, 0x00000000, 0x00000000, 0xF7F7F7F7,
+		0x00000000, 0x00000000, 0x00000000, 0xF7F7F7F7,
+		0xF7F7F7F7, 0xFFFFFFFF, 0x00000000, 0x00000000,
+		0xF7F7F7F7, 0xFFFFFFFF, 0x00000000, 0x00000000,
+		0xFFFFFFFF, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0xFFFFFFFF, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x00000000,
+		0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x00000000,
+		0x00000000, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0x00000000, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0xF7F7F7F7, 0x00000000, 0xF7F7F7F7, 0x00000000,
+		0xF7F7F7F7, 0x00000000, 0xF7F7F7F7, 0x00000000,
+		0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x08080808,
+		0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x08080808,
+		0xFFFFFFFF, 0xFFFFFFFF, 0x08080808, 0x00000000,
+		0xFFFFFFFF, 0xFFFFFFFF, 0x08080808, 0x00000000,
+		0x08080808, 0xFFFFFFFF, 0xF7F7F7F7, 0xF7F7F7F7,
+		0x08080808, 0xFFFFFFFF, 0xF7F7F7F7, 0xF7F7F7F7
+	},
+	{
+		0x10101010, 0x00000000, 0x10101010, 0xFFFFFFFF,
+		0x10101010, 0x00000000, 0x10101010, 0xFFFFFFFF,
+		0xEFEFEFEF, 0xEFEFEFEF, 0x10101010, 0xEFEFEFEF,
+		0xEFEFEFEF, 0xEFEFEFEF, 0x10101010, 0xEFEFEFEF,
+		0x10101010, 0xEFEFEFEF, 0x10101010, 0x10101010,
+		0x10101010, 0xEFEFEFEF, 0x10101010, 0x10101010,
+		0xEFEFEFEF, 0x10101010, 0xEFEFEFEF, 0x00000000,
+		0xEFEFEFEF, 0x10101010, 0xEFEFEFEF, 0x00000000,
+		0xFFFFFFFF, 0x00000000, 0xFFFFFFFF, 0x00000000,
+		0xFFFFFFFF, 0x00000000, 0xFFFFFFFF, 0x00000000,
+		0xFFFFFFFF, 0x00000000, 0xFFFFFFFF, 0x10101010,
+		0xFFFFFFFF, 0x00000000, 0xFFFFFFFF, 0x10101010,
+		0x00000000, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0x00000000, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0x00000000, 0x00000000, 0x00000000, 0xEFEFEFEF,
+		0x00000000, 0x00000000, 0x00000000, 0xEFEFEFEF,
+		0xEFEFEFEF, 0xFFFFFFFF, 0x00000000, 0x00000000,
+		0xEFEFEFEF, 0xFFFFFFFF, 0x00000000, 0x00000000,
+		0xFFFFFFFF, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0xFFFFFFFF, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x00000000,
+		0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x00000000,
+		0x00000000, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0x00000000, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0xEFEFEFEF, 0x00000000, 0xEFEFEFEF, 0x00000000,
+		0xEFEFEFEF, 0x00000000, 0xEFEFEFEF, 0x00000000,
+		0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x10101010,
+		0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x10101010,
+		0xFFFFFFFF, 0xFFFFFFFF, 0x10101010, 0x00000000,
+		0xFFFFFFFF, 0xFFFFFFFF, 0x10101010, 0x00000000,
+		0x10101010, 0xFFFFFFFF, 0xEFEFEFEF, 0xEFEFEFEF,
+		0x10101010, 0xFFFFFFFF, 0xEFEFEFEF, 0xEFEFEFEF
+	},
+	{
+		0x20202020, 0x00000000, 0x20202020, 0xFFFFFFFF,
+		0x20202020, 0x00000000, 0x20202020, 0xFFFFFFFF,
+		0xDFDFDFDF, 0xDFDFDFDF, 0x20202020, 0xDFDFDFDF,
+		0xDFDFDFDF, 0xDFDFDFDF, 0x20202020, 0xDFDFDFDF,
+		0x20202020, 0xDFDFDFDF, 0x20202020, 0x20202020,
+		0x20202020, 0xDFDFDFDF, 0x20202020, 0x20202020,
+		0xDFDFDFDF, 0x20202020, 0xDFDFDFDF, 0x00000000,
+		0xDFDFDFDF, 0x20202020, 0xDFDFDFDF, 0x00000000,
+		0xFFFFFFFF, 0x00000000, 0xFFFFFFFF, 0x00000000,
+		0xFFFFFFFF, 0x00000000, 0xFFFFFFFF, 0x00000000,
+		0xFFFFFFFF, 0x00000000, 0xFFFFFFFF, 0x20202020,
+		0xFFFFFFFF, 0x00000000, 0xFFFFFFFF, 0x20202020,
+		0x00000000, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0x00000000, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0x00000000, 0x00000000, 0x00000000, 0xDFDFDFDF,
+		0x00000000, 0x00000000, 0x00000000, 0xDFDFDFDF,
+		0xDFDFDFDF, 0xFFFFFFFF, 0x00000000, 0x00000000,
+		0xDFDFDFDF, 0xFFFFFFFF, 0x00000000, 0x00000000,
+		0xFFFFFFFF, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0xFFFFFFFF, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x00000000,
+		0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x00000000,
+		0x00000000, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0x00000000, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0xDFDFDFDF, 0x00000000, 0xDFDFDFDF, 0x00000000,
+		0xDFDFDFDF, 0x00000000, 0xDFDFDFDF, 0x00000000,
+		0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x20202020,
+		0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x20202020,
+		0xFFFFFFFF, 0xFFFFFFFF, 0x20202020, 0x00000000,
+		0xFFFFFFFF, 0xFFFFFFFF, 0x20202020, 0x00000000,
+		0x20202020, 0xFFFFFFFF, 0xDFDFDFDF, 0xDFDFDFDF,
+		0x20202020, 0xFFFFFFFF, 0xDFDFDFDF, 0xDFDFDFDF
+	},
+	{
+		0x40404040, 0x00000000, 0x40404040, 0xFFFFFFFF,
+		0x40404040, 0x00000000, 0x40404040, 0xFFFFFFFF,
+		0xBFBFBFBF, 0xBFBFBFBF, 0x40404040, 0xBFBFBFBF,
+		0xBFBFBFBF, 0xBFBFBFBF, 0x40404040, 0xBFBFBFBF,
+		0x40404040, 0xBFBFBFBF, 0x40404040, 0x40404040,
+		0x40404040, 0xBFBFBFBF, 0x40404040, 0x40404040,
+		0xBFBFBFBF, 0x40404040, 0xBFBFBFBF, 0x00000000,
+		0xBFBFBFBF, 0x40404040, 0xBFBFBFBF, 0x00000000,
+		0xFFFFFFFF, 0x00000000, 0xFFFFFFFF, 0x00000000,
+		0xFFFFFFFF, 0x00000000, 0xFFFFFFFF, 0x00000000,
+		0xFFFFFFFF, 0x00000000, 0xFFFFFFFF, 0x40404040,
+		0xFFFFFFFF, 0x00000000, 0xFFFFFFFF, 0x40404040,
+		0x00000000, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0x00000000, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0x00000000, 0x00000000, 0x00000000, 0xBFBFBFBF,
+		0x00000000, 0x00000000, 0x00000000, 0xBFBFBFBF,
+		0xBFBFBFBF, 0xFFFFFFFF, 0x00000000, 0x00000000,
+		0xBFBFBFBF, 0xFFFFFFFF, 0x00000000, 0x00000000,
+		0xFFFFFFFF, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0xFFFFFFFF, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x00000000,
+		0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x00000000,
+		0x00000000, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0x00000000, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0xBFBFBFBF, 0x00000000, 0xBFBFBFBF, 0x00000000,
+		0xBFBFBFBF, 0x00000000, 0xBFBFBFBF, 0x00000000,
+		0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x40404040,
+		0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x40404040,
+		0xFFFFFFFF, 0xFFFFFFFF, 0x40404040, 0x00000000,
+		0xFFFFFFFF, 0xFFFFFFFF, 0x40404040, 0x00000000,
+		0x40404040, 0xFFFFFFFF, 0xBFBFBFBF, 0xBFBFBFBF,
+		0x40404040, 0xFFFFFFFF, 0xBFBFBFBF, 0xBFBFBFBF
+	},
+	{
+		0x80808080, 0x00000000, 0x80808080, 0xFFFFFFFF,
+		0x80808080, 0x00000000, 0x80808080, 0xFFFFFFFF,
+		0x7F7F7F7F, 0x7F7F7F7F, 0x80808080, 0x7F7F7F7F,
+		0x7F7F7F7F, 0x7F7F7F7F, 0x80808080, 0x7F7F7F7F,
+		0x80808080, 0x7F7F7F7F, 0x80808080, 0x80808080,
+		0x80808080, 0x7F7F7F7F, 0x80808080, 0x80808080,
+		0x7F7F7F7F, 0x80808080, 0x7F7F7F7F, 0x00000000,
+		0x7F7F7F7F, 0x80808080, 0x7F7F7F7F, 0x00000000,
+		0xFFFFFFFF, 0x00000000, 0xFFFFFFFF, 0x00000000,
+		0xFFFFFFFF, 0x00000000, 0xFFFFFFFF, 0x00000000,
+		0xFFFFFFFF, 0x00000000, 0xFFFFFFFF, 0x80808080,
+		0xFFFFFFFF, 0x00000000, 0xFFFFFFFF, 0x80808080,
+		0x00000000, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0x00000000, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0x00000000, 0x00000000, 0x00000000, 0x7F7F7F7F,
+		0x00000000, 0x00000000, 0x00000000, 0x7F7F7F7F,
+		0x7F7F7F7F, 0xFFFFFFFF, 0x00000000, 0x00000000,
+		0x7F7F7F7F, 0xFFFFFFFF, 0x00000000, 0x00000000,
+		0xFFFFFFFF, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0xFFFFFFFF, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x00000000,
+		0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x00000000,
+		0x00000000, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0x00000000, 0xFFFFFFFF, 0x00000000, 0xFFFFFFFF,
+		0x7F7F7F7F, 0x00000000, 0x7F7F7F7F, 0x00000000,
+		0x7F7F7F7F, 0x00000000, 0x7F7F7F7F, 0x00000000,
+		0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x80808080,
+		0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x80808080,
+		0xFFFFFFFF, 0xFFFFFFFF, 0x80808080, 0x00000000,
+		0xFFFFFFFF, 0xFFFFFFFF, 0x80808080, 0x00000000,
+		0x80808080, 0xFFFFFFFF, 0x7F7F7F7F, 0x7F7F7F7F,
+		0x80808080, 0xFFFFFFFF, 0x7F7F7F7F, 0x7F7F7F7F
+	}
+};
+
+#endif /* __DDR3_PATTERNS_16_H */
diff -aurN u-boot-2016.05.orig/drivers/Makefile u-boot-2016.05-88fxxxx/drivers/Makefile
--- u-boot-2016.05.orig/drivers/Makefile	2016-05-16 07:40:32.000000000 -0700
+++ u-boot-2016.05-88fxxxx/drivers/Makefile	2016-05-21 11:33:32.848226195 -0700
@@ -14,6 +14,7 @@
 obj-$(CONFIG_SPL_GPIO_SUPPORT) += gpio/
 obj-$(CONFIG_SPL_MMC_SUPPORT) += mmc/
 obj-$(CONFIG_SPL_MPC8XXX_INIT_DDR_SUPPORT) += ddr/fsl/
+obj-$(CONFIG_ARMADA_370) += ddr/marvell/axp/
 obj-$(CONFIG_ARMADA_38X) += ddr/marvell/a38x/
 obj-$(CONFIG_ARMADA_XP) += ddr/marvell/axp/
 obj-$(CONFIG_ALTERA_SDRAM) += ddr/altera/
diff -aurN u-boot-2016.05.orig/drivers/mtd/nand/Makefile u-boot-2016.05-88fxxxx/drivers/mtd/nand/Makefile
--- u-boot-2016.05.orig/drivers/mtd/nand/Makefile	2016-05-16 07:40:32.000000000 -0700
+++ u-boot-2016.05-88fxxxx/drivers/mtd/nand/Makefile	2016-05-21 11:33:32.848226195 -0700
@@ -67,6 +67,7 @@
 obj-$(CONFIG_TEGRA_NAND) += tegra_nand.o
 obj-$(CONFIG_NAND_OMAP_GPMC) += omap_gpmc.o
 obj-$(CONFIG_NAND_OMAP_ELM) += omap_elm.o
+obj-$(CONFIG_NAND_ORION) += orion_nand.o
 obj-$(CONFIG_NAND_PLAT) += nand_plat.o
 obj-$(CONFIG_NAND_DOCG4) += docg4.o
 
diff -aurN u-boot-2016.05.orig/drivers/mtd/nand/orion_nand.c u-boot-2016.05-88fxxxx/drivers/mtd/nand/orion_nand.c
--- u-boot-2016.05.orig/drivers/mtd/nand/orion_nand.c	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/drivers/mtd/nand/orion_nand.c	2016-05-21 11:33:32.848226195 -0700
@@ -0,0 +1,98 @@
+/*
+ * (C) Copyright 2012
+ * Steve Shih <shihsung@gmail.com>
+ *
+ * SPDX-License-Identifier:	GPL-2.0+
+ */
+
+#include <common.h>
+#include <nand.h>
+#include <malloc.h>
+#include <asm/io.h>
+
+#define NF_CLE			0x01	/* Commands with A[1:0] == 01 */
+#define NF_ALE			0x02	/* Addresses with A[1:0] == 10 */
+
+#define NF_CTRL			(u32*)(ORION5X_REGS_PHY_BASE + 0x104e8)
+#define NF_CTRL_NF0		0x00000004
+#define NF_CTRL_NFACTCEN0	0x00000008
+
+#ifndef CONFIG_SYS_NAND_BASE_LIST
+#define CONFIG_SYS_NAND_BASE_LIST	{ CONFIG_SYS_NAND_BASE }
+#endif
+
+#ifndef CONFIG_SYS_NAND_OPTIONS
+#define CONFIG_SYS_NAND_OPTIONS	(NAND_COPYBACK|NAND_CACHEPRG|NAND_NO_PADDING)
+#endif
+
+#ifndef CONFIG_SYS_NAND_DELAY
+#define CONFIG_SYS_NAND_DELAY	25	/* 25 micro seconds */
+#endif
+
+static void nand_cmd_ctrl (struct mtd_info *mtd, int cmd, unsigned ctrl)
+{
+	struct nand_chip *nc = mtd->priv;
+	u32 offs;
+
+	if (cmd == NAND_CMD_NONE)
+		return;
+
+	if (ctrl & NAND_CLE)
+		offs = NF_CLE;
+	else if (ctrl & NAND_ALE)
+		offs = NF_ALE;
+	else
+		return;
+
+	if (nc->options & NAND_BUSWIDTH_16)
+		offs <<= 1;
+
+	writeb(cmd, nc->IO_ADDR_W + offs);
+}
+
+static void nand_select_chip (struct mtd_info *mtd, int chip)
+{
+	u32 data;
+
+	data = readl(NF_CTRL);
+	data |= NF_CTRL_NF0 | NF_CTRL_NFACTCEN0;
+	writel(data, NF_CTRL);
+}
+
+void board_nand_init (void)
+{
+	ulong io_addr[CONFIG_SYS_MAX_NAND_DEVICE] = CONFIG_SYS_NAND_BASE_LIST;
+	struct nand_chip *chip;
+	struct mtd_info *mtd;
+	int i;
+
+	for (i = 0; i < CONFIG_SYS_MAX_NAND_DEVICE; i++) {
+		chip = malloc(sizeof(*chip));
+		if (!chip) {
+			printf("alloc nand resource failed\n");
+			return;
+		}
+
+		memset(chip, 0, sizeof(*chip));
+		mtd = &nand_info[i];
+		mtd->priv = chip;
+		chip->IO_ADDR_R = chip->IO_ADDR_W = (void __iomem *)io_addr[i];
+
+		chip->options = CONFIG_SYS_NAND_OPTIONS;
+#if defined(CONFIG_NAND_ECC_BCH)
+		chip->ecc.mode = NAND_ECC_SOFT_BCH;
+#else
+		chip->ecc.mode = NAND_ECC_SOFT;
+#endif
+		chip->cmd_ctrl = nand_cmd_ctrl;
+		chip->chip_delay = CONFIG_SYS_NAND_DELAY;
+		chip->select_chip = nand_select_chip;
+
+		if (nand_scan(mtd, i)) {
+			free(chip);
+			continue;
+		}
+
+		nand_register(i);
+	}
+}
diff -aurN u-boot-2016.05.orig/drivers/mtd/nand/pxa3xx_nand.c u-boot-2016.05-88fxxxx/drivers/mtd/nand/pxa3xx_nand.c
--- u-boot-2016.05.orig/drivers/mtd/nand/pxa3xx_nand.c	2016-05-16 07:40:32.000000000 -0700
+++ u-boot-2016.05-88fxxxx/drivers/mtd/nand/pxa3xx_nand.c	2016-05-21 11:33:32.848226195 -0700
@@ -217,6 +217,10 @@
 	{ 10,  0, 20,  40, 30,  40, 11123, 110, 10, },
 	{ 10, 25, 15,  25, 15,  30, 25000,  60, 10, },
 	{ 10, 35, 15,  25, 15,  25, 25000,  60, 10, },
+	/* Hynix H27U4G8F2D 4Gbit(512M x 8bit) 3.0V(ID: 0xDCAD) */
+	{  5, 20, 10,  12, 10,  12, 25000,  60, 10, },
+	/* Toshiba TC58NVG2S3ETAI0 4Gbit(512M x 8bit) 3.3V(ID: 0xDC98) */
+	{  5, 20, 10,  12, 10,  12, 30000,  60, 10, },
 };
 
 static struct pxa3xx_nand_flash builtin_flash_types[] = {
@@ -228,8 +232,13 @@
 	{ 0xdc2c,  8,  8, &timing[2] },
 	{ 0xcc2c, 16, 16, &timing[2] },
 	{ 0xba20, 16, 16, &timing[3] },
+	/* Hynix H27U4G8F2D 4Gbit(512M x 8bit) 3.0V(ID: 0xDCAD) */
+	{ 0xdcad,  8,  8, &timing[4] },
+	/* Toshiba TC58NVG2S3ETAI0 4Gbit(512M x 8bit) 3.3V(ID: 0xDC98) */
+	{ 0xdc98,  8,  8, &timing[5] },
 };
 
+#ifdef CONFIG_SYS_NAND_USE_FLASH_BBT
 static u8 bbt_pattern[] = {'M', 'V', 'B', 'b', 't', '0' };
 static u8 bbt_mirror_pattern[] = {'1', 't', 'b', 'B', 'V', 'M' };
 
@@ -252,6 +261,7 @@
 	.maxblocks = 8,		/* Last 8 blocks in each chip */
 	.pattern = bbt_mirror_pattern
 };
+#endif
 
 static struct nand_ecclayout ecc_layout_2KB_bch4bit = {
 	.eccbytes = 32,
@@ -300,7 +310,7 @@
 #define NDTR1_tAR(c)	(min((c), 15) << 0)
 
 /* convert nano-seconds to nand flash controller clock cycles */
-#define ns2cycle(ns, clk)	(int)((ns) * (clk / 1000000) / 1000)
+#define ns2cycle(ns, clk)	(int)DIV_ROUND_UP((ns) * (clk / 1000000), 1000)
 
 static enum pxa3xx_nand_variant pxa3xx_nand_get_variant(void)
 {
@@ -460,8 +470,8 @@
 			nand_writel(info, NDECCCTRL, 0x1);
 	} else {
 		ndcr &= ~NDCR_ECC_EN;
-		if (info->ecc_bch)
-			nand_writel(info, NDECCCTRL, 0x0);
+		/* Must be cleared for Read ID command */
+		nand_writel(info, NDECCCTRL, 0x0);
 	}
 
 	ndcr &= ~NDCR_DMA_EN;
@@ -1216,6 +1226,10 @@
 	info->reg_ndcr |= (chip->page_shift == 6) ? NDCR_PG_PER_BLK : 0;
 	info->reg_ndcr |= (mtd->writesize == 2048) ? NDCR_PAGE_SZ : 0;
 
+#ifdef CONFIG_SYS_NAND_KEEP_CONFIG
+	nand_writel(info, NDCR, info->reg_ndcr);
+#endif
+
 	return 0;
 }
 
@@ -1533,6 +1547,14 @@
 	pdata->enable_arbiter = 1;
 	pdata->num_cs = 1;
 
+#ifdef CONFIG_SYS_NAND_ECC_STRENGTH
+	pdata->ecc_strength = CONFIG_SYS_NAND_ECC_STRENGTH;
+#endif
+
+#ifdef CONFIG_SYS_NAND_ECC_STEP_SIZE
+	pdata->ecc_step_size = CONFIG_SYS_NAND_ECC_STEP_SIZE;
+#endif
+
 	info->pdata = pdata;
 
 	return 0;
diff -aurN u-boot-2016.05.orig/drivers/net/mvneta.c u-boot-2016.05-88fxxxx/drivers/net/mvneta.c
--- u-boot-2016.05.orig/drivers/net/mvneta.c	2016-05-16 07:40:32.000000000 -0700
+++ u-boot-2016.05-88fxxxx/drivers/net/mvneta.c	2016-05-21 11:33:32.848226195 -0700
@@ -1379,6 +1379,11 @@
 	u32 smi_reg;
 	u32 timeout;
 
+	/* Phyadr read requet */
+	if (addr == 0xEE && reg == 0xEE) {
+		return pp->phyaddr;
+	}
+
 	/* check parameters */
 	if (addr > MVNETA_PHY_ADDR_MASK) {
 		printf("Error: Invalid PHY address %d\n", addr);
@@ -1641,11 +1646,22 @@
 	mvneta_port_disable(pp);
 }
 
+static int mvneta_write_hwaddr(struct udevice *dev)
+{
+	struct mvneta_port *pp = dev_get_priv(dev);
+	struct eth_pdata *pdata = dev_get_platdata(dev);
+
+	mvneta_mac_addr_set(pp, pdata->enetaddr, rxq_def);
+
+	return 0;
+}
+
 static const struct eth_ops mvneta_ops = {
 	.start		= mvneta_start,
 	.send		= mvneta_send,
 	.recv		= mvneta_recv,
 	.stop		= mvneta_stop,
+	.write_hwaddr	= mvneta_write_hwaddr,
 };
 
 static int mvneta_ofdata_to_platdata(struct udevice *dev)
diff -aurN u-boot-2016.05.orig/drivers/pci/pci_mvebu.c u-boot-2016.05-88fxxxx/drivers/pci/pci_mvebu.c
--- u-boot-2016.05.orig/drivers/pci/pci_mvebu.c	2016-05-16 07:40:32.000000000 -0700
+++ u-boot-2016.05-88fxxxx/drivers/pci/pci_mvebu.c	2016-05-21 11:33:32.848226195 -0700
@@ -88,7 +88,36 @@
 static void __iomem *mvebu_pcie_membase = (void __iomem *)MBUS_PCI_MEM_BASE;
 #define PCIE_MEM_SIZE	(32 << 20)
 
-#if defined(CONFIG_ARMADA_38X)
+#if defined(CONFIG_ARMADA_370)
+#define PCIE_BASE(if)					\
+	(MVEBU_REG_PCIE_BASE + 0x40000 * (if))
+
+/*
+ * On A370 MV6710 these PEX ports are supported:
+ *  0 - Port 0.0
+ *  1 - Port 0.1
+ */
+#ifdef CONFIG_MVEBU_PCI_MAX
+#define MAX_PEX	CONFIG_MVEBU_PCI_MAX
+#else
+#define MAX_PEX 2
+#endif
+static struct mvebu_pcie pcie_bus[MAX_PEX];
+
+static void mvebu_get_port_lane(struct mvebu_pcie *pcie, int pex_idx,
+				int *mem_target, int *mem_attr)
+{
+	u8 port[] = { 0, 0 };
+	u8 lane[] = { 1, 0 };
+	u8 target[] = { 4, 8 };
+	u8 attr[] = { 0xe8, 0xe8 };
+
+	pcie->port = port[pex_idx];
+	pcie->lane = lane[pex_idx];
+	*mem_target = target[pex_idx];
+	*mem_attr = attr[pex_idx];
+}
+#elif defined(CONFIG_ARMADA_38X)
 #define PCIE_BASE(if)					\
 	((if) == 0 ?					\
 	 MVEBU_REG_PCIE_BASE + 0x40000 :		\
diff -aurN u-boot-2016.05.orig/drivers/spi/kirkwood_spi.c u-boot-2016.05-88fxxxx/drivers/spi/kirkwood_spi.c
--- u-boot-2016.05.orig/drivers/spi/kirkwood_spi.c	2016-05-16 07:40:32.000000000 -0700
+++ u-boot-2016.05-88fxxxx/drivers/spi/kirkwood_spi.c	2016-05-21 11:33:32.848226195 -0700
@@ -329,6 +329,7 @@
 };
 
 static const struct udevice_id mvebu_spi_ids[] = {
+	{ .compatible = "marvell,armada-370-spi" },
 	{ .compatible = "marvell,armada-375-spi" },
 	{ .compatible = "marvell,armada-380-spi" },
 	{ .compatible = "marvell,armada-xp-spi" },
diff -aurN u-boot-2016.05.orig/drivers/usb/host/ehci-marvell.c u-boot-2016.05-88fxxxx/drivers/usb/host/ehci-marvell.c
--- u-boot-2016.05.orig/drivers/usb/host/ehci-marvell.c	2016-05-16 07:40:32.000000000 -0700
+++ u-boot-2016.05-88fxxxx/drivers/usb/host/ehci-marvell.c	2016-05-21 11:33:32.848226195 -0700
@@ -123,7 +123,11 @@
 };
 
 #else
+#ifdef MVUSB1_BASE
+#define MVUSB_BASE(port)	((port) ? MVUSB1_BASE : MVUSB0_BASE)
+#else
 #define MVUSB_BASE(port)	MVUSB0_BASE
+#endif
 
 static void usb_brg_adrdec_setup(int index)
 {
@@ -157,12 +161,12 @@
 		if ((size) && (attrib))
 			writel(MVCPU_WIN_CTRL_DATA(size, USB_TARGET_DRAM,
 						   attrib, MVCPU_WIN_ENABLE),
-				MVUSB0_BASE + USB_WINDOW_CTRL(i));
+				MVUSB_BASE(index) + USB_WINDOW_CTRL(i));
 		else
 			writel(MVCPU_WIN_DISABLE,
-			       MVUSB0_BASE + USB_WINDOW_CTRL(i));
+			       MVUSB_BASE(index) + USB_WINDOW_CTRL(i));
 
-		writel(base, MVUSB0_BASE + USB_WINDOW_BASE(i));
+		writel(base, MVUSB_BASE(index) + USB_WINDOW_BASE(i));
 	}
 }
 
diff -aurN u-boot-2016.05.orig/include/configs/ls400.h u-boot-2016.05-88fxxxx/include/configs/ls400.h
--- u-boot-2016.05.orig/include/configs/ls400.h	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/include/configs/ls400.h	2016-05-21 11:33:32.848226195 -0700
@@ -0,0 +1,338 @@
+/*
+ * (C) Copyright 2014, 2016
+ * Steve Shih <shihsung@gmail.com>
+ *
+ * SPDX-License-Identifier:	GPL-2.0+
+ */
+
+#ifndef _CONFIG_LS400_H
+#define _CONFIG_LS400_H
+
+/*
+ * Board specific configuration
+ */
+#ifdef CONFIG_TARGET_LS420
+#define CONFIG_IDENT_STRING		" LinkStation 420 Series NAS"
+#define CONFIG_CMD_IDE
+#define CONFIG_MVEBU_PCI_MAX		1
+#elif defined(CONFIG_TARGET_LS440)
+#define CONFIG_IDENT_STRING		" LinkStation 440 Series NAS"
+#define CONFIG_CMD_SCSI
+#endif
+
+/*
+ * Board specific MPP configuration
+ */
+#ifdef CONFIG_TARGET_LS420
+#define LS400_MPP_0_7			0x00011111
+#define LS400_MPP_8_15			0x00000000
+#define LS400_MPP_16_23			0x22222110
+#define LS400_MPP_24_31			0x02222222
+#define LS400_MPP_32_39			0x11111111
+#define LS400_MPP_40_47			0x31111111
+#define LS400_MPP_48_55			0x00033333
+#define LS400_MPP_56_63			0x10040005
+#define LS400_MPP_64_65			0x00000011
+#elif defined(CONFIG_TARGET_LS440)
+#define LS400_MPP_0_7			0x00011111
+#define LS400_MPP_8_15			0x00000000
+#define LS400_MPP_16_23			0x22222110
+#define LS400_MPP_24_31			0x02222222
+#define LS400_MPP_32_39			0x11111111
+#define LS400_MPP_40_47			0x01111111
+#define LS400_MPP_48_55			0x00500000
+#define LS400_MPP_56_63			0x10000005
+#define LS400_MPP_64_65			0x00000011
+#endif
+
+/*
+ * Board specific GPIO configuration
+ */
+#ifdef CONFIG_TARGET_LS420
+#define LS400_GPIO_LO_OUT_KEEP		0x00000300	/* keep HD power */
+#define LS400_GPIO_LO_OUT_VAL		0x00002020
+#define LS400_GPIO_LO_OUT_ENA		0xffff9c5f
+#define LS400_GPIO_LO_BLINK_ENA		0x00000000
+#define LS400_GPIO_LO_IN_POL		0x00000000
+#define LS400_GPIO_LO_BLINK_SEL		0x00000000
+#define LS400_GPIO_MID_OUT_VAL		0x00400000
+#define LS400_GPIO_MID_OUT_ENA		0x913fffff
+#define LS400_GPIO_MID_BLINK_ENA	0x00400000
+#define LS400_GPIO_MID_IN_POL		0x00000000
+#define LS400_GPIO_MID_BLINK_SEL	0x00000000
+#define LS400_GPIO_HI_OUT_VAL		0x00000000
+#define LS400_GPIO_HI_OUT_ENA		0x00000003
+#define LS400_GPIO_HI_BLINK_ENA		0x00000000
+#define LS400_GPIO_HI_IN_POL		0x00000000
+#define LS400_GPIO_HI_BLINK_SEL		0x00000000
+#elif defined(CONFIG_TARGET_LS440)
+#define LS400_GPIO_LO_OUT_KEEP		0x00000180	/* keep HD power */
+#define LS400_GPIO_LO_OUT_VAL		0x00002020
+#define LS400_GPIO_LO_OUT_ENA		0xffff9c5f
+#define LS400_GPIO_LO_BLINK_ENA		0x00000020
+#define LS400_GPIO_LO_IN_POL		0x80006000
+#define LS400_GPIO_LO_BLINK_SEL		0x00000000
+#define LS400_GPIO_MID_OUT_KEEP		0x00120000	/* keep HD power */
+#define LS400_GPIO_MID_OUT_VAL		0x00000000
+#define LS400_GPIO_MID_OUT_ENA		0x812c7fff
+#define LS400_GPIO_MID_BLINK_ENA	0x00000000
+#define LS400_GPIO_MID_IN_POL		0x00000000
+#define LS400_GPIO_MID_BLINK_SEL	0x00000000
+#define LS400_GPIO_HI_OUT_VAL		0x00000000
+#define LS400_GPIO_HI_OUT_ENA		0x00000002
+#define LS400_GPIO_HI_BLINK_ENA		0x00000000
+#define LS400_GPIO_HI_IN_POL		0x00000000
+#define LS400_GPIO_HI_BLINK_SEL		0x00000000
+#endif
+
+/*
+ * SoC configuration
+ */
+#define CONFIG_SYS_L2CLK		600000000
+#define CONFIG_SYS_TCLK			200000000
+
+/*
+ * Miscellaneous SoC configuration
+ */
+#ifdef CONFIG_TARGET_LS420
+#define LS400_PM_CLK_GATING_CTRL	0xf1ccc23a
+#elif defined(CONFIG_TARGET_LS440)
+#define LS400_PM_CLK_GATING_CTRL	0x91cc023e
+#endif
+
+/*
+ * Memory configuration
+ */
+#define CONFIG_SYS_TEXT_BASE		0x00080000
+
+/*
+ * Init configuration
+ */
+#define CONFIG_BOARD_LATE_INIT
+#define CONFIG_LAST_STAGE_INIT	
+
+/*
+ * Console configuration
+ */
+#define CONFIG_SYS_CONSOLE_IS_IN_ENV
+#define CONFIG_SYS_CONSOLE_OVERWRITE_ROUTINE
+
+/*
+ * Flash configuration
+ */
+#define CONFIG_SYS_NO_FLASH
+
+/*
+ * Environment configuration
+ */
+#define CONFIG_ENV_IS_IN_SPI_FLASH
+#define CONFIG_ENV_OFFSET		0xf0000
+#define CONFIG_ENV_SIZE			CONFIG_ENV_SECT_SIZE
+#define CONFIG_ENV_SECT_SIZE		0x10000
+
+/*
+ * Auto-boot configuration
+ */
+#define CONFIG_ZERO_BOOTDELAY_CHECK
+
+/*
+ * Linux boot configuration
+ */
+#define CONFIG_SYS_PARAMS_PHYS		0x00000100
+
+/*
+ * Command configuration
+ */
+#define CONFIG_CMD_DATE
+#define CONFIG_CMD_GREPENV
+#define CONFIG_CMD_MEMINFO
+#define CONFIG_CMD_NAND
+#define CONFIG_CMD_PCI
+#define CONFIG_CMD_READ
+#define CONFIG_CMD_SF
+#define CONFIG_CMD_TFTPPUT
+#define CONFIG_CMD_TIME
+#define CONFIG_CMD_TIMER
+#define CONFIG_CMD_USB
+
+/*
+ * CMD_DATE configuration
+ */
+#ifdef CONFIG_CMD_DATE
+#define CONFIG_RTC_RS5C372A
+#define CONFIG_CMD_I2C
+#endif
+
+/*
+ * CMD_I2C configuration
+ */
+#ifdef CONFIG_CMD_I2C
+#define CONFIG_SYS_I2C
+#define CONFIG_SYS_I2C_MVTWSI
+#define CONFIG_I2C_MVTWSI_BASE0		MVEBU_TWSI_BASE
+#endif
+
+/*
+ * CMD_IDE configuration
+ */
+#ifdef CONFIG_CMD_IDE
+#define CONFIG_MVSATA_IDE
+#define CONFIG_SYS_ATA_BASE_ADDR	MVEBU_AXP_SATA_BASE
+#define CONFIG_SYS_ATA_IDE0_OFFSET	0x2000
+#define CONFIG_SYS_ATA_IDE1_OFFSET	0x4000
+#define CONFIG_SYS_ATA_DATA_OFFSET	0x0100
+#define CONFIG_SYS_ATA_REG_OFFSET	0x0100
+#define CONFIG_SYS_ATA_ALT_OFFSET	0x0100
+#define CONFIG_SYS_ATA_STRIDE		4
+#define __io
+#define CONFIG_IDE_SWAP_IO
+#define CONFIG_SYS_IDE_MAXBUS		2
+#define CONFIG_SYS_IDE_MAXDEVICE	2
+#define ATA_DEVICE(x)			0
+#define CONFIG_IDE_PREINIT
+#define CONFIG_LBA48
+#define CONFIG_SYS_64BIT_LBA
+#endif
+
+/*
+ * CMD_NAND configuration
+ */
+#ifdef CONFIG_CMD_NAND
+#define CONFIG_NAND_PXA3XX
+#define CONFIG_SYS_NAND_SELF_INIT
+#define CONFIG_SYS_MVEBU_NAND_CLOCK	250000000	/* 250 MHz */
+#define CONFIG_SYS_NAND_USE_FLASH_BBT
+#define CONFIG_SYS_NAND_KEEP_CONFIG
+#define CONFIG_SYS_NAND_ECC_STRENGTH	4
+#define CONFIG_SYS_NAND_ECC_STEP_SIZE	512
+#define CONFIG_SYS_NAND_ONFI_DETECTION
+#define CONFIG_MTD_DEVICE
+#define CONFIG_MTD_PARTITIONS
+#define CONFIG_CMD_MTDPARTS
+#define MTDIDS_DEFAULT			"nand0=armada370-nand"
+#define MTDPARTS_DEFAULT		"mtdparts=armada370-nand:-(UBIFS)"
+#define CONFIG_CMD_UBI
+#define CONFIG_CMD_UBIFS
+#define CONFIG_RBTREE
+#define CONFIG_LZO
+#endif
+
+/*
+ * CMD_NET configuration
+ */
+#ifdef CONFIG_CMD_NET
+#define CONFIG_RESET_PHY_R
+#define CONFIG_SYS_FAULT_ECHO_LINK_DOWN
+#define PHY_ANEG_TIMEOUT		15000
+#define CONFIG_ETHADDR			02:08:8f:67:10:a1
+#define CONFIG_NETCONSOLE
+#define CONFIG_CMD_PING
+#define CONFIG_IPADDR			192.168.11.150
+#define CONFIG_NETMASK			255.255.255.0
+#define CONFIG_SERVERIP			192.168.11.1
+#define CONFIG_EXTRA_ENV_SETTINGS	"stdin=nc\0" \
+					"stdout=nc\0" \
+					"stderr=nc"
+#endif
+
+/*
+ * CMD_PCI configuration
+ */
+#ifdef CONFIG_CMD_PCI
+#define CONFIG_PCI_MVEBU
+#define CONFIG_PCI
+#define CONFIG_PCI_PNP
+#define CONFIG_SCAN_SHOW
+#endif
+
+/*
+ * CMD_SCSI configuration
+ */
+#ifdef CONFIG_CMD_SCSI
+#define CONFIG_LIBATA
+#define CONFIG_SCSI_AHCI
+#define CONFIG_SCSI_DEV_LIST		{ 0x1b4b, 0x9215 }
+#define CONFIG_SYS_SCSI_MAX_SCSI_ID	4
+#define CONFIG_SYS_SCSI_MAX_LUN		1
+#define CONFIG_SYS_SCSI_MAX_DEVICE	(CONFIG_SYS_SCSI_MAX_SCSI_ID * \
+					 CONFIG_SYS_SCSI_MAX_LUN)
+#define CONFIG_LBA48
+#define CONFIG_SYS_64BIT_LBA
+#endif
+
+/*
+ * CMD_SF configuration
+ */
+#ifdef CONFIG_CMD_SF
+#define CONFIG_CMD_SPI
+/* Boot time probe */
+#define CONFIG_ENV_SPI_BUS		0
+#define CONFIG_ENV_SPI_CS		0
+#define CONFIG_ENV_SPI_MODE		SPI_MODE_3
+#define CONFIG_ENV_SPI_MAX_HZ		80000000 /* 86 MHz MAX for MX25L8006E */
+/* Post boot time probe */
+#define CONFIG_SF_DEFAULT_BUS		CONFIG_ENV_SPI_BUS
+#define CONFIG_SF_DEFAULT_CS		CONFIG_ENV_SPI_CS
+#define CONFIG_SF_DEFAULT_MODE		CONFIG_ENV_SPI_MODE
+#define CONFIG_SF_DEFAULT_SPEED		CONFIG_ENV_SPI_MAX_HZ
+#else
+#define CONFIG_ENV_IS_NOWHERE
+#endif
+
+/*
+ * CMD_USB configuration
+ */
+#ifdef CONFIG_CMD_USB
+#define CONFIG_DM_USB
+#define CONFIG_USB_EHCI_MARVELL
+#define CONFIG_USB_EHCI
+#define CONFIG_EHCI_IS_TDI
+#define CONFIG_USB_STORAGE
+#endif
+
+/*
+ * Filesystem configuration
+ */
+#if defined(CONFIG_CMD_IDE) || defined(CONFIG_CMD_SCSI) || defined(CONFIG_CMD_USB)
+#define CONFIG_DOS_PARTITION
+#define CONFIG_EFI_PARTITION
+#define CONFIG_CMD_FAT
+#define CONFIG_FAT_WRITE
+#define CONFIG_CMD_EXT2
+#define CONFIG_CMD_EXT4
+#define CONFIG_CMD_EXT4_WRITE
+#define CONFIG_CMD_FS_GENERIC
+#endif
+
+#include "mv-common.h"
+
+/*
+ * SPL configuration
+ */
+#ifdef CONFIG_SPL_BUILD
+#define CONFIG_SPL_FRAMEWORK
+#define CONFIG_SPL_TEXT_BASE		0x40004030
+#define CONFIG_SPL_MAX_SIZE		((128 << 10) - 0x4030)
+
+#define CONFIG_SPL_BSS_START_ADDR	(0x40000000 + (128 << 10))
+#define CONFIG_SPL_BSS_MAX_SIZE		(16 << 10)
+
+#define CONFIG_SPL_STACK		(0x40000000 + ((192 - 16) << 10))
+#define CONFIG_SPL_BOOTROM_SAVE		(CONFIG_SPL_STACK + 4)
+
+#define CONFIG_SPL_LIBCOMMON_SUPPORT
+#define CONFIG_SPL_LIBGENERIC_SUPPORT
+#define CONFIG_SPL_SERIAL_SUPPORT
+
+/* SPL related SPI defines */
+#define CONFIG_SPL_SPI_SUPPORT
+#define CONFIG_SPL_SPI_FLASH_SUPPORT
+#define CONFIG_SPL_SPI_LOAD
+
+#define CONFIG_DDR_FIXED_SIZE		0x20000000	/* 512 MB */
+#endif
+
+#define CONFIG_SYS_SPI_U_BOOT_OFFS	0x20000
+#define CONFIG_SYS_U_BOOT_OFFS		CONFIG_SYS_SPI_U_BOOT_OFFS
+
+#endif
diff -aurN u-boot-2016.05.orig/include/configs/lsgl.h u-boot-2016.05-88fxxxx/include/configs/lsgl.h
--- u-boot-2016.05.orig/include/configs/lsgl.h	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/include/configs/lsgl.h	2016-05-21 11:33:32.852226195 -0700
@@ -0,0 +1,313 @@
+/*
+ * (C) Copyright 2012, 2016
+ * Steve Shih <shihsung@gmail.com>
+ *
+ * SPDX-License-Identifier:	GPL-2.0+
+ */
+
+#ifndef _CONFIG_LSGL_H
+#define _CONFIG_LSGL_H
+
+/*
+ * Boot type
+ */
+#ifdef CONFIG_SYS_RAMBOOT
+#define IDENT_SUFFIX			" (RAMBOOT)"
+#else
+#define IDENT_SUFFIX			""
+#endif
+
+/*
+ * Board specific information
+ */
+#ifdef CONFIG_TARGET_KUROPRO
+#define CONFIG_IDENT_STRING		" Kurobox-Pro" IDENT_SUFFIX
+#define CONFIG_MACH_KUROBOX_PRO
+#define CONFIG_MACH_TYPE		MACH_TYPE_KUROBOX_PRO
+#define CONFIG_CMD_NAND
+#elif defined(CONFIG_TARGET_LSGL)
+#define CONFIG_IDENT_STRING		" Linkstation Pro/Live" IDENT_SUFFIX
+#define CONFIG_MACH_LINKSTATION_PRO
+#define CONFIG_MACH_TYPE		MACH_TYPE_LINKSTATION_PRO
+#endif
+
+/*
+ * Board-specific values for Orion5x MPP low level init:
+ * - MPPs 2 and 3 are GPIO inputs from Micon and RTC
+ * - MPPs 6 and 7 are NAND REn and WEn
+ * - MPPs 12 to 15 are SATA presence and LEDs (mode 5)
+ * - MPPs 16 to 19 are UART1 RXD, TXD, CTS, and RTS with Micon
+ */
+#define ORION5X_MPP0_7			0x44000003
+#define ORION5X_MPP8_15			0x55550000
+#define ORION5X_MPP16_23		0x00000000
+
+/*
+ * Board-specific values for Orion5x GPIO low level init:
+ * - GPIO2 is input Micon interrupt
+ * - GPIO3 is input RTC interrupt
+ */
+#define ORION5X_GPIO_OUT_VALUE		0x00000000
+#define ORION5X_GPIO_OUT_ENABLE		0x0000000c
+#define ORION5X_GPIO_IN_POLARITY	0x00000000
+
+/*
+ * SoC configuration
+ */
+#define CONFIG_FEROCEON
+#define CONFIG_88F5182
+
+/*
+ * Miscellaneous SOC configuration
+ */
+#define CONFIG_DISPLAY_CPUINFO
+
+/*
+ * Memory configuration
+ */
+#define CONFIG_SYS_DCACHE_OFF
+#define CONFIG_SYS_DDR2
+#define CONFIG_NR_DRAM_BANKS		1
+#define CONFIG_MAX_RAM_BANK_SIZE	(128 * 1024 * 1024)
+#define CONFIG_SYS_INIT_SP_ADDR		(0x1000 - GENERATED_GBL_DATA_SIZE)
+#define CONFIG_SYS_MALLOC_LEN		0x00800000
+#define CONFIG_SYS_LOAD_ADDR		0x00080000
+#if defined(CONFIG_SYS_RAMBOOT)
+#define CONFIG_SYS_TEXT_BASE		0x00080000
+#else
+#define CONFIG_SYS_TEXT_BASE		0xfffc0000
+#endif
+
+/*
+ * Init configurations
+ */
+#define CONFIG_ARCH_CPU_INIT
+#define CONFIG_BOARD_EARLY_INIT_F
+#define CONFIG_ARCH_MISC_INIT
+#define CONFIG_BOARD_LATE_INIT
+#define CONFIG_LAST_STAGE_INIT
+
+/*
+ * Console configuration
+ */
+#define CONFIG_SYS_NS16550_SERIAL
+#define CONFIG_SYS_NS16550_REG_SIZE	(-4)
+#define CONFIG_SYS_NS16550_CLK		CONFIG_SYS_TCLK
+#define CONFIG_SYS_NS16550_COM1		ORION5X_UART0_BASE
+#define CONFIG_SYS_CONSOLE_IS_IN_ENV
+#define CONFIG_SYS_CONSOLE_OVERWRITE_ROUTINE
+#define CONFIG_CONS_INDEX		1
+#define CONFIG_BAUDRATE			115200
+#define	CONFIG_SYS_CBSIZE		1024
+#define	CONFIG_SYS_PBSIZE		2048
+#define CONFIG_SYS_MAXARGS		36
+
+#if defined(CONFIG_SYS_RAMBOOT)
+#define CONFIG_SYS_LONGHELP
+#define CONFIG_CMDLINE_EDITING
+#define CONFIG_AUTO_COMPLETE
+#endif
+
+/*
+ * Flash configuration
+ */
+#define CONFIG_SYS_FLASH_CFI
+#define CONFIG_FLASH_CFI_DRIVER
+#define CONFIG_FLASH_CFI_LEGACY
+#define CONFIG_SYS_MAX_FLASH_BANKS	1
+#define CONFIG_SYS_MAX_FLASH_SECT	64
+#define CONFIG_SYS_FLASH_BASE		0xfffc0000
+#define CONFIG_SYS_FLASH_EMPTY_INFO
+#define CONFIG_FLASH_SHOW_PROGRESS	45
+
+/*
+ * Environment configuration
+ */
+#ifdef CONFIG_ENV_IS_IN_NAND
+#define CONFIG_ENV_OFFSET		0xc0000
+#define CONFIG_ENV_SIZE			0x20000
+#else
+#define CONFIG_ENV_IS_IN_FLASH
+#define CONFIG_ENV_OFFSET		0x3f000
+#define CONFIG_ENV_SIZE			0x1000
+#endif
+
+/*
+ * Auto-boot configuration
+ */
+#define CONFIG_BOOTDELAY		3
+#define CONFIG_ZERO_BOOTDELAY_CHECK
+
+/*
+ * Linux boot configuration
+ */
+#define CONFIG_SYS_PARAMS_PHYS		0x00000100
+#define CONFIG_CMDLINE_TAG
+#define CONFIG_INITRD_TAG
+#define CONFIG_SETUP_MEMORY_TAGS
+
+#if defined(CONFIG_SYS_RAMBOOT) || !defined(CONFIG_TARGET_KUROPRO)
+#define CONFIG_OF_LIBFDT
+#define CONFIG_CMD_FDT
+#endif
+
+/*
+ * Command configuration
+ */
+#define CONFIG_CMD_DATE
+#define CONFIG_CMD_IDE
+#define CONFIG_CMD_TFTPPUT
+#define CONFIG_CMD_USB
+
+#if defined(CONFIG_SYS_RAMBOOT)
+#define CONFIG_CMD_GREPENV
+#define CONFIG_CMD_MEMINFO
+#define CONFIG_CMD_READ
+#define CONFIG_CMD_TIME
+#define CONFIG_CMD_TIMER
+#endif
+
+/*
+ * CMD_DATE configuration
+ */
+#ifdef CONFIG_CMD_DATE
+#define CONFIG_RTC_RS5C372A
+#define CONFIG_SYS_I2C
+#define CONFIG_SYS_I2C_MVTWSI
+#define CONFIG_I2C_MVTWSI_BASE0		ORION5X_TWSI_BASE
+#define CONFIG_SYS_I2C_SLAVE		0x0
+#define CONFIG_SYS_I2C_SPEED		100000
+
+#if defined(CONFIG_SYS_RAMBOOT)
+#define CONFIG_CMD_I2C
+#endif
+
+#endif
+
+/*
+ * CMD_IDE configuration
+ */
+#ifdef CONFIG_CMD_IDE
+#define CONFIG_MVSATA_IDE
+#define CONFIG_SYS_ATA_BASE_ADDR	ORION5X_SATA_BASE
+/* Port 0 for Linkstation v2 and Kurobox-Pro; Port 1 for Linkstation v1 */
+#define CONFIG_SYS_ATA_IDE0_OFFSET	ORION5X_SATA_PORT0_OFFSET
+#define CONFIG_SYS_ATA_IDE1_OFFSET	ORION5X_SATA_PORT1_OFFSET
+#define CONFIG_SYS_ATA_DATA_OFFSET	0x0100
+#define CONFIG_SYS_ATA_REG_OFFSET	0x0100
+#define CONFIG_SYS_ATA_ALT_OFFSET	0x0100
+#define CONFIG_SYS_ATA_STRIDE		4
+#define __io
+#define CONFIG_IDE_SWAP_IO
+#define CONFIG_SYS_IDE_MAXBUS		1
+#define CONFIG_SYS_IDE_MAXDEVICE	1
+#define CONFIG_IDE_PREINIT
+#define CONFIG_LBA48
+#define CONFIG_SYS_64BIT_LBA
+#endif
+
+/*
+ * CMD_NAND configuration
+ */
+#ifdef CONFIG_CMD_NAND
+#define CONFIG_NAND_ORION
+#define CONFIG_SYS_NAND_SELF_INIT
+#define CONFIG_SYS_NAND_BASE		0xfa000000
+#define CONFIG_SYS_MAX_NAND_DEVICE	1
+
+#if defined(CONFIG_SYS_RAMBOOT) || !defined(CONFIG_TARGET_KUROPRO)
+#define CONFIG_MTD_DEVICE
+#define CONFIG_MTD_PARTITIONS
+#define CONFIG_CMD_MTDPARTS
+#define MTDIDS_DEFAULT			"nand0=orion-nand"
+#define MTDPARTS_DEFAULT		"mtdparts=orion-nand:" \
+		"0xc0000(U-Boot),0x40000@0xc0000(ENV),-@0x100000(UBIFS)"
+#define CONFIG_CMD_UBI
+#define CONFIG_CMD_UBIFS
+#define CONFIG_RBTREE
+#define CONFIG_LZO
+#endif
+
+#endif
+
+/*
+ * CMD_NET configuration
+ */
+#ifdef CONFIG_CMD_NET
+#define CONFIG_MVGBE
+#define CONFIG_MVGBE_PORTS		{1}
+#define CONFIG_PHY_BASE_ADR		8
+#define CONFIG_CMD_MII
+#define CONFIG_SYS_FAULT_ECHO_LINK_DOWN
+#define CONFIG_PHY_GIGE
+#define CONFIG_ENV_OVERWRITE
+#define CONFIG_ETHADDR			02:08:8f:51:82:a2
+#define CONFIG_NETCONSOLE
+#define CONFIG_CMD_PING
+#define CONFIG_IPADDR			192.168.11.150
+#define CONFIG_NETMASK			255.255.255.0
+#define CONFIG_SERVERIP			192.168.11.1
+#define CONFIG_EXTRA_ENV_SETTINGS	"stdin=nc\0" \
+					"stdout=nc\0" \
+					"stderr=nc"
+#endif
+
+/*
+ * CMD_USB configuration
+ */
+#ifdef CONFIG_CMD_USB
+#define CONFIG_USB_EHCI_MARVELL
+#define CONFIG_USB_EHCI
+#define CONFIG_EHCI_IS_TDI
+#define CONFIG_USB_MAX_CONTROLLER_COUNT	2
+#define CONFIG_USB_STORAGE
+#endif
+
+/*
+ * Filesystem configuration
+ */
+#if defined(CONFIG_CMD_IDE) || defined(CONFIG_CMD_USB)
+#define CONFIG_DOS_PARTITION
+#define CONFIG_CMD_FAT
+#define CONFIG_CMD_EXT2
+#define CONFIG_CMD_EXT4
+#define CONFIG_CMD_FS_GENERIC
+
+#if defined(CONFIG_SYS_RAMBOOT) || !defined(CONFIG_TARGET_KUROPRO)
+#define CONFIG_FAT_WRITE
+#define CONFIG_CMD_EXT4_WRITE
+#endif
+
+#endif
+
+/*
+ * Reduce ROMBOOT image size
+ */
+#ifdef CONFIG_SYS_ROMBOOT
+/* undef from config_defaults.h */
+#undef CONFIG_BOOTM_NETBSD
+#undef CONFIG_BOOTM_PLAN9
+#undef CONFIG_BOOTM_RTEMS
+#undef CONFIG_BOOTM_VXWORKS
+/* undef from config_cmd_defaults.h */
+#undef CONFIG_CMD_CRC32
+#undef CONFIG_CMD_EXPORTENV
+#undef CONFIG_CMD_IMPORTENV
+/* undef from config_cmd_default.h */
+#undef CONFIG_CMD_BDI
+#undef CONFIG_CMD_ECHO
+#undef CONFIG_CMD_FPGA
+#undef CONFIG_CMD_ITEST
+#undef CONFIG_CMD_IMLS
+#undef CONFIG_CMD_LOADB
+#undef CONFIG_CMD_LOADS
+#undef CONFIG_CMD_MISC
+#undef CONFIG_CMD_NFS
+#undef CONFIG_CMD_SETGETDCR
+#undef CONFIG_CMD_SOURCE
+#undef CONFIG_CMD_XIMG
+#endif
+
+#include <asm/arch/orion5x.h>
+
+#endif /* _CONFIG_LSGL_H */
diff -aurN u-boot-2016.05.orig/include/configs/pogo.h u-boot-2016.05-88fxxxx/include/configs/pogo.h
--- u-boot-2016.05.orig/include/configs/pogo.h	1969-12-31 16:00:00.000000000 -0800
+++ u-boot-2016.05-88fxxxx/include/configs/pogo.h	2016-05-21 11:33:32.852226195 -0700
@@ -0,0 +1,268 @@
+/*
+ * (C) Copyright 2013, 2016
+ * Steve Shih <shihsung@gmail.com>
+ *
+ * SPDX-License-Identifier:	GPL-2.0+
+ */
+
+#ifndef _CONFIG_POGO_H
+#define _CONFIG_POGO_H
+
+/*
+ * Board specific configuration
+ */
+#define CONFIG_BUILD_TARGET		"u-boot.kwb"
+
+#ifdef CONFIG_POGO1
+#define CONFIG_SYS_KWD_CONFIG		$(CONFIG_BOARDDIR)/kwbimage-pogo1.cfg
+#define CONFIG_IDENT_BOARD		" POGO-E01"
+#define CONFIG_MACH_SHEEVAPLUG
+#define CONFIG_MACH_TYPE		MACH_TYPE_SHEEVAPLUG
+#define CONFIG_KW88F6281
+#define CONFIG_ETHADDR			02:08:8f:62:81:a0
+#elif defined(CONFIG_POGO2)
+#define CONFIG_SYS_KWD_CONFIG		$(CONFIG_BOARDDIR)/kwbimage-pogo1.cfg
+#define CONFIG_IDENT_BOARD		" POGO-E02"
+#define CONFIG_MACH_POGO_E02
+#define CONFIG_MACH_TYPE		MACH_TYPE_POGO_E02
+#define CONFIG_KW88F6281
+#define CONFIG_ETHADDR			02:08:8f:62:81:a0
+#elif defined(CONFIG_POGO4)
+#define CONFIG_SYS_KWD_CONFIG		$(CONFIG_BOARDDIR)/kwbimage-pogo4.cfg
+#define CONFIG_IDENT_BOARD		" POGOPLUG SERIES 4"
+#define CONFIG_MACH_POGOPLUGV4
+#define CONFIG_MACH_TYPE		MACH_TYPE_POGOPLUGV4
+#define CONFIG_KW88F6192
+#define CONFIG_ETHADDR			02:08:8f:61:92:a0
+#define CONFIG_KIRKWOOD_GPIO
+#define CONFIG_POGO_EJECT_BUTTON	29
+#define CONFIG_CMD_IDE
+#define CONFIG_CMD_MMC
+#elif defined(CONFIG_POGOM)
+#define CONFIG_SYS_KWD_CONFIG		$(CONFIG_BOARDDIR)/kwbimage-pogo4.cfg
+#define CONFIG_IDENT_BOARD		" POGOPLUG MOBILE"
+#define CONFIG_MACH_POGOPLUGV4
+#define CONFIG_MACH_TYPE		MACH_TYPE_POGOPLUGV4
+#define CONFIG_KW88F6192
+#define CONFIG_ETHADDR			02:08:8f:61:92:a0
+#define CONFIG_KIRKWOOD_GPIO
+#define CONFIG_POGO_EJECT_BUTTON	29
+#define CONFIG_CMD_MMC
+#elif defined(CONFIG_SHEEVA)
+#define CONFIG_SYS_KWD_CONFIG		$(CONFIG_BOARDDIR)/kwbimage-sheeva.cfg
+#define CONFIG_IDENT_BOARD		" SHEEVA PLUG"
+#define CONFIG_MACH_SHEEVAPLUG
+#define CONFIG_MACH_TYPE		MACH_TYPE_SHEEVAPLUG
+#define CONFIG_KW88F6281
+#define CONFIG_ETHADDR			02:08:8f:62:81:a0
+#define CONFIG_CMD_MMC
+#endif
+
+/*
+ * Board specific Kirkwood MPP configuration
+ */
+#if defined(CONFIG_TARGET_POGO1) || defined(CONFIG_TARGET_POGO2)
+#define KIRKWOOD_MPP_0_7		0x01111111
+#define KIRKWOOD_MPP_8_15		0x11113322
+#define KIRKWOOD_MPP_16_23		0x00001111
+#define KIRKWOOD_MPP_24_31		0x00000000
+#define KIRKWOOD_MPP_32_39		0x00000000
+#define KIRKWOOD_MPP_40_47		0x00000000
+#define KIRKWOOD_MPP_48_49		0x00000000
+#elif defined(CONFIG_TARGET_POGO4) || defined(CONFIG_TARGET_POGOM)
+#define KIRKWOOD_MPP_0_7		0x21111111
+#define KIRKWOOD_MPP_8_15		0x11113311
+#define KIRKWOOD_MPP_16_23		0x00551111
+#define KIRKWOOD_MPP_24_31		0x00000000
+#define KIRKWOOD_MPP_32_39		0x00000000
+#define KIRKWOOD_MPP_40_47		0x00000000
+#define KIRKWOOD_MPP_48_49		0x00000000
+#elif defined(CONFIG_TARGET_SHEEVA)
+#define KIRKWOOD_MPP_0_7		0x01111111
+#define KIRKWOOD_MPP_8_15		0x11113322
+#define KIRKWOOD_MPP_16_23		0x00001111
+#define KIRKWOOD_MPP_24_31		0x00000000
+#define KIRKWOOD_MPP_32_39		0x00000000
+#define KIRKWOOD_MPP_40_47		0x00000000
+#define KIRKWOOD_MPP_48_49		0x00000000
+#endif
+
+/*
+ * Board specific Kirkwood GPIO configuraton
+ */
+#if defined(CONFIG_TARGET_POGO1)
+#define KIRKWOOD_GPIO_LO_OUT_VAL	0xeff00000
+#define KIRKWOOD_GPIO_LO_OUT_ENA	0xcfffffff
+#define KIRKWOOD_GPIO_LO_BLINK_ENA	0x00000000
+#define KIRKWOOD_GPIO_HI_OUT_VAL	0xffffffff
+#define KIRKWOOD_GPIO_HI_OUT_ENA	0xfffc3fff
+#define KIRKWOOD_GPIO_HI_BLINK_ENA	0x00018000
+#elif defined(CONFIG_TARGET_POGO2)
+#define KIRKWOOD_GPIO_LO_OUT_VAL	0xeff00000
+#define KIRKWOOD_GPIO_LO_OUT_ENA	0xefffffff
+#define KIRKWOOD_GPIO_LO_BLINK_ENA	0x00000000
+#define KIRKWOOD_GPIO_HI_OUT_VAL	0xffffffff
+#define KIRKWOOD_GPIO_HI_OUT_ENA	0xfffc3fff
+#define KIRKWOOD_GPIO_HI_BLINK_ENA	0x00010000
+#elif defined(CONFIG_TARGET_POGO4) || defined(CONFIG_TARGET_POGOM)
+#define KIRKWOOD_GPIO_LO_OUT_VAL	0x0101c441
+#define KIRKWOOD_GPIO_LO_OUT_ENA	0xee004800
+#define KIRKWOOD_GPIO_LO_BLINK_ENA	0x00400000
+#define KIRKWOOD_GPIO_HI_OUT_VAL	0x00000008
+#define KIRKWOOD_GPIO_HI_OUT_ENA	0x00000005
+#define KIRKWOOD_GPIO_HI_BLINK_ENA	0x00000000
+#elif defined(CONFIG_TARGET_SHEEVA)
+#define KIRKWOOD_GPIO_LO_OUT_VAL	0x20000000
+#define KIRKWOOD_GPIO_LO_OUT_ENA	0x00000000
+#define KIRKWOOD_GPIO_LO_BLINK_ENA	0x00000000
+#define KIRKWOOD_GPIO_HI_OUT_VAL	0xfffcffff
+#define KIRKWOOD_GPIO_HI_OUT_ENA	0x00008000
+#define KIRKWOOD_GPIO_HI_BLINK_ENA	0x00020000
+#endif
+
+/*
+ * SoC configuration
+ */
+#define CONFIG_FEROCEON_88FR131
+
+/*
+ * Miscellaneous SoC configuration
+ */
+#define CONFIG_KIRKWOOD_EGIGA1_DISABLE
+
+/*
+ * Memory configuration
+ */
+#define CONFIG_SYS_TEXT_BASE		0x00080000
+
+/*
+ * Init configuration
+ */
+#define CONFIG_SKIP_LOWLEVEL_INIT
+#define CONFIG_BOARD_LATE_INIT
+#define CONFIG_LAST_STAGE_INIT
+
+/*
+ * Console configuration
+ */
+#define CONFIG_SYS_CONSOLE_IS_IN_ENV
+#define CONFIG_SYS_CONSOLE_OVERWRITE_ROUTINE
+
+/*
+ * Flash configuration
+ */
+#define CONFIG_SYS_NO_FLASH
+
+/*
+ * Environment configuration
+ */
+#define CONFIG_ENV_IS_IN_NAND		1
+#define CONFIG_ENV_OFFSET		0xc0000
+#define CONFIG_ENV_SIZE			0x20000
+
+/*
+ * Auto-boot configuration
+ */
+#define CONFIG_ZERO_BOOTDELAY_CHECK
+
+/*
+ * Linux boot configuration
+ */
+#define CONFIG_SYS_PARAMS_PHYS		0x00000100
+
+/*
+ * Command configuration
+ */
+#define CONFIG_CMD_DATE
+#define CONFIG_CMD_GREPENV
+#define CONFIG_CMD_MEMINFO
+#define CONFIG_CMD_NAND
+#define CONFIG_CMD_READ
+#define CONFIG_CMD_TFTPPUT
+#define CONFIG_CMD_TIME
+#define CONFIG_CMD_TIMER
+#define CONFIG_CMD_USB
+
+/*
+ * CMD_DATE and RTC configuration
+ */
+#ifdef CONFIG_CMD_DATE
+#define CONFIG_RTC_MV
+#endif
+
+/*
+ * CMD_IDE configuration
+ */
+#ifdef CONFIG_CMD_IDE
+#define CONFIG_SYS_ATA_IDE0_OFFSET	KW_SATA_PORT0_OFFSET
+#define CONFIG_SYS_64BIT_LBA
+#endif
+
+/*
+ * CMD_MMC configuration
+ */
+#ifdef CONFIG_CMD_MMC
+#define CONFIG_GENERIC_MMC
+#define CONFIG_MVEBU_MMC
+#define CONFIG_SYS_MMC_BASE		KW_SDIO_BASE
+#define CONFIG_MMC
+#endif
+
+/*
+ * CMD_NAND configuration
+ */
+#ifdef CONFIG_CMD_NAND
+#define CONFIG_MTD_DEVICE
+#define CONFIG_MTD_PARTITIONS
+#define CONFIG_CMD_MTDPARTS
+#define MTDIDS_DEFAULT			"nand0=orion-nand"
+#define MTDPARTS_DEFAULT		"mtdparts=orion-nand:" \
+		"0xc0000(U-Boot),0x40000@0xc0000(ENV),-@0x100000(UBIFS)"
+#define CONFIG_CMD_UBI
+#define CONFIG_CMD_UBIFS
+#define CONFIG_RBTREE
+#define CONFIG_LZO
+#else
+#define CONFIG_ENV_IS_NOWHERE
+#endif
+
+/*
+ * CMD_NET configuration
+ */
+#ifdef CONFIG_CMD_NET
+#define CONFIG_MVGBE_PORTS		{1}
+#define CONFIG_PHY_BASE_ADR		0
+#define CONFIG_SYS_FAULT_ECHO_LINK_DOWN
+#define CONFIG_PHY_GIGE
+#define CONFIG_CMD_PING
+#define CONFIG_IPADDR			192.168.11.150
+#define CONFIG_NETMASK			255.255.255.0
+#define CONFIG_SERVERIP			192.168.11.1
+#define CONFIG_EXTRA_ENV_SETTINGS	"stdin=nc\0" \
+					"stdout=nc\0" \
+					"stderr=nc"
+#endif
+
+/*
+ * CMD_USB configuration
+ */
+#ifdef CONFIG_CMD_USB
+#endif
+
+/*
+ * Filesystem configuration
+ */
+#if defined(CONFIG_CMD_IDE)||defined(CONFIG_CMD_MMC)||defined(CONFIG_CMD_USB)
+#define CONFIG_DOS_PARTITION
+#define CONFIG_EFI_PARTITION
+#define CONFIG_CMD_FAT
+#define CONFIG_FAT_WRITE
+#define CONFIG_CMD_EXT2
+#define CONFIG_CMD_EXT4
+#define CONFIG_CMD_EXT4_WRITE
+#define CONFIG_CMD_FS_GENERIC
+#endif
+
+#include "mv-common.h"
+
+#endif
diff -aurN u-boot-2016.05.orig/tools/kwboot.c u-boot-2016.05-88fxxxx/tools/kwboot.c
--- u-boot-2016.05.orig/tools/kwboot.c	2016-05-16 07:40:32.000000000 -0700
+++ u-boot-2016.05-88fxxxx/tools/kwboot.c	2016-05-21 11:33:32.852226195 -0700
@@ -661,7 +661,8 @@
 	 * This way, the execution address is identical to the
 	 * one the image is compiled for (TEXT_BASE).
 	 */
-	hdr->destaddr = hdr->destaddr - sizeof(struct image_header);
+	if (image_check_hcrc(img + hdr->srcaddr))
+		hdr->destaddr = hdr->destaddr - sizeof(struct image_header);
 
 	if (image_ver == 0) {
 		struct main_hdr_v0 *hdr_v0 = img;
diff -aurN u-boot-2016.05.orig/tools/Makefile u-boot-2016.05-88fxxxx/tools/Makefile
--- u-boot-2016.05.orig/tools/Makefile	2016-05-16 07:40:32.000000000 -0700
+++ u-boot-2016.05-88fxxxx/tools/Makefile	2016-05-21 11:33:32.852226195 -0700
@@ -162,6 +162,7 @@
 
 HOSTCFLAGS_ubsha1.o := -pedantic
 
+kwboot-objs := lib/crc32.o common/image.o kwboot.o
 hostprogs-$(CONFIG_KIRKWOOD) += kwboot
 hostprogs-$(CONFIG_ARCH_MVEBU) += kwboot
 hostprogs-y += proftool
